\\ Copyright 2018, 2019, 2020, 2022 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


\\ Some references:
\\
\\ Daniel W. Cranston, Douglas B. West, problem 11712, American Mathematical
\\ Monthly, volume 120, number 6, June-July 2013, pages 569-576.
\\ http://www.jstor.org/stable/10.4169/amer.math.monthly.120.06.569
\\
\\ Asking how many moves needed to go from single pile n to a position which
\\ is in a cycle.
\\
\\ -------
\\
\\ Brian Hopkins and James A. Sellers, "Exact Enumeration of Garden of
\\ Eden Partitions", INTEGERS: Electronic Journal of Combinatorial
\\ Number Theory, volume 7, number 2, 2007, #A19.
\\
\\ Garden of Eden partitions are those with rank <= -2.  Formula, generating
\\ function, and combinatorial correspondence.
\\
\\ -------
\\
\\ Kiyoshi Igusa, "Solution of the Bulgarian Solitaire Conjecture",
\\ Mathematics Magazine, volume 58, number 5, November 1985, pages 259-271.
\\ http://www.jstor.org/stable/2690174
\\
\\ -------
\\
\\ Ethan Akin and Morton Davis, "Bulgarian Solitaire", American Mathematical
\\ Monthly, volume 92, number 4, April 1985, pages 237-250.
\\ http://www.jstor.org/stable/2323643
\\
\\ Number of disconnected components = compositions up to rotation.
\\
\\ -------
\\
\\ Jerrold R. Griggs and Chih-Chang Ho, "The Cycling of Partitions and
\\ Compositions under Repeated Shifts", Advances In Applied Mathematics,
\\ volume 21, 1998, pages 205-227, Am980597.
\\
\\ Including rule applied to compositions.
\\
\\ -------
\\
\\ Brian Hopkins, Michael A. Jones, "Shift-Induced Dynamical Systems on
\\ Partitions and Compositions", The Electronic Journal of Combinatorics,
\\ volume 13, 2006.
\\
\\ Garden of Eden compositions = 2^(n-1) - fibonacci(n+1).
\\
\\ -------

default(recover,0)
default(strictargs,1)
read("../vpar.gp");
read("../vpar-wip.gp");
read("../test-helpers.gp");
read("memoize.gp");


\\ p is a partition, return the next partition.
Bulgarian_partition_step(p) =
{
  p=Vec(p);
  for(i=1,#p, p[i]--);
  Vecsmall(vecsort(select(x->x!=0, concat(p,[#p]))));
}

numbpart_limited(n,k) =
{
  if(n<0 || k<=0, 0,
     n==0, 1,
     n==1, k>=1,
     sum(i=1,k, numbpart_limited(n-i,i)));
}
{
  for(n=1,10,
     for(k=1,n,
        printf(" %6d", numbpart_limited(n,k)));
     print());
}
vector(10,n,numbpart(n))

numbpart_ge(n,k) =
{
  if(n<0, 0,
     n==0, 1,
     sum(i=max(k,1),n, numbpart_ge(n-i,i)));
}
check_equal(numbpart_ge(0,0), 1);
check_equal(numbpart_ge(0,1), 1);
check_equal(numbpart_ge(5,5), 1);

test_numbpart_ge_by_partitions(n,k) = \
  #select(p->p[1]>=k, partitions(n));
{
  for(n=1,5,
     for(k=1,n,
        printf(" %6d", numbpart_ge(n,k)));
     print());
}
{
  for(n=1,5,
     for(k=1,n,
        printf(" %6d", test_numbpart_ge_by_partitions(n,k)));
     print());
}
{
  for(n=1,10,
     for(k=1,n,
        check_equal(numbpart_ge(n,k),
                    test_numbpart_ge_by_partitions(n,k))));
}


numbpart_of_pieces(n,k,lo=1) =
{
  if(k<=0 || n<k || n<lo, 0,
     k==1, 1,
     sum(i=lo,n, numbpart_of_pieces(n-i,k-1,i)));
}
numbpart_of_pieces=memoize(numbpart_of_pieces);

test_numbpart_of_pieces_by_partitions(n,k) =
  #select(p->#p==k, partitions(n));
{
  print("numbpart_of_pieces");
  for(n=1,5,
     for(k=1,n,
        printf(" %6d", numbpart_of_pieces(n,k)));
     print());
}
{
  for(n=1,5,
     for(k=1,n,
        printf(" %6d", test_numbpart_of_pieces_by_partitions(n,k)));
     print());
}
{
  for(n=1,10,
     for(k=1,n,
        check_equal(numbpart_of_pieces(n,k),
                    test_numbpart_of_pieces_by_partitions(n,k))));
}

partitions(5)
select(p->p[1]>=1,partitions(5))
test_numbpart_ge_by_partitions(5,1)

part_to_n(p) =
{
  \\ print("part_to_n() "Vec(p));
  my(n=vecsum(Vec(p)),
     lo=1,
     ret = sum(i=1,#p-1,numbpart_of_pieces(n,i)) + 1);
  \\ print(" initial smaller "ret);
  for(i=1,#p,
     \\ print(" i="i" n="n" lo="lo" p[i]="p[i]);
     for(j=lo,p[i]-1,
        \\ print("  j="j" num "n-j","#p-i","j" = "numbpart_of_pieces(n-j,#p-i,j));
        ret += numbpart_of_pieces(n-j,#p-i,j));
     lo = p[i];
     n -= p[i];
  );
  \\ print(" ret "ret);
  ret;
}

forpart(p=5,print(p"  "part_to_n(p)));
check_equal(vector(14,n, apply(part_to_n,partitions(n))), \
            vector(14,n, [1 .. numbpart(n)]));

check_equal(vector(14,n, partitions(n)), \
            vector(14,n, Set(partitions(n))));

n_to_part(n,v) = partitions(n)[v];
check_equal(vector(14,n, vector(numbpart(n),v, n_to_part(n,v))), \
            vector(14,n, partitions(n)));

triangular(t) = t*(t+1)/2;

make_Bulgarian_solitaire_by_partitions(n) =
{
  my(partitions_set=Set(partitions(n)));
  vector(#partitions_set,v,
              my(part=partitions_set[v],
                 parent_part=Bulgarian_partition_step(part),
                 parent_v=setsearch(partitions_set,parent_part));
              parent_v!=0 || error("oops, parent not found: "parent_part);
              \\ print("v="v" parent "parent_v);
              if(parent_v==v, 0, parent_v));
}

make_Bulgarian_solitaire(n,loops=0) =
{
  my(vpar=vector(numbpart(n)),
     v=1);
  forpart(p=n,
          my(parent_p=Bulgarian_partition_step(p));
          vpar[v]=if(parent_p==p&&!loops,0,part_to_n(parent_p));
          v++);
  vpar;
}

check_equal(vector(7,n, make_Bulgarian_solitaire(n)), \
            vector(7,n, make_Bulgarian_solitaire_by_partitions(n)));


\\-----------------------------------------------------------------------------
\\ Cycle Lengths

{
  concat(vector(20,n,n+=5;
     my(vpar=make_Bulgarian_solitaire(n),
        lens=vpar_component_cycle_lengths(vpar));
     vecsort(lens);
     vecmin(lens)
               ))
}
quit;     


\\-----------------------------------------------------------------------------

\\ cycle 4 at n = 7 8 9 30 32 34

\\ cf 8n^2 =  0, 8, 32, 72, 128, 200, 288, 392,
{
  print("with cycle 4:");
  for(n=1,200,
     my(vpar=make_Bulgarian_solitaire(n),
        lens=vpar_component_cycle_lengths(vpar));
     lens=vecsort(lens);
     if(!setsearch(Set(lens),4), next);
     print(n"  "lens));
  quit;     
}





\\-----------------------------------------------------------------------------

vpar=make_Bulgarian_solitaire(10);
vpar_validate(vpar);
\\ vpar_dot(vpar);
vpar_degrees_vector(vpar)
vpar_roots(vpar)
vpar_leaves(vpar)
apply(v->vpar_neighbours_of_vertex(vpar,v), vpar_leaves(vpar))
apply(v->vpar_degree_of_vertex(vpar,vpar_neighbours_of_vertex(vpar,v)[1]), vpar_leaves(vpar))

\\ vector(10,t, numbpart(triangular(t)))


\\-----------------------------------------------------------------------------
\\ Num Cyclic Vertices

\\ without self-loops
\\ 2,0,3,3,0,4,6,4,0,5,10,10,5,0,6,15,20,15,6,0,7,21,35,35,21,7,0,8,28,56,70,56
\\ A325191
\\ would be A023532(n-1) * binomial(A002024(n),A002260(n)),
\\
\\ A074909 with loops
\\ 1,2,1,3,3,1,4,6,4,1,5,10,10,5,1,6,15,20,15,6,1,7,21,35,35,21,7,1,8,28,56,70,56,28,
\\ similar A135278
\\ binomial(A002024(n),A002260(n)),

{
  for(n=1,50,
     my(vpar=make_Bulgarian_solitaire(n,1),
        depth=0);
     print1(vpar_num_cyclic_vertices(vpar),","));
  quit;
}
{
  for(t=1,5,
     my(T=t*(t-1)/2);
     for(r=1,t,
        my(n=T + r,
           vpar=make_Bulgarian_solitaire(n));
        print("n="n"  "vpar_num_cyclic_vertices(vpar)" "binomial(t,r))));
  quit;
}


\\-----------------------------------------------------------------------------
\\ Girth

rotate_bits(n,b) =
{
  b <<= 1;
  if(bittest(b,n),
     b += 1 - (1<<n));
  b;
}
check_equal(rotate_bits(3,7), 7);
check_equal(rotate_bits(3,3), 6);
check_equal(rotate_bits(3,6), 5);

binary_str(len,b) =
{
  b=binary(b);
  while(#b<len, b=concat(0,b));
  concat(apply(n->Str(n),b));
}
check_equal(binary_str(1,1), "1");
check_equal(binary_str(3,1), "001");
{
  \\ pattern of k 1-bits in t positions
  \\ step rotates them 1 place
  \\ m = t / gcd(t,k) = minimum back to same
  \\ repeat length m, and t/m blocks, so m divides t
  \\ each block same number of bits so t/m divides k
  \\ minimize m by maximizing t/m so t/m=gcd(t,k)
  \\   
  \\ A054531
  \\ A277227
  \\   additive order of k mod n, for k=0..n-1
  \\   additive order of element x of group G,+ is smallest j with j*x=0
  \\
  for(t=1,5,
     for(k=1,t,
        my(min_len=t, min_cycles=List([]));
        for(b=0,2^t-1,
           hammingweight(b)==k || next;
           my(c=b,
              len=1,
              cycle=List([]));
           while(1,
                 listput(cycle,binary_str(t,c));
                 c=rotate_bits(t,c);
                 if(c<b, next(2));
                 if(c==b, break);
                 len++);
        if(len==min_len, listput(min_cycles,Vec(cycle)),
           len<min_len, min_len=len; min_cycles=List([Vec(cycle)])));

        my(n=triangular(t-1)+k,
           vpar=make_Bulgarian_solitaire(n),
           girth=vpar_girth(vpar),
           g=t/gcd(t,k));
        print("t="t" k="k" n="n"  min_len "g" "girth" "min_len"  by "Vec(min_cycles));
  ));
}
quit



\\-----------------------------------------------------------------------------
\\ A260894

gA260894(limit) = O(x^limit) + sum(m=0,limit, x^m*prod(k=1,m, (1-x^(m+k))/(1-x^k +x*O(x^limit))));
gA260894(4)
check_equal(test_gf_terms(gA260894(#A260894_samples), #A260894_samples), \
            A260894_samples, "gA260894");
{
  my(want=[1,0+1,  2,2,4,5,8,10,15,20,28,36,50,64,86,110,145,184,238,300,384]);
  check_equal(test_gf_terms(gA260894(#want), #want),
              want, "want A260894");
}

\\ A123975
gA123975(limit) =
{
  my(L=O(x^limit));
  prod(i=1,limit, 1/(1-x^i + L))
  * sum(r=1,limit, (-1)^(r-1) * x^((3*r^2+3*r) / 2))
  + L;
}
{
  my(want=[0,1-1,  0,1,1,2,3,5,7,10,14,20,27,37,49,66,86,113,147,190,243]);
  check_equal(test_gf_terms(gA123975(#want), #want),
              want, "want A123975");
}
check_equal(test_gf_terms(gA123975(#A123975_samples+1)/x, #A123975_samples), \
            A123975_samples, "gA123975");

gNumPart(limit) = sum(i=0,limit-1, x^i*numbpart(i)) + O(x^limit);
check_equal(test_gf_terms(gNumPart(#A000041_samples), #A000041_samples), \
            A000041_samples, "gNumPart A000041");

limit = 20
gNumPart(5)
gA123975(5)
gA260894(5)
limit = 200
check_equal(gNumPart(limit),  gA123975(limit) + gA260894(limit));
\\ gNumPart(limit) -  ( gA123975(limit) + gA260894(limit) )


quit


\\-----------------------------------------------------------------------------
\\ vpar_component_cycle_lengths()

{
  for(n=1,50,
     my(vpar=make_Bulgarian_solitaire(n));
     print("n="n"  "vpar_component_cycle_lengths(vpar)));
  quit;
}




\\-----------------------------------------------------------------------------
\\ vpar_to_balanced_binary() of triangular tree
\\
\\ not in OEIS: 2, 56, 4083584, 18856346122018022267346944, 
\\ not in OEIS: 1, 28, 2041792, 9428173061009011133673472, 
\\
\\ in premax unlabelled
\\ not in OEIS: 2, 56, 4162448, 19340910827448974580010592, 
\\ not in OEIS: 1, 28, 2081224, 9670455413724487290005296, 

{
  for(t=1,4,
     my(vpar=make_Bulgarian_solitaire(test_triangular(t)));
     vpar = vpar_to_unlabelled(vpar);
     print1(vpar_to_balanced_binary(vpar),", "));
  print();
  quit;
}


\\-----------------------------------------------------------------------------
\\ vpar_to_primecode() of triangular tree
\\
\\ not in OEIS: 1, 3, 2767, 

{
  \\ soon becomes big
  for(t=1,4,
     my(vpar=make_Bulgarian_solitaire(test_triangular(t)));
     print1(vpar_to_primecode(vpar),", "));
  print();
  quit;
}


\\-----------------------------------------------------------------------------
\\ Depth of partition [n], asked by Cranston and West

\\ 0, 0, 1, 1, 2, 3, 3, 4, 5, 6, 6, 7, 8, 9, 10, 10, 11, 12, 13, 14, 15, 15, 16, 17, 18, 19, 20, 21, 21, 22, 23, 24, 25, 26, 27, 28, 28, 29, 
\\ A083920 num non-triangulars <= n
\\ repeat each triangular

{
  for(n=1,50,
     print("n="n);
     my(p=[n],
        seen=Map(),
       count=0);
     while(!mapisdefined(seen,p),
           count++;
           mapput(seen,p,count);
           print("  "count" "Vec(p));
           p=Bulgarian_partition_step(p));
     print("  cycle back to "Vec(p)" at "mapget(seen,p));
     print("  count "count));
  quit;
}
{
  for(n=1,50,
     my(vpar=make_Bulgarian_solitaire(n),
        v=1,
        depth=0);
     \\ print(vpar);
     while(!vpar_is_cyclic_vertex(vpar,v),
           v=vpar[v];
           \\ print(" to v="v);
           if(!v,break);
           depth++;
           if(depth>#vpar+10, error()));
     print1(depth", ");
  );
  quit;
}


\\-----------------------------------------------------------------------------
\\ Height

\\ K. Igusa, Solution of the Bulgarian Solitaire Conjecture, Math. Mag. 58
\\ (1985), 259-271.

\\ H. J. Bentz, Proof of the Bulgarian Solitaire Conjectures, Ars Combin. 23
\\ (1987), 151-170.

{
  for(t=1,4,
     my(n=triangular(t),
        vpar=make_Bulgarian_solitaire(n),
        top_part=concat([1, [1..t-1], t-1]));
     vecsum(top_part)==n || error();
     my(top_v=part_to_n(top_part),
        r=vpar_root(vpar),
        r_part=n_to_part(n,r));
     part_to_n(r_part)==r || error();
     part_to_n(top_part)==top_v || error();
     print("t="t"  root="r"  "Vec(n_to_part(n,r)));
     print(" height "vpar_height(vpar));
     print(" top "top_v"  "Vec(top_part));
     print(" depth "vpar_depth_of_vertex(vpar,r)"  cf "t*(t+1));
     my(vertices=vpar_height_vertices(vpar));
     for(i=1,#vertices,
        my(v=vertices[i]);
        print("  "v"  "Vec(n_to_part(n,v))"  depth "vpar_depth_of_vertex(vpar,v)));
     print(" ");
  );
  quit;
}


\\-----------------------------------------------------------------------------

A037306_table =
{[[ 1 ],
  [ 1,  1 ],
  [ 1,  1,  1 ],
  [ 1,  2,  1,  1 ],
  [ 1,  2,  2,  1,  1 ],
  [ 1,  3,  4,  3,  1,  1 ],
  [ 1,  3,  5,  5,  3,  1,  1 ],
  [ 1,  4,  7, 10,  7,  4,  1,  1 ],
  [ 1,  4, 10, 14, 14, 10,  4,  1,  1 ],
  [ 1,  5, 12, 22, 26, 22, 12,  5,  1,  1 ],
  [ 1,  5, 15, 30, 42, 42, 30, 15,  5,  1,  1 ]
]};
{
  for(t=1,4,
     my(T=t*(t-1)/2);
     for(r=1,t,
        my(n=T + r);
        print("t="t" r="r" n="n
              "  "vpar_num_components(make_Bulgarian_solitaire(n))
              " "A037306_table[t][r])));
}
quit

\\-----------------------------------------------------------------------------
\\ Garden of Eden partitions = those with no predecessor, so childless

{
  \\ all n
  \\ 1,0,1,1,2,3,5,7,10,14,20,27,37,49,66,86,113,147,190,243,311,394,499,627,786,980,1220,1510,
  \\ A123975
  for(n=1,30,
     print1(vpar_num_childless(make_Bulgarian_solitaire(n)),","));
  quit;
}
{
  \\ triangular
  \\ 1, 1, 3, 14, 66, 311, 1510, 7507, 38032, 195752,
  for(t=1,14,
     print1(vpar_num_childless(make_Bulgarian_solitaire(triangular(t))),", "));
  quit;
}

\\ vpar_view(make_Bulgarian_solitaire(7))
\\ A123975 Number of Garden of Eden partitions of n in Bulgarian Solitaire.
\\ A225794 Sizes of Garden of Eden partitions in Bulgarian Solitaire, given in the same order as the runlength codes of the corresponding partitions (A227753).

\\ for(n=1,20, print1(vpar_is_cyclic(make_Bulgarian_solitaire(n)),", "));
\\ 0, 1, 0, 1, 1, 0, 1, 1, 1, 0, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1,

\\-----------------------------------------------------------------------------
\\ Row Widths

\\ triangular
\\ 1, 1, 1, 1, 1, 1, 2, 3, 2, 1, 1, 1, 1, 3, 5, 5, 3, 4, 4, 4, 3, 3, 3, 3, 1, 1, 3, 7, 12, 12, 7, 8, 8, 7, 6, 5, 6, 7, 10, 11, 9, 11, 13, 16, 16, 1, 1, 3, 8, 17, 26, 26, 18, 20, 24, 20, 15, 15, 13, 15, 17, 15, 14, 15, 14, 17, 24, 34, 47, 47, 33, 41, 55, 62, 70, 65
\\ not in OEIS: 3, 5, 5, 3, 4, 4, 4, 3, 3, 3, 3, 1, 1, 3, 7, 12, 12, 7, 8, 8, 7, 6, 5, 6, 7, 10, 11, 9, 11, 13, 16, 16, 1, 1, 3, 8, 17, 26, 26, 18, 20, 24, 20, 15, 15, 13, 15, 17, 15, 14, 15, 14, 17, 24, 34, 47, 47, 33, 41, 55, 62, 70, 65
\\
\\ triangular, upwards
\\ 1, 1, 1, 1, 1, 1, 2, 3, 2, 1, 1, 3, 3, 3, 3, 4, 4, 4, 3, 5, 5, 3, 1, 1, 16, 16, 13, 11, 9, 11, 10, 7, 6, 5, 6, 7, 8, 8, 7, 12, 12, 7, 3, 1, 1, 65, 70, 62, 55, 41, 33, 47, 47, 34, 24, 17, 14, 15, 14, 15, 17, 15, 13, 15, 15, 20, 24, 20, 18, 26, 26, 17, 8, 3, 1, 1
\\ not in OEIS: 4, 4, 4, 3, 5, 5, 3, 1, 1, 16, 16, 13, 11, 9, 11, 10, 7, 6, 5, 6, 7, 8, 8, 7, 12, 12, 7, 3, 1, 1, 65, 70, 62, 55, 41, 33, 47, 47, 34, 24, 17, 14, 15, 14, 15, 17, 15, 13, 15, 15, 20, 24, 20, 18, 26, 26, 17, 8, 3, 1, 1

\\
{
  my(l=List([]));
  for(t=1,6,
     my(vpar=make_Bulgarian_solitaire(triangular(t)),
        widths=vpar_row_widths_vector(vpar));
     widths=Vecrev(widths);
     for(i=1,#widths, listput(l,widths[i])));
  print(Vec(l));
  quit;
}
{
  for(t=1,14,
     my(vpar=make_Bulgarian_solitaire(triangular(t)));
     print(vpar_row_widths_vector(vpar)));
  print();
  quit;
}

\\-----------------------------------------------------------------------------
\\ Num Children of Root

\\ triangular, root children
\\ 0, 1, 1, 1, 1, 1, 1, 1, 1,
\\ always 1 ?
\\
\\ triangular, root grandchildren
\\ 0, 1, 2, 3, 3, 3, 3, 3, 3,
\\ always 3 for n>=10 ?
\\
{
  print("starting ",vector(4,t,triangular(t)));
  for(t=1,14,
     my(vpar=make_Bulgarian_solitaire(triangular(t)));
     print1(vpar_num_grandchildren_of_vertex(vpar,vpar_root(vpar)),", "));
  print();
  quit;
}

\\-----------------------------------------------------------------------------
\\ Number of Components
\\
\\ all n
\\ 1,1,1,1,1,1,1,2,1,1,1,2,2,1,1,1,3,4,3,1,1,1,3,5,5,3,1,1,1,4,7,10,7,4,1,1,1,4,10,14,14,10,4,1,1,1,5,12,22,26,22,12,
\\ A037306 triangule num compositions n into k parts, up to cyclic shifting
\\ Akin and Davis ...

{
  for(n=1,60,
     print1(vpar_num_components(make_Bulgarian_solitaire(n)),","));
  print();
  quit
}

\\-----------------------------------------------------------------------------
\\ Diameter

\\ triangular diameter
\\              0, 2,
\\ not in OEIS: 8, 17, 29, 44, 62, 83, 107, 138,
\\ is not A143208 which has 134 where here want 138
\\
\\ diameter +1
\\ not in OEIS: 1, 3, 9, 18, 30, 45, 63, 84, 108, 139,
{
  for(t=1,14,
     my(vpar=make_Bulgarian_solitaire(triangular(t)));
     print1(vpar_diameter(vpar)+1,", "));
  quit;
}

\\-----------------------------------------------------------------------------
\\ Root Vertex Number, in partitions() ordering
\\
\\ triangular
\\ not in OEIS: 1, 2, 6, 20, 74, 295, 1226, 5237,

{
  for(t=1,14,
     my(vpar=make_Bulgarian_solitaire(triangular(t)));
     print1(vpar_root(vpar),", "));
  print();
  quit
}

\\-----------------------------------------------------------------------------

\\ A226062 a(n) = Bulgarian solitaire operation applied to the partition encoded in the runlengths of binary expansion of n.
\\ A242424 Bulgarian solitaire operation on partition list A112798: a(1) = 1, a(n) = A000040(A001222(n)) * A064989(n).

\\ A227452

\\--------------------
\\ heights 0, 2, 6, 12, 20, 30, 42, 56, 72
\\ A002378 pronic numbers n*(n+1).
\\ vector(9,t, vpar_height(make_Bulgarian_solitaire(triangular(t))))
\\ with +1
\\ vector(8,t, vpar_height(make_Bulgarian_solitaire(triangular(t)))+1)
\\ 1, 3, 7, 13, 21, 31, 43, 57
\\ A002061 pronic+1, as described in A227452

\\ num vertices attaining height
\\ 1, 1, 1, 3, 16, 65, 293, 1267, 5686,
\\ not in OEIS: 3, 16, 65, 293, 1267, 5686,
\\ for(t=1,10, print1(vpar_height_num_vertices(make_Bulgarian_solitaire(triangular(t)))", "));

\\ vector(5,t, t*(t+1)/2)
\\ for(n=10,10, print(vpar_to_sparse6(make_Bulgarian_solitaire(n))));

\\--------------------
\\ Automorphisms
\\ {
\\   for(t=1,7,
\\      my(n=triangular(t),
\\         vpar=make_Bulgarian_solitaire(n));
\\      print("n="n" t="t" size "#vpar" "
\\            "  "vpar_num_automorphism_generators(vpar),
\\            "  "vpar_num_free_automorphism_generators(vpar),
\\            "  "vpar_num_automorphisms(vpar)
\\           ));
\\ }
\\ vpar_dot(make_Bulgarian_solitaire(triangular(2)));
