\\ Copyright 2020 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(recover,0)
default(strictargs,1)
default(parisizemax,300*10^6);
read("../vpar.gp");
read("../vpar-wip.gp");
read("../test-helpers.gp");
read("../examples/Huffman-1-to-n.gp");

print("===============================================================\n\n");

\\ GP-DEFINE  read("../vpar.gp");
\\ GP-DEFINE  read("../vpar-wip.gp");

option_verbose=0;
option_progress = 1;


\\-----------------------------------------------------------------------------
progress("Huffman New Vertex Pos");

\\ When inserting the new weight w, if there's an existing w_list[] entry of
\\ that same w then insert new p vertex after that entry, instead of before
\\ as done in make_Huffman_n(n).
\\
\\ The effect is to keep w_list+v_list sorted first by descending w and
\\ among equal w by ascending v.
\\
new_vertex_first_make_Huffman_n(n) =
{
  my(vpar=vector(2*n-1),
     w_list=List(Vecrev([1..n])),
     v_list=w_list,
     p=n,
     k);
  while((k=#w_list)>=2,
        \\ print(Vec(w_list)" "Vec(v_list));
        for(i=1,#w_list-1,
           lex([w_list[i],-v_list[i]],[w_list[i+1],-v_list[i+1]]) > 0
             || error("oops, w_list is meant to be sorted"));
   
        if(#w_list>=3,
           if(w_list[#w_list]==w_list[#w_list-1]
              && w_list[#w_list]==w_list[#w_list-2],
              error("oops, 3 same weight")));

        if(#w_list>=3,
           if(w_list[#w_list-1]==w_list[#w_list-2],
              print("2nd and 3rd same")));
     
        p++;
        vpar[v_list[k]] = vpar[v_list[k-1]] = p;  \\ children of new p
        my(w = w_list[k] + w_list[k-1]);          \\ new weight at p
        listpop(w_list); listpop(w_list);
        listpop(v_list); listpop(v_list);
        forstep(pos=#w_list,1,-1,                 \\ last to first
           if(w_list[pos]>=w,
              \\ print("  popped to "Vec(w_list)" "Vec(v_list));
              \\ print("  insert at pos="pos" cf "w_list[pos]);
              listinsert(w_list,w,pos+1);  \\ immediately after pos
              listinsert(v_list,p,pos+1);
              next(2)));
        listinsert(w_list,w,1);   \\ w is the biggest weight
        listinsert(v_list,p,1));
  vpar;
}

{
  for(n=1,150,
     my(vpar=make_Huffman_n(n),
        newfirst_vpar=new_vertex_first_make_Huffman_n(n));
     check_equal(vpar,newfirst_vpar));
}

\\------------------------------------------------------------------------------
progress("end");
