\\ Copyright 2020, 2021 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(recover,0)
default(strictargs,1)
\\ default(parisizemax,300*10^6);
read("../test-helpers.gp");
read("memoize.gp");
\\ GP-DEFINE  read("../vpar.gp");
\\ GP-DEFINE  read("../vpar-wip.gp");

option_verbose=1;
progress_prev=-1;
progress(str) =
{
  my(t=getwalltime()\2000);
  if(t!=progress_prev,
     progress_prev=t;
     print(str));
}

\\default(realprecision,200);
nearly_equal_epsilon = 1e-10;
nearly_equal(x,y, epsilon=nearly_equal_epsilon) = {
  \\ print(type(x)," ",type(y)," ",x," ",y);
  my(d=x-y); \\ ensure compatible types
  if(type(x)=="t_MAT",x=Vec(x);y=Vec(y));
  if(type(x)=="t_VEC" || type(x)=="t_COL",
       sum(i=1,max(#x,#y), !nearly_equal(x[i],y[i],epsilon)) == 0,
     type(x)=="t_QUAD" || type(x)=="t_COMPLEX",
          nearly_equal(real(x-y),0,epsilon)
       && nearly_equal(imag(x-y),0,epsilon),
     abs(d) < epsilon);
}
check_nearly_equal(got,want,name="") =
{
  my(bad=0);
  check_count++;
  if(!nearly_equal(got,want),
     print("wrong at "check_count": "name);
     print("got  "got);
     print("want "want);
     bad=1);

  if(bad,
     quit(1),
     option_verbose,
     print("ok "check_count" "name));
}


\\ Jacobsthal numbers A001045
\\ for(n=3,20,print(J(n)" "factor(J(n))))
\\
\\ biggest prime factor
\\ vector(20,n,n+=2; my(m=factor(J(n))[,1]); m[#m])
\\ 3, 5, 11, 7, 43, 17, 19, 31, 683, 13, 2731, 127, 331, 257, 43691, 73, 174763, 41, 5419, 683
\\ A271314
\\
J(n) = (2^n - (-1)^n)/3;
{
  check_equal(vector(#A001045_samples,n,n--; J(n)),
              A001045_samples,  \\ OFFSET=0
              "J() = A001045_samples");

  \\ A014113 = 2*J(n)
  check_equal(vector(#A014113_samples,n,n--; 2*J(n)),
              A014113_samples,  \\ OFFSET=0
              "2*J() = A014113_samples");

  \\ A139818 = J(n)^2
  check_equal(vector(#A139818_samples,n,n--; J(n)^2),
              A139818_samples,  \\ OFFSET=0
              "J()^2 = A139818_samples");
}

twokp(k) = 2^k+1;
twokm(k) = 2^k-1;

maxprimefactor(n) = my(m=factor(n)[,1]); if(#m,m[#m],1);

{
  \\ J new prime factor by gcd
  \\
  \\ 2^k-1  k=6  63 = 3*3*7
  my(
     func=twokp, min_k=1,
     \\ func=twokm,min_k=2,
     \\ func=J,min_k=3,
     x=0);
  for(k=min_k,100,
     my(p=func(k),
        l=List([]));
\\     print1("k="k,Strchr(13));
     
     for(i=min_k,k-1,
        my(d=func(i));
        while(1,
              my(g=gcd(p,d));
              \\ print("  "k" p="p" i="i" d="d" gcd "g" to "p/g" "factor(p/g)[,1]~);
              if(g==1,break);
              p /= g;
              listput(l, d);
              if(p==1,
                 print("k="k" no new, used "Vec(l));
                 \\ print(factor(func(k))" took "Vec(l));
                 next(3))));
     print(k" left "test_to_binary(p)"  ",if(p==func(k)/3," unchanged",""));
      \\ if(p==func(k), print1(k","));
  );
  quit;
}
\\ no common factor with earlier, 2^k-1 for k prime ?
\\ 7,11,13,17,19,23,29,31,37,41,43,47,53,59,61,67,71,73,79,83,89,97,101,103,107



\\ 255 - 2*127 == 1
\\ 255 - 4*63 == 3
\\ 255 - 8*31 == 7
\\ 255 - 16*15 == 15
\\ gcd(255,15) == 15
\\ 255/15 == 17


\\ ways_2kp1(n,min_k=1) =
\\ {
\\   for(k=min_k,logint(n,2),
\\      my(p=2^k+1);
\\      while(n%p==0,
\\ }
{
  \\ products ways
  \\ A282572 products of 2^k-1
  \\
  my(v=[ [1] ],
     \\ func=twokm,min_k=2,
     func=J,min_k=3,
     max_k=16,
     limit=func(max_k));
  for(k=min_k,max_k,
     my(p=func(k));
     \\ print(p"  "v);
     for(i=1,#v,
        my(elem=v[i]);
        for(j=1,oo,
           elem[1]*=p;
           if(elem[1]>limit,break);
           elem=concat(elem,[k]);
           v=concat(v,[elem]))));
  v=vecsort(v);
  for(i=1,#v,
     print(v[i]));

  for(i=1,#v-1,
     if(v[i][1]==v[i+1][1],
       print("dup "v[i]"  "v[i+1])));

  my(rep=1);
  for(i=1,#v-1,
     if(v[i][1]==v[i+1][1],
        rep++,
        \\ print1(rep",");
        \\ print1(v[i][1]",");
        print1(maxprimefactor(v[i][1])",");
        rep=1;));
  print();
  quit;
}
