\\ Copyright 2021, 2022 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(recover,0);
default(strictargs,1);

option_verbose=0;
option_progress=1;

read("../vpar.gp");
read("../vpar-wip.gp");
read("../test-helpers.gp");
read("OEIS-data.gp");
read("OEIS-data-wip.gp");
read("a061773-Matula-sorted.gp");
print();

print("primelimit = ",default(primelimit));


\\-----------------------------------------------------------------------------
\\ Helpers

greatest_prime_factor(n) = if(n==1,1, my(f=factor(n)[,1]); f[#f]);
{
  my(want=OEIS_data("A006530"));  \\ OFFSET=1
  check_equal(vector(#want,n,greatest_prime_factor(n)),
              want,
              "greatest_prime_factor() vs A006530 data");
}


\\-----------------------------------------------------------------------------
\\ num_trees_vector()

{
  my(want=OEIS_data("A000081"));  \\ OFFSET=0
  check_equal(want[1], 0);
  want=want[^1];  \\ drop A000081(0)=0
  check_equal(num_trees_vector(#want), want,
             "num_trees_vector() vs A000081 data");
}
{
  check_equal(vector(20,len,len--; #num_trees_vector(len)),
              vector(20,len,len--; len),
              "num_trees_vector() length");
  check_equal(vector(20,len,len--; num_trees_vector(len)),
              vector(20,len,len--; vpar_count_unlabelled_vector(len)),
              "num_trees_vector() vs vpar_count_unlabelled_vector()");
}


\\-----------------------------------------------------------------------------
\\ A061773_rows()

check_equal(A061773_rows(4), [ [1], [2], [3,4], [5,6,7,8] ], "example");

row_by_vpar(n) =
{
  my(vpar=vpar_unlabelled_first(n),
     l=List([]));
  until((vpar=vpar_unlabelled_next(vpar)) == 'none,  \\ trees
        listput(l,vpar_to_primecode(vpar)));
  vecsort(Vec(l));
}

check_equal(A061773_rows(0), []);
check_equal(A061773_rows(1), [ [1] ]);
check_equal(A061773_rows(2), [ [1], [2] ]);
check_equal(A061773_rows(3), [ [1], [2], [3,4] ]);

maxn=13;
print(num_trees_vector(maxn)"  ",vecsum(num_trees_vector(maxn)));
gettime(); got=A061773_rows(maxn);
print("got time ",gettime());
print();
print("all got size ",vecsum(apply(length,got)));
print("max term ",vecmax(concat(got)));
check_equal(got, vector(maxn,n, row_by_vpar(n)));
{
  for(n=1,#got,
     check_equal(vecmin(got[n]), OEIS_data("A005517")[n],
                "row min");
     check_equal(vecmax(got[n]), OEIS_data("A005518")[n],
                "row max");
     check_equal(#got[n], vpar_count_unlabelled(n),
                "row width");

     check_equal(Set(got[n]), got[n],
                "row distinct and sorted");

     check_equal(apply(primecode->#vpar_from_primecode(primecode), got[n]),
                 vector(#got[n],i,n),
                 "row num vertices values");
     );
}

{
  \\ ~/OEIS/b061773.txt
  \\ rows n=1..12 = 80 kbytes ending 50728129
  my(want=OEIS_data("A061773",'bfile));
  print("b061773.txt size ",#want);
  my(allgot=concat(got));
  check_equal(allgot[1..#want], want,
             "A061773_rows() vs b061773.txt");
}

rows_code_with_checks(maxn) =
{
  my(P_to_bigness=Map());

  my(num_trees=num_trees_vector(maxn),
     rows=vector(maxn,n, if(n==1,[1], vector(num_trees[n]))),
     bigs=apply(Vecsmall,rows),
     bigidx=1);
  for(n=2,maxn,
     my(upto=0);

     \\ non-planted trees
     for(p=2,n-1,       \\ number of vertices planted part
        my(s=n-p+1);    \\ number of vertices other part, p+s == n+1
        for(i=num_trees[p]-num_trees[p-1]+1, num_trees[p],  \\ end-most
           my(P=rows[p][i],       \\ planted tree P
              P_big=bigs[p][i]);  \\ bigness of its Z subtree
           for(j=1,#rows[s],
              if(p>s, bigs[p][i] >= bigs[s][j] || error());  \\ always when p>s

              P_big >= bigs[s][j] || break; \\ must Z biggest
              rows[n][upto++] = P*rows[s][j];
              bigs[n][upto]   = P_big)));

     \\ planted trees
     for(i=1,num_trees[n-1],
        rows[n][upto++] = prime(rows[n-1][i]);  \\ new tree
        bigs[n][upto]   = bigidx++);


     \\ CHECKS:
     my(plants=rows[n][num_trees[n]-num_trees[n-1]+1 .. #rows[n]]);
     select(isprime,rows[n]) == plants || error();
     bigs[n]==vecsort(bigs[n])  || error();  \\ ascending order

     for(i=1, num_trees[n]-num_trees[n-1],               
        !isprime(rows[n][i])                 || error();   \\ composites
        my(F=factor(rows[n][i])[,1],
           Fbigs=apply(P->mapget(P_to_bigness,P),F));
        \\ print("consider ",rows[n][i]," bigness ",bigs[n][i]," gpf "P" is ",mapget(P_to_bigness,P));
        bigs[n][i] == vecmax(Fbigs)   || error());

     for(i=num_trees[n]-num_trees[n-1]+1, num_trees[n],    \\ end-most
        isprime(rows[n][i])  || error();                   \\ primes
        \\ print("P=",rows[n][i]," bigness ",bigs[n][i]);
        !mapisdefined(P_to_bigness,rows[n][i])  || error();
        mapput(P_to_bigness, rows[n][i], bigs[n][i]));
  );

  apply(vecsort,rows);
}

check_equal(got, rows_code_with_checks(maxn));


\\-----------------------------------------------------------------------------

max_of_num_vertices(n) =
{
  OEIS_data("A005518")[n];
}

rows_by_search(len) =
{
  my(maxn=max_of_num_vertices(len),
     prev=vectorsmall(maxn\2),
     rows=apply(vector,num_trees_vector(len)),
     upto=vector(len),
     pi=0);               \\ number primes seen so far
  forfactored(elem=1,maxn,
     my(n,f); [n,f]=elem;
     my(nv = if(matsize(f)[1]==1 && f[1,2]==1,
                prev[pi++],                      \\ n prime
                [prev[p]-1 | p<-f[,1]] * f[,2])  \\ n composite
             + 1);
     if(n<=#prev,
        prev[n] = nv);   \\ save 
     if(nv <= len,
        rows[nv][upto[nv]++] = n));   \\ desired size, add to row[nv]
  rows;
}

check_equal(  max_of_num_vertices(12), 50728129);
check_equal(4*max_of_num_vertices(12), 202912516);
check_equal(2^31-1,                    2147483647);

check_equal(rows_by_search(10), A061773_rows(10));


\\-----------------------------------------------------------------------------
check_equal(prev_vpar,'prev_vpar);
check_equal(seen,'seen);
check_equal(levels,'levels);
check_equal(vpar,'vpar);
print("end a061773-Matula-sorted--test.gp");
