\\ Copyright 2021, 2022 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(recover,0);
default(strictargs,1);

option_verbose=0;
option_progress=1;

read("../vpar.gp");
read("../vpar-wip.gp");
read("../test-helpers.gp");
read("OEIS-data.gp");
read("OEIS-data-wip.gp");
read("a347539-BH-Matula.gp");


\\-----------------------------------------------------------------------------
\\ A087803 cumulative num rooted trees

A087803(n) =
{
  n>=1 || error();
  vecsum(vpar_count_unlabelled_vector(n));
}
{
  my(v=OEIS_data("A087803"));
  check_equal(vector(#v,n,A087803(n)), v,
              "A087803 cumualtive rooted trees");
}


\\-----------------------------------------------------------------------------
\\ levels_to_primecode_with_limit()

{
  check_equal(levels_to_primecode_with_limit([1],10), 1);
  check_equal(levels_to_primecode_with_limit([1],1), 1);
  check_equal(levels_to_primecode_with_limit([1,2,3,4],1), 'toobig);

  check_equal(levels_to_primecode_with_limit([1,2,3,4],5), 5);
  check_equal(levels_to_primecode_with_limit([1,2,3,4],4), 'toobig);

  check_equal(levels_to_primecode_with_limit([1],0), 1,
              "limit 0 not enforced against singleton");
  check_equal(levels_to_primecode_with_limit([1,2],0), 'toobig);
}


\\-----------------------------------------------------------------------------
\\ next_levels() examples

{
  my(levels=[1],
     seen=0);
  for(n=1,100,
     if(n==1, check_equal(levels,[1]); seen++);
     if(n==2, check_equal(levels,[1,2]); seen++);
     if(n==3, check_equal(levels,[1,2,3]); seen++);
     if(n==33, check_equal(levels,[1,2,3,3,2,3]); seen++);
     levels=next_levels(levels);
     );
  check_equal(seen,4);
}


\\-----------------------------------------------------------------------------
\\ A346913  levels
\\ next_levels() iteration

{
  my(want=OEIS_data("A346913"));
  my(got=[],
     levels=[1]);
  while(#got<#want,
        got=concat(got,levels);
        levels=next_levels(levels));
  check_equal(got,want);     
}

level_sequence_of_vpar(vpar) = \
  apply(depth->depth+1, vpar_depths_vector(vpar));
{
  \\ versus vpar_unlabelled_next()
  my(bad=0);
  for(n=0,12,
     my(vpar=vpar_unlabelled_first(n),
        levels=[1..n]);
     until((vpar=vpar_unlabelled_next(vpar)) == 'none,
           if(levels != level_sequence_of_vpar(vpar),
              bad++);
           levels=next_levels(levels)));
  check_equal(bad, 0, 
              "next_levels() vs vpar_unlabelled_next() trees");
}


\\-----------------------------------------------------------------------------
\\ A347539_vector()

{
  check_equal(A347539_vector(0), []);
  check_equal(A347539_vector(1), [1]);
  check_equal(A347539_vector(2), [1,2]);
  check_equal(vector(30,len,len--; #A347539_vector(len)),
              vector(30,len,len--; len));
}

{
  if(0,
     my(v=A347539_vector(64));
     for(i=1,#v,print1(v[i],",");if(i==25||i==45,print()));
     quit);

  my(want=OEIS_data("A347539"));
  check_equal(A347539_vector(#want), want);
}

{
  check_equal(vecsum(vpar_count_unlabelled_vector(12)), 7813);
  check_equal(OEIS_data("A005518")[12],
              50728129,
              "max primecode of 12 vertices");

  \\ 7813 all to 12 vertices is 80 kbytes
  if(0,
     system("rm -f /tmp/b347539.txt");
     my(v=A347539_vector(7813));
     for(n=1,#v, 
        write("/tmp/b347539.txt", Str(n" "v[n])));
     system("ls -l /tmp/b347539.txt");
     quit);

  \\ ~/OEIS/b347539.txt
  if(0,
     my(want=OEIS_data("A347539",'bfile));
     print("b347539.txt length ",#want);
     check_equal(A347539_vector(#want), want));
}


\\-----------------------------------------------------------------------------
\\ A347540_vector()

primerep(n) = my(ret=1); for(i=1,n,ret=prime(ret)); ret;
{
  check_equal(A347540_vector(0), []);
  check_equal(A347540_vector(1), [1]);
  check_equal(A347540_vector(2), [1,2]);
  check_equal(vector(30,len,len--; #A347540_vector(len)),
              vector(30,len,len--; len));

  check_equal(OEIS_data("A005517")[14], 1125);
  check_equal(OEIS_data("A087803")[14], 53272);
  my(got=A347540_vector(1125));
  check_equal(vecmax(got), 53112);
  check_equal(OEIS_data("A005518")[13], 997525853,
              "A005518(13) = 997525853");
  check_equal(isprime(997 525 853), 1,
              "isprime() 997525853");
  check_equal(primepi(997 525 853), 50 728 129,
              "primepi() 997525853");
}

{
  if(0,
     \\ DATA
     my(v=A347540_vector(66));
     for(i=1,#v,print1(v[i],",");if(i==25||i==47,print()));
     quit);

  my(want=OEIS_data("A347540"));
  check_equal(A347540_vector(#want), want);
}

check_equal(OEIS_data("A005517")[17], 5625);
check_equal(OEIS_data("A005517")[18], 9375);
check_equal(vecsum(vpar_count_unlabelled_vector(17)), 1011311);
check_equal(#vpar_from_primecode(5624), 13);
check_equal(vecmax([#vpar_from_primecode(c)|c<-[1..5624]]), 16);
check_equal(#vpar_from_primecode(5625), 17, "num vertices of 5625");

{
  \\ 5624 stop before needing 17 vertices (at primecode=5625)
  \\ 7813 would match b347539.txt
  \\ 9374 stop before needing 18 vertices (at primecode=9375)
  if(0,
     system("rm -f /tmp/b347540.txt");
     my(limit=7813);
     print("make /tmp/b347540.txt limit = ",limit);
     my(v=A347540_vector(limit));
     for(n=1,#v, 
        write("/tmp/b347540.txt", Str(n" "v[n])));
     system("ls -l /tmp/b347540.txt");
     quit);

  \\ ~/OEIS/b347540.txt
  if(0,
     my(want=OEIS_data("A347540",'bfile));
     print("b347540.txt length ",#want);
     check_equal(A347540_vector(#want), want));
}


\\-----------------------------------------------------------------------------
check_equal(prev_levels,'prev_levels);
check_equal(prev_vpar,'prev_vpar);
check_equal(seen,'seen);
check_equal(levels,'levels);
check_equal(vpar,'vpar);
print("end a347539-BH-Matula-test.gp");
