\\ Copyright 2021, 2022 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(parisizemax,500*10^6);
default(recover,0);
default(strictargs,1);

option_verbose=0;
option_progress=1;

read("../vpar.gp");
read("../vpar-wip.gp");
read("../test-helpers.gp");
read("OEIS-data.gp");
read("OEIS-data-wip.gp");
read("a347620-Matula-inverse.gp");


\\-----------------------------------------------------------------------------
\\ Helpers

A087803(n) = vecsum(vpar_count_unlabelled_vector(n));
{
  my(want=OEIS_data("A087803")); \\ OFFSET=1
  check_equal(vector(#want,n, A087803(n)),
              want);
}

\\ num vertices
A061775(n) = #vpar_from_primecode(n);
{
  my(want=OEIS_data("A061775")); \\ OFFSET=1
  check_equal(vector(#want,n, A061775(n)),
              want,
              "A061775()");
}

\\-----------------------------------------------------------------------------
\\ Progressive

A347620_progressive(maxn,callback) =
{
  my(nv       =vector(maxn),
     c        =List([]),
     num_trees=List([1]),
     upto     =List([0,1]),
     total_trees=1,
     pi=0);               \\ number of prime n seen so far
  forfactored(elem=1,maxn,
     my(n,f); [n,f]=elem;
     my(num_vertices);
     if(matsize(f)[1]==1 && f[1,2]==1,
        pi++;
        num_vertices = nv[pi]+1;
        , 
        num_vertices = [nv[p]-1|p<-f[,1]] * f[,2] + 1);
     nv[n]=num_vertices;

     while(#num_trees<num_vertices-1,
           my(k=#num_trees);
           listput(c, sumdiv(#num_trees,d, num_trees[d]*d));
           my(this=Vec(num_trees)*Colrev(c) / #num_trees);
           \\ this==vpar_count_unlabelled(#num_trees+1) || error();
           listput(num_trees, this);
           listput(upto, total_trees += this);
          );
     callback(n, upto[num_vertices]++));
}
A347620_progressive_output=vector(1000);
{
  A347620_progressive(1000, (n,pos)-> A347620_progressive_output[n]=pos);
  check_equal(A347620_progressive_output,
              A347620_vector(1000),
              "A347620_progressive()");
}


\\-----------------------------------------------------------------------------
\\ num_vertices_vector()

{
  my(reps=100,
     want=vector(reps,primecode, #vpar_from_primecode(primecode)));
  for(len=1,reps,
     check_equal(num_vertices_vector(len), want[1..len],
                 "num_vertices_vector()"));

  check_equal(num_vertices_vector(1000),
              vector(1000,n, A061775(n)),
              "num_vertices_vector() vs A061775()");
}

\\ num_vertices_vector() adds subtree size ret[pi] to each n which has a
\\ prime factor p and thus a subtree with primecode pi = primepi(p).  If n
\\ has multiple factors of p then all those subtrees are added by working
\\ through powers p^reps to find n divisible by reps many copies of p and so
\\ additional subtree,
\\
\\ prime factor p and thus a subtree with primecode pi =
\\ primepi(p).  If n has multiple factors of p then all those subtrees are
\\ added by working through powers p^reps to find n divisible by reps many
\\ copies of p and so additional subtree,
num_vertices_vector_by_forprime(len) =
{
  my(ret=vector(len,i,1),
     pi=0);     \\ primepi(p)
  forprime(p=1,len, pi++;
     my(P=p);   \\ power p^reps
     until((P*=p) > len,
       forstep(n=P,len,P,   \\ each n divisible by P
          ret[n] += ret[pi])));
  ret;
}

{
  check_equal(num_vertices_vector_by_forprime(1000),
              num_vertices_vector(1000),
              "num_vertices_vector_by_forprime()");
}

\\-----------------------------------------------------------------------------
\\ Examples

{
  check_equal(OEIS_data("A005517")[7], 25);
  check_equal(vecsum(vpar_count_unlabelled_vector(6)), 37);
  check_equal(A087803(6), 37);
  check_equal(#vpar_from_primecode(25), 7);
  check_equal(#vpar_from_primecode(26), 6);
  check_equal(#vpar_from_primecode(27), 7);

  my(inv=A347620_vector(30));
  check_equal(num_vertices_vector(30),
              [1, 2, 3, 3, 4, 4, 4, 4, 5, 5, 5, 5, 5, 5, 6, 5, 5, 6, 5, 6, 6, 6, 6, 6, 7, 6, 7, 6, 6, 7]);
  \\           1  2  3  4  5  6          10             15       18    20             25    27
  check_equal(inv,
              [1, 2, 3, 4, 5, 6, 7, 8, 9,10,11,12,13,14,18,15,16,19,17,20,21,22,23,24,38,25,39,26,27,40]);
  check_equal(inv[25], 38);
  check_equal(inv[27], 39);

  \\----------------
  check_equal(OEIS_data("A005517")[6], 15);
  check_equal(vecsum(vpar_count_unlabelled_vector(5)), 17);

  check_equal(num_vertices_vector(23),
              [1, 2, 3, 3, 4, 4, 4, 4, 5, 5, 5, 5, 5, 5, 6, 5, 5, 6, 5, 6, 6, 6, 6]);
  \\           1  2  3  4  5  6        9                15       18    19 20
  check_equal(A347620_vector(23),
              [1, 2, 3, 4, 5, 6, 7, 8, 9,10,11,12,13,14,18,15,16,19,17,20,21,22,23]);
}

{
  \\ DATA section
  if(0,
     my(got=A347620_vector(68));
     for(i=1,#got,print1(got[i],",");if(i==25||i==47,print()));
     print();
     quit);

  my(want=OEIS_data("A347620"));
  check_equal(A347620_vector(#want), want);
}


{
  \\ b-file, cf size A061773
  \\ #OEIS_data("A061773",'bfile) == 7813 is to 12 vertices
  \\ ~/OEIS/b061773.txt
  if(0,
     system("rm -f /tmp/b347620.txt");
     my(got=A347620_vector(7813));
     for(n=1,#got, 
        write("/tmp/b347620.txt", Str(n" "got[n])));
     system("ls -l /tmp/b347620.txt");
     quit);

  \\ ~/OEIS/b347620.txt
  if(0,
     my(want=OEIS_data("A347620",'bfile));
     print("b347620.txt length ",#want);
     check_equal(A347620_vector(#want), want));
}


\\-----------------------------------------------------------------------------
\\ num_trees_vector()

{
  check_equal(vector(30,len, num_trees_vector(len)),
              vector(30,len, vpar_count_unlabelled_vector(len)));

  check_equal(num_trees_vector(100),
              vpar_count_unlabelled_vector(100));

  check_equal(num_trees_vector(1)[1], 1);
  check_equal(vpar_count_unlabelled_vector(5), [1, 1, 2, 4, 9]);
  check_equal(vecsum(vpar_count_unlabelled_vector(4)), 8);
  check_equal(OEIS_data("A005517")[5], 9);
  check_equal(OEIS_data("A005518")[5], 19);
  check_equal(vpar_count_unlabelled(5), 9);

  my(want=OEIS_data("A000081"));  \\ OFFSET=0
  check_equal(want[1], 0);
  want=want[^1];  \\ drop A000081(0)=0
  check_equal(num_trees_vector(#want), want,
             "num_trees_vector() vs A000081 data");
}


\\-----------------------------------------------------------------------------
\\ num_smaller_trees_vector()

{
  my(want=OEIS_data("A087803"));
  check_equal(num_smaller_trees_vector(#want+1),
              concat(0,want),
             "num_smaller_trees_vector() vs A087803 data");
}

\\-----------------------------------------------------------------------------
\\ Formula

A347620_by_count(n) =
{
  my(k = A061775(n),
     s = sum(i=1,n, A061775(i)==k));
  A087803(k-1) + s;
}
{
  check_equal(vector(100,n, A347620_by_count(n)),
              A347620_vector(100),
              "A347620_by_count() vs A347620_vector()");
}


\\-----------------------------------------------------------------------------
\\ A347620_vector()

{
  my(triangle=OEIS_data("A061773"),
     inv=A347620_vector(vecmax(triangle)),
     got=vector(#triangle,i, inv[triangle[i]]));
  check_equal(got,[1..#triangle],
             "A347620_vector() vs A061773 data");
}

A061773_row(n) =
{
  my(vpar=vpar_unlabelled_first(n),
     l=List([]));
  until((vpar=vpar_unlabelled_next(vpar)) == 'none,  \\ trees
        listput(l,vpar_to_primecode(vpar)));
  vecsort(Vec(l));
}
{
  my(triangle=concat(vector(11,N, A061773_row(N))),
     hi=vecmax(triangle));
  print("triangle length ",#triangle," max ",hi);
  my(inv=A347620_vector(hi),
     got=vector(#triangle,i, inv[triangle[i]]));
  check_equal(got,[1..#triangle],
             "A347620_vector() vs A061773_row()");

  \\ first of
  my(v=OEIS_data("A005517"));
  for(k=1,#v,
     my(n=v[k]);
     check_equal(#vpar_from_primecode(n-1) < k, 1);
     check_equal(#vpar_from_primecode(n) == k, 1, "A005517 num vertices");
     if(n>#inv, print("A005517 stop at n="n" k="k); break);
     check_equal(inv[n], A087803(k-1) + 1);
  );     

  \\ last of
  my(v=OEIS_data("A005518"));
  for(k=1,#v,
     my(n=v[k]);
     check_equal(#vpar_from_primecode(n) == k, 1, "A005518 num vertices");
     check_equal(#vpar_from_primecode(n+1) != k, 1);
     if(n>#inv, print("A005518 stop at n="n" k="k); break);
     check_equal(inv[n], A087803(k));
  );     
}

inverse_vector_unified(len) =
{
  my(c=List([]),           \\ A209397
     num_trees=List([1]),  \\ A000081
     N_to_pos=List([0,1]));
  vector(len,primecode,
         my(N=#vpar_from_primecode(primecode));
         while(#N_to_pos<N,
               my(k=#num_trees);
               listput(c, sumdiv(k,d, num_trees[d]*d));
               listput(num_trees, sum(i=1,k, c[i]*num_trees[k+1-i]) / k);
               listput(N_to_pos, vecsum(Vec(num_trees)));
               num_trees[#num_trees] == vpar_count_unlabelled(#num_trees) || error();
               \\ print("extend "num_trees" now end="end);
               );
         \\ print("primecode = "primecode"  N="N" into ",Vec(N_to_pos));
         N_to_pos[N]++);
}
{
  check_equal(A347620_vector(10000),
              inverse_vector_unified(10000),
              "A347620_vector() vs inverse_vector_unified()");
}


\\-----------------------------------------------------------------------------
check_equal(b,'b);
check_equal(c,'c);
check_equal(f,'f);
check_equal(n,'n);
check_equal(num_vertices,'num_vertices);
check_equal(idx,'idx);
check_equal(got,'got);
check_equal(ret,'ret);
check_equal(upto,'upto);
check_equal(vec,'vec);
check_equal(vpar,'vpar);
check_equal(want,'want);
print("end a347620-Matula-inverse-test.gp");



\\ num_vertices_vector_by_forfactored(len) =
\\ {
\\   my(ret=vector(len));
\\   forfactored(elem=1,len,
\\      my(n,f); [n,f]=elem;
\\      ret[n] = vecextract(ret,apply(primepi,f[,1])) * f[,2] + 1);
\\   ret;
\\ }
\\ 
\\ num_vertices_vector_by_forfactored(len) =
\\ {
\\   my(ret=Vec([1],len), pi=0);
\\   forfactored(elem=2,len,
\\      my(n,f); [n,f]=elem;
\\      my(num_rows=matsize(f)[1]);
\\      ret[n] = if(num_rows==1 && f[1,2]==1,
\\                  ret[pi++],
\\                  sum(i=1,num_rows, (ret[f[i,1]]-1)*f[i,2]))
\\               + 1);
\\   ret;
\\ }
\\ num_vertices_vector_by_forfactored(len) =
\\ {
\\   my(ret=vector(len),     \\ starting as num_vertices[n] - 1
\\      pi=0);               \\ number of prime n seen so far
\\   forfactored(elem=2,len,
\\      my(n,f); [n,f]=elem;
\\      ret[n] = if(matsize(f)[1]==1 && f[1,2]==1,
\\                  ret[pi++] + 1,                \\ n prime
\\                  vecextract(ret,f[,1])*f[,2])  \\ n composite
\\               );
\\   for(n=1,#ret,ret[n]++);  \\ increment to num_vertices[n]
\\   ret;
\\ }
\\ num_vertices_vector_by_forfactored(len) =
\\ {
\\   my(ret=Vec([1],len),    \\ starting as num_vertices[n] - 1
\\      pi=0);               \\ number of prime n seen so far
\\   forfactored(elem=2,len,
\\      my(n,f); [n,f]=elem;
\\      ret[n] = if(matsize(f)[1]==1 && f[1,2]==1,
\\                  ret[pi++],                                    \\ n prime
\\                  vecextract(ret,f[,1])*f[,2] - vecsum(f[,2]))  \\ n composite
\\               + 1);
\\   ret;
\\ }
\\ num_vertices_vector_by_forfactored(len) =
\\ {
\\   my(ret=Vec([1],len),
\\      pi=0);   \\ number of prime n seen so far
\\   forfactored(elem=2,len,
\\      my(n,f); [n,f]=elem;
\\      ret[n] = if(matsize(f)[1]==1 && f[1,2]==1,
\\                  ret[pi++],                   \\ n prime
\\                  [ret[p]-1|p<-f[,1]]*f[,2])   \\ n composite
\\               + 1);
\\   ret;
\\ }
