\\ Copyright 2019 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(recover,0);
default(strictargs,1);
read("../vpar.gp");
read("../test-helpers.gp");
option_verbose=1;


\\-----------------------------------------------------------------------------

gWtopDistK_by_sum(k) = sum(n=0,2^k-1, x^hammingweight(n));
gWtopDistK(k) = if(k==0, 1, (1+x)*gWtopDistK(k-1));
gWtopDistK(k) = (1+x)^k;
for(k=0,3, print(gWtopDistK(k));  print(gWtopDistK_by_sum(k)));

gWdistK_by_sum(k) =
{
  my(m=vpar_path_lengths_matrix(vpar_make_binomial_tree(2^k)));
  sum(i=1,2^k, sum(j=i+1,2^k, x^m[i,j]));
}
check_equal(gWdistK_by_sum(0), 0);
check_equal(gWdistK_by_sum(1), x);

\\ recurrence per Emeric Deutsch
gWdistK_by_recurrence(k) = \
  if(k==0,0, 2*gWdistK_by_recurrence(k-1) + x*gWtopDistK(k-1)^2);

gWdistK(k) =
{
  sum(i=0,k-1, 2^(k-1-i)*x*gWtopDistK(i)^2);
  x * sum(i=0,k-1, 2^(k-1-i)*(1+x)^(2*i));
  x * (2^k - (1+x)^(2*k)) / (2 - (1+x)^2);
  x * ((1+x)^(2*k) - 2^k) / ((1+x)^2 - 2);
  x * (2^k - (1+x)^(2*k)) / (1 - 2*x - x^2);
}
check_equal(simplify(gWdistK(0)), 0);
check_equal(gWdistK(1), x);
check_equal(gWdistK(2), x*(2 + (1+x)^2));
check_equal(gWdistK(2), x*(2*(1+x)^0 + (1+x)^2));
check_equal(gWdistK(2), x*( ((1+x)^4 - 2^2) / ((1+x)^2 - 2)))
check_equal(gWdistK(3), x*(4 + (1+x)^2*(2 + (1+x)^2)));
check_equal(gWdistK(4), x*(8 + (1+x)^2*(4 + (1+x)^2*(2 + (1+x)^2))));

{
  for(k=0,4,
     print("k="k);
     print("rec  "gWdistK_by_recurrence(k));
     print("sum  "gWdistK_by_sum(k));
     print("func "gWdistK(k));
  );
}
quit

by rows A192020

\\-----------------------------------------------------------------------------
read("../examples/binomial-tree-W.gp");

\\ Or bittest() and sum rather than building any vectors.
\\ The various /2 fractions are correctly handled by GP in its bit-shift
\\ operator "<<", but it's convenient to have 2* so integers throughout and
\\ at the end shift down.
\\
W_by_bittest(n) =
{
  my(L=logint(n+1,2),
     ret = n,
     gc=0);    \\ 2*count of 1-bits seen by g
  for(g=0,L,
     if(bittest(n,g),
        ret += (g-1) << (2*g);
        gc+=2;
        my(kc=0);    \\ 2*count of 1-bits seen by g
        for(k=0,g-1,
           if(bittest(n,k),
              kc+=2;
              ret += (g + k + gc-kc) << (g+k)))));
  ret >> 1;
}
check_equal(vector(100,n,n--; W_by_bittest(n)), \
            vector(100,n,n--; W(n) ));

\\    a  b  c  d  e
\\  a -  1  2  3  4       a * (b c d e)       a * La * (abcde)
\\  b    -  1  2  3       (a b) * (c d e)     b * Lb * (abcde)
\\  c       -  1  2       (a b c) * (d e)     c * Lc * (abcde)
\\  d          -  1       (a b c d) * e
\\  e             -

W_by_bits(n) =
{
  my(k=binary_powers_lowtohigh(n));
  my(ret=0, low=0, orig_n=n);

  while(n,
        my(b=lowest_1_bit(n));
        low += b;
        n -= b;
        ret += low*n + 1/2*b*logint(b,2)*(low + n) + 1/2*b*(1-b);
       );
  ret;
}
check_equal(vector(100,n,n--; W_by_bits(n)), \
            vector(100,n,n--; W(n) ));

W_by_bits(n) =
{
  my(k=binary_powers_lowtohigh(n));
  my(ret=0, low=0, orig_n=n);

  \\ ret = n/2;
  \\ n * (1/2 + 0*sum(i=1,#k, k[i]/2*2^k[i] ))     \\ n*PosPowers(n)
  \\ ;
  while(n,
        my(b=lowest_1_bit(n));
        low += b;
        n -= b;
        ret += low*n + 1/2*b*( 1-b + orig_n*logint(b,2) );
       );
  ret;
}
check_equal(vector(100,n,n--; W_by_bits(n)), \
            vector(100,n,n--; W(n) ));




vector(10,n, PosPowers(n)*n)
\\ not in OEIS: 2, 3, 16, 20, 30, 35, 96, 108, 130

cross(n) =
{
  my(k=binary_powers_hightolow(n));
  sum(i=1,#k,
      sum(j=i+1,#k,
          (k[i]/2 + k[j]/2)*2^(k[i] + k[j])));
}
vector(10,n, cross(n))
\\ not in OEIS: 1, 0, 4, 12, 17, 0, 12, 32

cross(n) =
{
  my(k=binary_powers_hightolow(n));
  sum(i=1,#k,  
      sum(j=1,#k,
          (k[i]/2 + k[j]/2) * 2^(k[i] + k[j])
      ));
}
check_equal(vector(16,n, 2*n*PosPowers(n)), \
            vector(16,n, cross(n) ));

cross(n) =
{
  my(k=binary_powers_hightolow(n));
  sum(i=1,#k,  
      sum(j=1,i-1,
          (k[i]/2 + k[j]/2) * 2^(k[i] + k[j])  ))
  +
  sum(i=1,#k,  
      sum(j=i,#k,
          (k[i]/2 + k[j]/2) * 2^(k[i] + k[j])  ))
  ;
}
check_equal(vector(16,n, 2*n*PosPowers(n)), \
            vector(16,n, cross(n) ));

cross(n) =
{
  my(k=binary_powers_hightolow(n));
  2*sum(i=1,#k,  
      sum(j=1,i-1,
          (k[i]/2 + k[j]/2) * 2^(k[i] + k[j])  ))
  +
  sum(i=1,#k,  k[i] * 4^k[i]  );
}
check_equal(vector(16,n, 2*n*PosPowers(n)), \
            vector(16,n, cross(n) ));

cross(n) =
{
  my(k=binary_powers_hightolow(n));
  sum(i=1,#k,  
      sum(j=1,i-1,
          (k[i]/2 + k[j]/2) * 2^(k[i] + k[j])  ))
  +
  sum(i=1,#k,  k[i]/2 * 4^k[i]  );
}
check_equal(vector(16,n, n*PosPowers(n)), \
            vector(16,n, cross(n) ));

W_by_cross(n) =
{
  my(k=binary_powers_lowtohigh(n));

  n * sum(i=1,#k, k[i]/2*2^k[i])     \\ n*PosPowers(n)
  + n/2
  + sum(i=1,#k,
        -1/2*4^k[i]
        + sum(j=i+1,#k,
              (j-i)*2^(k[i] + k[j])));
}
check_equal(vector(100,n, W(n)), \
            vector(100,n, W_by_cross(n)), \
            "W_by_cross");

Wend_i_part_times_n(n) =
{
  my(k=binary_powers_lowtohigh(n));
  sum(i=1,#k, 
      sum(j=i+1,#k,
          j*2^(k[i] + k[j])))
  +
  sum(i=1,#k, 
      sum(j=1,i,
          j*2^(k[i] + k[j])));
}
check_equal(vector(16,n, Wend_i_part_times_n(n)), \
            vector(16,n, n*Wend_i_part(n) ));

\\ 1101
check_equal(Wend_i_part(13), 33);
check_equal(Wend_i_part(13), 3*2^3 + 2*2^2 + 1*2^0);
\\
\\           2^0        2^2       2^3
\\  1*2^0  1*2^(0+0)  1*2^(2+0)  1*2^(3+0)
\\  2*2^2  2*2^(0+2)  2*2^(2+2)  2*2^(3+2)
\\  3*2^3  3*2^(0+3)  3*2^(2+3)  3*2^(3+3)
\\
\\           2^0        2^2       2^3
\\  1*2^0  1*2^(0+0)  2*2^(2+0)  3*2^(3+0)
\\  2*2^2  1*2^(0+2)  2*2^(2+2)  3*2^(3+2)
\\  3*2^3  1*2^(0+3)  2*2^(2+3)  3*2^(3+3)

\\ matrix(3,3,i,j, (-1)^i*(-1)^j)

Wtop_i_part_times_n(n) =
{
  my(k=binary_powers_lowtohigh(n));
  sum(i=1,#k, 
      sum(j=1,i-1, 
          (#k-i)*2^(k[i] + k[j])))
  +
  sum(i=1,#k, 
      sum(j=i,#k, 
          (#k-i)*2^(k[i] + k[j])));
}
check_equal(vector(16,n, Wtop_i_part_times_n(n)), \
            vector(16,n, n*Wtop_i_part(n) ));

\\ 1101
check_equal(Wtop_i_part(13), 6);
check_equal(Wtop_i_part(13), 0*2^3 + 1*2^2 + 2*2^0);

W_i_part(n) =
{
  my(k=binary_powers_lowtohigh(n));
  sum(i=1,#k,
      sum(j=i+1,#k,
          -i*2^(k[i] + k[j])));
}
W_i_part_by_bits(n) =
{
  my(k=binary_powers_lowtohigh(n));
  my(ret=0, i=1);
  while(n,
        my(b=lowest_1_bit(n));
        b==2^k[i] || error();
        n -= b;
        \\ my(m=n, j=i);
        \\ while(m,
        \\       my(c=lowest_1_bit(m));
        \\       m -= c;
        \\       j++;
        \\       c==2^k[j] || error();
        \\       ret += -i*b*c;
        \\      );
        \\ ret += -i*b*m;
         ret += -i*b*n;
        i++;
       );
  ret;
}
check_equal(vector(16,n, W_i_part_by_bits(n) ), \
            vector(16,n, W_i_part(n)), \
            "W_i_part_by_bits");


W_j_part(n) =
{
  my(k=binary_powers_lowtohigh(n));
  sum(i=1,#k,
      sum(j=i+1,#k,
          j*2^(k[i] + k[j])));
}
W_j_part_by_bits(n) =
{
  my(k=binary_powers_lowtohigh(n));
  my(ret=0, i=1);
  while(n,
        my(b=lowest_1_bit(n));
        b==2^k[i] || error();
        n -= b;
        my(m=n, j=i);
        while(m,
              my(c=lowest_1_bit(m));
              m -= c;
              j++;
              c==2^k[j] || error();
              ret += j*b*c;
             );
        i++;
       );
  ret;
}
check_equal(vector(16,n, W_j_part_by_bits(n) ), \
            vector(16,n, W_j_part(n)), \
            "W_j_part");


W_ij_part(n) =
{
  my(k=binary_powers_lowtohigh(n));
  sum(i=1,#k,
      + sum(j=1,i-1,
            j<i || error();
            (i-j)*2^(k[i] + k[j])));

  \\ my(k=binary_powers_hightolow(n));
  \\ sum(i=1,#k,
  \\     + sum(j=i+1,#k,
  \\           (j-i)*2^(k[i] + k[j])));
}
vector(16,n, W_ij_part(n))
vector(16,n, W_ij_part(n) + n^2)
\\ not in OEIS: 2, 0, 4, 8, 18, 0, 8, 16, 34, 32, 52, 72, 106, 0
\\ not in OEIS: 1, 4, 11, 16, 29, 44, 67, 64, 89, 116, 155, 176, 221, 268, 331, 256

W_ij_part_by_bits(n) =
{
  my(ret=0);
  while(n,
        my(b=lowest_1_bit(n));
        my(m=n);
        while(m,
              m -= lowest_1_bit(m);
              ret += m*b;
             );
        n -= b;
       );
  ret;
}
check_equal(vector(16,n, W_ij_part_by_bits(n) ), \
            vector(16,n, W_ij_part(n)), \
            "W_ij_part");


W_ij_part_by_bits(n) =
{
  my(ret=0, low=0);
  while(n,
        my(b=lowest_1_bit(n));
        low += b;
        n -= b;
        ret += low*n;
       );
  ret;
}
check_equal(vector(16,n, W_ij_part_by_bits(n) ), \
            vector(16,n, W_ij_part(n)), \
            "W_ij_part");


quit

\\-----------------------------------------------------------------------------

gOnes(x) = 1/(1-x);
gN(x) = x/(1-x) * gOnes(x);
gOddN(x) = 1/2*(gN(x) - gN(-x));
check_equal(gOddN(x), (x+x^3)/(1-x^2)^2);
print("odds ",test_gf_terms(gOddN(x), 50));
gNpm(x) = (-1+x)*gN(x^2);
test_gf_terms(gNpm(x), 50)

gbit(k) = x^(2^k) / (1-x) / (1 + x^(2^k));
my(k=2); test_gf_terms(2^k* ( k/2 + 1/(1+x^(2^k)) ) * gbit(k),  64)

quit


\\-----------------------------------------------------------------------------

Wend_i_part_by_mask(n) =
{
  my(ret=0, b=1);
  while(bitand(n,-b),
        if(bitand(n,b),
           ret += bitand(n,-b));
       b<<=1);
  ret;
}
vector(100,n,n--; Wend_i_part(n)) == \
vector(100,n,n--; Wend_i_part_by_mask(n))  || error();

\\-----------------------------------------------------------------------------

\\ 0 0000
\\   0001
\\   0010  -1/2*1
\\   0011  +1/2*1
\\ 4 0100  -1/2*2  -1/2 * 1 *2   = -2
\\   0101  +1/2*2  -1/2 * 1 *2
\\   0110          +1/2 * 1 *2
\\   0111          +1/2 * 1 *2
\\
\\ -1/2*2 * 2 / 4 == -1/2
\\ -1/2*1 * 2 / 4 
\\ k=2; k/2 == 1
\\ k=2; 2^k * k/2 == 4
\\ k=2; k/2 == 2
\\ 2^2 * 2/2 -1 -2 == 1

gOnes(x) = 1/(1-x);
gN(x) = x/(1-x) * gOnes(x);
gNpm(x) = (-1+x)*gN(x^2);
test_gf_terms(gNpm(x), 16)
k=1
test_gf_terms(1/2*gNpm(x^(2^k)) * (1 - x^(2^k))/(1-x),  32)
quit

\\-----------------------------------------------------------------------------

expand(g) = 2*(x+1)*subst(g,'x,'x^2) + x*(1+x^2)/(1-x^2)^2;

g0 = expand(0)
g1 = expand(g0)
(g1 - g0) * (1-x^4)^2
g2 = expand(g1)
(g2 - g1 - g0) * (1-x^8)^2
quit

gWend_i_part(k_limit) = x*ghammingweight(k_limit)' - gWtop_i_part(k_limit);

gWend_i_part(8) + O(x^256) == sum(n=0,255, Wend_i_part(n)*x^n)   || error();


\\-----------------------------------------------------------------------------
gbit(k) = x^(2^k) / (1-x) / (1 + x^(2^k));

gOnes(x) = 1/(1-x);
test_gf_terms(gOnes(x),10)
gN(x) = x/(1-x) * gOnes(x);
test_gf_terms(gN(x),10)

\\  00   0
\\  01
\\  10
\\  11   
\\ 100   4
\\ 101   5
\\ 110
\\ 111   
gOddN(x) = 1/2*(gN(x) - gN(-x));
check_equal(gOddN(x), (x+x^3)/(1-x^2)^2);
print("odds ",test_gf_terms(gOddN(x), 10));

print("block ", (1 - x^4)/(1-x));

\\ A298011
gWend_i_term(k) = 2^k * gOddN(x^2^k)  *(1 - x^2^k)/(1 - x);
test_gf_terms(gWend_i_term(0), 16)
test_gf_terms(gWend_i_term(1), 16)
test_gf_terms(gWend_i_term(2), 16)

test_gf_terms(sum(k=0,4,gWend_i_term(k)), 16)

gWend_i_term2(k) = 2^k * x^2^k*(1+x^2^(k+1))/(1-x^2^(k+1))^2 *(1 - x^2^k)/(1 - x);
check_equal(gWend_i_term2(4), gWend_i_term(4));

gWend_i_term2(k) = 2^k * (1+x^2^(k+1))/(1-x^2^(k+1))   * x^2^k/(1 - x)/(1+x^2^k);
check_equal(gWend_i_term2(4) == gWend_i_term(4), 1);

gWend_i_term2(k) = 2^k * (-1 + 2/(1-x^2^(k+1)) )  * x^2^k/(1 - x)/(1+x^2^k);
check_equal(gWend_i_term2(4) == gWend_i_term(4), 1);

gWend_i_term2(k) = 2^k * (-1 + 2/(1-x^2^(k+1)) )  * gbit(k);
check_equal(gWend_i_term2(4) == gWend_i_term(4), 1);

gWend_i_term2(k) = 2^k * (-1 + ( 1+x^2^k)/(1-x^2^k) /(1+x^2^k)   + 1/(1+x^2^k) )  * gbit(k);
check_equal(gWend_i_term2(4) == gWend_i_term(4), 1);

gWend_i_term2(k) = 2^k * (-1 + ( 1 + x^2^k)/(1-x^2^(k+1))   + 1/(1+x^2^k) )  * gbit(k);
check_equal(gWend_i_term2(4) == gWend_i_term(4), 1);

test_gf_terms(sum(k=0,4,gWend_i_term2(k)), 16)

k/2 - k*1/2
k/2 + k*1/2

1+k/2 - k*1/2
1+k/2 + k*1/2
quit


\\-----------------------------------------------------------------------------

gWend_i_part(k_limit) =
{
  sum(k=0,k_limit,
      x * gbit(k)'
      - x/(1-x) * gbit(k) 
      + k/2*2^k * gbit(k))
}
gWend_i_part(8) + O(x^256) == sum(n=0,255, Wend_i_part(n)*x^n)   || error();

gWend_i_part(k_limit) =
{
  sum(k=0,k_limit,
      x*gbit(k)^2
      + ( 2^k*x^(2^k) - (2^k-1)*x^(2^k+1) ) / (1-x)^2 / (1+x^2^k)^2
      - x/(1-x) * gbit(k) 
      + k/2*2^k * gbit(k));
}
gWend_i_part(8) + O(x^256) == sum(n=0,255, Wend_i_part(n)*x^n)   || error();

gWend_i_part(k_limit) =
{
  sum(k=0,k_limit,
      (x + ( 2^k - (2^k-1)*x ) /x^(2^k) ) * gbit(k)^2
      - x/(1-x) * gbit(k) 
      + k/2*2^k * gbit(k));
}
gWend_i_part(8) + O(x^256) == sum(n=0,255, Wend_i_part(n)*x^n)   || error();

gWend_i_part(k_limit) =
{
  sum(k=0,k_limit,
      2^k*  ( k/2 + 1/(1+x^(2^k)) ) * gbit(k));
}
gWend_i_part(8) + O(x^256) == sum(n=0,255, Wend_i_part(n)*x^n)   || error();

gbit(3) == x^(2^3) / (1-x) / (1+x^2^3)
gbit(3)'*(1-x)^2*(1+x^2^3)^2

gbit(3)' == (8*x^7 - 7*x^8 + x^(2^4)) / (1-x)^2 / (1+x^2^3)^2
x*gbit(3)' == \
x*gbit(3)^2 + ( 8*x^(2^3) - 7*x^(2^3+1) ) / (1-x)^2 / (1+x^2^3)^2

\\ my(k=3); k/2*2^k * gbit(k)*(1+x^2^3)*(1-x)


my(k=0); (2^k*  ( k/2 + 1/(1+x^(2^k)) ) * gbit(k) + O(x^16))
my(k=0); Vec(2^k*  ( k/2 + 1/(1+x^(2^k)) ) * gbit(k) + O(x^16))

my(k=1); (2^k*  ( k/2 + 1/(1+x^(2^k)) ) * gbit(k) + O(x^16))
my(k=1); Vec(2^k*  ( k/2 + 1/(1+x^(2^k)) ) * gbit(k) + O(x^16))

my(k=2); (2^k*  ( k/2 + 1/(1+x^(2^k)) ) * gbit(k) + O(x^32))
my(k=2); Vec(2^k*  ( k/2 + 1/(1+x^(2^k)) ) * gbit(k) + O(x^32))

my(k=3); (2^k*  ( k/2 + 1/(1+x^(2^k)) ) * gbit(k) + O(x^32))
my(k=3); Vec(2^k*  ( k/2 + 1/(1+x^(2^k)) ) * gbit(k) + O(x^32))
quit
\\         1100          -0100        10000        -1000        +10100       -1100            11000
\\   1  2  3  4   5   6  7   8              12          15
\\   1,-1, 2,-2,  3, -3, 4, -4,  5, -5,  6, -6,  7, -7,  8
\\ [    3, 3,-2, -2,  5, 5, -4, -4,  7,  7, -6, -6,  9,  9
\\ [          8,  8,  8, 8, -4, -4, -4, -4, 12, 12, 12, 12, -8, -8, -8, -8, 16, 16, 16, 16, -12, -12, -12, -12, 20,
\\                          20, 20, 20, 20, 20, 20, 20, 20, -8, -8, -8, -8, -8, -8, -8, -8


quit


gWend(k_limit) = 
{
  sum(k=0,k_limit,
      2^k*  ( k + 1/(1+x^(2^k)) ) * gbit(k));
}

gWend(8) + O(x^256) == sum(n=0,255, Wend(n)*x^n)   || error();


my(k=0); (2^k*  ( k + 1/(1+x^(2^k)) ) * gbit(k) + O(x^16))
my(k=0); Vec(2^k*  ( k + 1/(1+x^(2^k)) ) * gbit(k) + O(x^16))

my(k=1); (2^k*  ( k + 1/(1+x^(2^k)) ) * gbit(k) + O(x^16))
my(k=1); Vec(2^k*  ( k + 1/(1+x^(2^k)) ) * gbit(k) + O(x^16))

my(k=2); (2^k*  ( k + 1/(1+x^(2^k)) ) * gbit(k) + O(x^32))
my(k=2); Vec(2^k*  ( k + 1/(1+x^(2^k)) ) * gbit(k) + O(x^32))

my(k=3); (2^k*  ( k + 1/(1+x^(2^k)) ) * gbit(k) + O(x^32))
my(k=3); Vec(2^k*  ( k + 1/(1+x^(2^k)) ) * gbit(k) + O(x^32))

\\ repeat 1 and -1
my(k=3); 1/(1+x^(2^k)) == (1 - x^(2^k))/(1-x^(2^(k+1)))  || error();
my(k=3); (1/(1+x^(2^k)) + O(x^32))
my(k=3); Vec(1/(1+x^(2^k)) + O(x^32))

\\ 1, 0, 0, 0, 0, 0, 0, 0, -1, -1, -1, -1, 0, 0, 0, 0, 0, 1, 2, 3, 3, 2, 1, 0, 0, 0, -1, -3, -6, -8, -8, -6
my(k=2); (1/(1+x^(2^k) * gbit(k)) + O(x^32))
my(k=2); Vec(1/(1+x^(2^k) * gbit(k)) + O(x^32))

\\         1100          -0100        10000        -1000        +10100       -1100            11000
\\   1  2  3  4   5   6  7   8              12          15
\\   1,-1, 2,-2,  3, -3, 4, -4,  5, -5,  6, -6,  7, -7,  8
\\ [    4, 4,-2, -2,  6, 6, -4, -4,  8,  8, -6, -6, 10, 10
\\ [         12, 12, 12,12, -4, -4, -4, -4, 16, 16, 16, 16, -8,-8,-8,-8, 20,20,20,20, -12,-12,-12,-12, 24,24,24,24]
\\                          32, 32, 32, 32, 32, 32, 32, 32, -8, -8, -8, -8, -8, -8, -8, -8
Wend(8) == 20
32 -4 -4 -4 == 20
3*2^2 == 12

12+6+4 == 22
\\ 7    100
\\      110
\\     1100
gWend(3) +O(x^32)
quit


\\-----------------------------------------------------------------------------

read("../examples/binomial-tree-W.gp");


vector(20,n, Wend(n+1)-Wend(n))
\\ not in OEIS: 2,3,2,5,2,7,-2,9,2,11,-6,13,2,15,-18,17,2,19,-14,21

gPosPowers_term(k) = k/2 * 2^k * x^(2^k) * ( 1 - x^(2^k) ) / (1-x) / (1 - x^(2^(k+1)));
gPosPowers_term(k) = k/2 * 2^k * x^(2^k) / (1-x) / (1 + x^(2^k));
gPosPowers(k_limit) = sum(k=0,k_limit, gPosPowers_term(k));

print(  gPosPowers_term(1) + O(x^16) );

{
  my(limit=256);
  check_equal(gPosPowers(8) + O(x^limit),
              sum(n=0,limit-1, PosPowers(n)*x^n) + O(x^limit));
}


(1-x^4)/(1-x) + O(x^16)
x^1*(1-x^8)/(1-x^2) /(1-x^16) + O(x^32)

\\ 0   1   2   3   4   5   6   7   8   9  10  11  12  13  14  15
\\ 0   0   0   0   0   0   0   0   1   1   1   1   1   1   1   1
\\ 0   0   0   0   1   1   1   1   0   0   0   0   1   1   1   1
\\ 0   0   1   1   0   0   1   1   0   0   1   1   0   0   1   1
\\ 0   1   0   1   0   1   0   1   0   1   0   1   0   1   0   1

\\ 1 where both bits k and j are set, k > j
gWbit_pair(k,j) =
{
  k>j || error();
  (   x^(2^j) - x^(2^(j+1)) )  / (1-x)
  * ( x^(2^k) - x^(2^(k+1)) )
  / (1-x^(2^(j+1)))
  / (1-x^(2^(k+1)));
}
Wend_i_part_pair(n,k,j) = bittest(n,k) && bittest(n,j);

{
  my(limit=1024);
  for(k=0,6,
     for(j=0,k-1,
        check_equal(gWbit_pair(k,j) + O(x^limit),
                    sum(n=0,limit-1, Wend_i_part_pair(n,j,k)*x^n) + O(x^limit))));
}

{
  g =
  sum(k=3,3,
      0*(x^(2^k) - x^(2^(k+1))) / (1-x)  / (1 - x^(2^(k+1)))
      + sum(j=1,1, gWbit_pair(k,j) ))
  + O(x^16)
}
g
g=Pol(g);
Vecrev(g)

\\ {
\\   g = gWbit_pair(1,1)
\\   + O(x^16)
\\ }
\\ g
\\ g=Pol(g);
\\ Vecrev(g)


gOnes_at(k) = (x^(2^k) - x^(2^(k+1))) / (1-x)  / (1 - x^(2^(k+1)));
{
  my(limit=1024);
  for(k=0,6,
     check_equal(gOnes_at(k) + O(x^limit),
                 sum(n=0,limit-1, bittest(n,k)*x^n) + O(x^limit),
                 "gOnes_at() vs bittest()"));
}

\\ A006520 Partial sums of A006519.
\\ A006519 lowest 1-bit of n.

\\ per Jorg Arndt
A006519(n) = if(n==0,0, 1 << valuation(n,2));
my(v=OEIS_samples("A006519")); check_equal(vector(#v,n, A006519(n)), v);

\\ per Ralf Stephan
gA006519(k_limit) = sum(k=0,k_limit, 2^k * x^(2^k)/(1 - x^(2^(k+1))));
{
  my(limit=1024);
  check_equal(gA006519(10) + O(x^limit),
              sum(n=0,limit-1, A006519(n)*x^n) + O(x^limit),
              "gA006519() vs A006519()");
}

A006520(n) = sum(i=1,n, A006519(i));
my(v=OEIS_samples("A006520")); check_equal(vector(#v,n, A006520(n)), v);

gA006520(k_limit) = gA006519(k_limit) / (1-x);
{
  my(limit=1024);
  check_equal(gA006520(10) + O(x^limit),
              sum(n=0,limit-1, A006520(n)*x^n) + O(x^limit),
              "gA006520() vs A006520()");
}

\\ k==j comes out with + and -
\\ gA006520_other(k_limit) =
\\ {
\\   sum(k=0,k_limit,
\\       + 2^k * gOnes_at(k)
\\       + 2^k * sum(j=0,k, gWbit_pair(k,j) ));
\\ }
\\ {
\\   my(limit=1024);
\\   check_equal(gA006520(10) + O(x^limit),
\\               gA006520_other(10) + O(x^limit));
\\ }

gA006520_other(k_limit) =
{
  sum(k=0,k_limit,
      2^k * (x^(2^k) - x^(2^(k+1)))
      * (
         + 1
         + sum(j=0,k,
               (   x^(2^j) - x^(2^(j+1)) )
               / (1-x^(2^(j+1)))
              )
        )
      / (1 - x^(2^(k+1))) )
  / (1-x);
}
{
  my(limit=1024);
  check_equal(gA006520(10) + O(x^limit),
              gA006520_other(10) + O(x^limit));
}

\\-----------
gWend_i_part(k_limit) =
{
  sum(k=0,k_limit,
      + 2^k * (gOnes_at(k)
               + sum(j=0,k-1, gWbit_pair(k,j) )));
}

\\ Wend_i_part  0,1,2,5,4,9,10,17,8,17,18,29,20,33,34,49,16,  (A298011)

g=gWend_i_part(5) + O(x^16)
g=Pol(g);
Vecrev(g)

{
  my(limit=1024);
  check_equal(gWend_i_part(10) + O(x^limit),
              sum(n=0,limit-1, Wend_i_part(n)*x^n) + O(x^limit),
              "gWend_i_part() vs Wend_i_part()");
}

gOnes(x) = 1/(1-x);
{
  my(limit=1024);
  check_equal(gOnes(x) + O(x^limit),
              sum(n=0,limit-1, x^n) + O(x^limit),
              "gOnes()");
}


gN(x) = x/(1-x)^2;
gN(x) + O(x^256) == sum(n=0,255, n*x^n)   || error();

gN(x) = x*subst(gOnes('x)', 'x, x);
print("gN  ",gN(x));
{
  my(limit=1024);
  check_equal(gN(x) + O(x^limit),
              sum(n=0,limit-1, n*x^n) + O(x^limit),
              "gN()");
}

gNat2N(x) = gN(x^2);
print("gNat2N  ",gNat2N(x));
{
  my(limit=1024);
  check_equal(gNat2N(x) + O(x^limit),
              sum(n=0,limit-1, n*x^(2*n)) + O(x^limit),
              "gN()");
}
{
  \\ gets different sign in num/den of RFRAC, so == but not ===
  check_equal(x,'x, "x");
  my(g=2*x^3 * subst(1/(x^2 - 2*x + 1), 'x, 'x^2) + x/(1-x^2),
     h=x*(1+x^2)/(1-x^2)^2);
  g=2*x^3 * 1/(x^4 - 2*x^2 + 1) + x/(1-x^2);

  check_equal(g==h, 1, "==");
  \\ check_equal(g, h, "g eq h");
}
{
  check_equal(2*x*gNat2N(x) + x/(1-x^2)
              ==
              x*(1+x^2)/(1-x^2)^2,
              1);
}

{
  \\ new low 0-bit, moves pos powers up
  \\ new low 1-bit, increase +1 at every bit position, is +2n, then 1 itself
  \\ a(2*n) = 2*a(n)
  \\ a(2*n + 1) = a(2*n) + 2*n + 1   \\ David A. Corneth

  my(limit=1024,
     g=gWend_i_part(10)   + O(x^limit));
  \\ Robert I. in A298011
  check_equal(g,
              2*(x+1)*subst(g,'x,'x^2) + x*(1+x^2)/(1-x^2)^2   + O(x^limit),
              "gWend_i_part() satisfies");
}

gWend_i_part2(k_limit) =
{
  sum(k=0,k_limit,
      2^k
      * (1 + sum(j=0,k-1, 
                 ( x^(2^j) - x^(2^(j+1)) )
                 / (1 - x^(2^(j+1)))
         )
        )
      * ( x^(2^k) - x^(2^(k+1)) )
      / (1 - x^(2^(k+1)))
  ) / (1-x);
}
check_equal(gWend_i_part(10) == gWend_i_part2(10), 1);

(1+x^2)*(1+x^4)*(1+x^8)

(x-x^2)/(1-x^2) + (x^2-x^4)/(1-x^4)
test_gf_terms((x-x^2)/(1-x^2) + (x^2-x^4)/(1-x^4) + (x^4-x^8)/(1-x^8),16)

my(k=4); (1-x^(2^k))*sum(j=0,k-1, ( x^(2^j) - x^(2^(j+1)) ) / (1 - x^(2^(j+1))))

my(g=1); 2*(x+1)*subst(g,'x,'x^2) + x*(1+x^2)/(1-x^2)^2 
test_gf_terms(x*(1+x^2)/(1-x^2)^2 , 16)  \\ n*x^n at odd n

2*(x+1)*1 + x*(1+x^2)/(1-x^2)^2 
2*(x+1)*( 2*(x^2+1)*1 + x^2*(1+x^4)/(1-x^4)^2  )  + x*(1+x^2)/(1-x^2)^2 
(2*(x^2+1)*1)^2 + 2*(x+1)* x^2*(1+x^4)/(1-x^4)^2    + x*(1+x^2)/(1-x^2)^2 

\\ A000120 count 1-bits
gWend_i_part_k_only(k_limit) =
{
  sum(k=0,k_limit,
      ( x^(2^k) - x^(2^(k+1)) )
      / (1 - x^(2^(k+1)))
  ) / (1-x);
}
print("A000120 ",test_gf_terms(gWend_i_part_k_only(8), 32));

\\ A088705 count low 0s
gWend_i_part_k_only_nocumul(k_limit) =
{
  sum(k=0,k_limit,
      ( x^(2^k) - x^(2^(k+1)) )
      / (1 - x^(2^(k+1)))
  );
}
print("A088705 ",test_gf_terms(gWend_i_part_k_only_nocumul(8), 32));

gWend_i_part_j_only(k_limit) =
{
  sum(k=0,k_limit,
  1 + sum(j=0,k-1, 
          ( x^(2^j) - x^(2^(j+1)) )
          / (1 - x^(2^(j+1))) )
);
}
test_gf_terms(gWend_i_part_j_only(8), 32) 
\\ not in OEIS: 9, 8, -1, 8, -9, 8, -1, 8, -16, 8, -1, 8, -9, 8, -1, 8, -22, 8, -1, 8, -9, 8, -1, 8, -16, 8, -1, 8, -9, 8, -1, 8

gWend_i_part_j_only(k_limit) =
{
  sum(k=0,k_limit,
   sum(j=0,k-1, 
          ( x^(2^j) - x^(2^(j+1)) )
          / (1 - x^(2^(j+1))) )
);
}
test_gf_terms(gWend_i_part_j_only(8), 32) 
\\ not in OEIS: 0, 8, -1, 8, -9, 8, -1, 8, -16, 8, -1, 8, -9, 8, -1, 8, -22, 8, -1, 8, -9, 8, -1, 8, -16, 8, -1, 8, -9, 8, -1, 8

\\-----------------------------------------------------------------------------

\\ 1-bits above is n sans its low bits, which means successive 1,2,3,...,
\\ but duplicated 2^k times each for the bits below stepping.
\\
\\ The simplest is bit position k=0 where want n*x^n at odd n.
\\ Proceeding then in some usual ways,
\\
\\ For higher k, the 1-bit and above is an odd spread out,
\\
\\  4  00100   1*2^2
\\     ...
\\  7  00111   1*2^2
\\ 12  01100   3*2^2
\\     ...
\\ 20  10100   5*2^2
\\     ...
\\ 28  11100   7*2^2
\\
\\ Such a spread is by substituting x^(2^k) into gOddN, for a term at 4,12,20 etc.
\\
gOddN(x^(2^2)) + O(x^32) == 1*x^4 + 3*x^12 + 5*x^20 + 7*x^28  || error();

\\ Each term is to repeat for all the 1-bits such as 4 through 7.  This
\\ means a multiply by
\\     1 + x + x^2 + x^3   = (1 - x^4) / (1-x)
\\

print("end bits");
quit

