\\ Copyright 2018, 2020 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(recover,0)
read("../vpar.gp");
read("../vpar-wip.gp");
read("../test-helpers.gp");
read("../examples/integral-tree-diameter.gp");

print("===============================================================\n\n");

option_verbose = 0;
option_progress = 1;

is_integral_poly(poly) =
{
  my(m=factor(poly));
  for(i=1,matsize(m)[1],
     if(poldegree(m[i,1])>1, return(0)));
  1;
}
is_integral_spectrum(vpar) = \
  is_integral_poly(charpoly(vpar_adjacency_matrix(vpar)));

concat_strs(vec) =
{
  if(type(vec)=="t_VEC",
     vec=apply(concat_strs,vec);
     vec=concat(vec));
  vec;
}
check_equal(concat_strs("1"), "1");
check_equal(concat_strs(["1","2"]), "12");
check_equal(concat_strs(["1",["2","3"]]), "123");

\\-----------------------------------------------------------------------------
data_size = 29;
data_n        = vector(data_size,i,-1);   \\ num vertices
data_name     = vector(data_size);        \\ string
data_depths   = vector(data_size);        \\ depths vector
data_spectrum = vector(data_size);        \\ non-negative roots with repetition
data_vpar     = vector(data_size);        \\ vpar vector

data_list_set(key,n,d,name,depths,spectrum) =
{
  data_n[key]==-1 || error();
  n>=0 || error();
  my(zero=Vecsmall("0")[1]);

  depths=concat_strs(depths);
  type(depths)=="t_STR" || error();
  depths=Vec(Vecsmall(depths));
  for(v=1,#depths, depths[v] -= zero);
  #depths == n || error("n=",n," depths length ",#depths);

  for(i=1,#spectrum,
     if(type(spectrum[i])!="t_VEC", spectrum[i]=[spectrum[i]]));
  if(#spectrum,
     spectrum=concat(spectrum));
  for(i=1,#spectrum,
     type(spectrum[i])=="t_INT" || error());

  data_n[key]        = n;
  data_name[key]     = name;
  data_depths[key]   = depths;
  data_spectrum[key] = spectrum;
  data_vpar[key]     = vpar_make_preorder_depths(depths);
}
rep(x,len) = vector(len,i,x);

{
  \\            key  n   diam  name      depths        spectrum
  data_list_set(1,   1,   0,  "K[1]",      "0",        [0]);
  data_list_set(2,   2,   1,  "K[2]",      "01",       [1]);
  data_list_set(3,   5,   2,  "K[1,4]",    "01111",    [2, rep(0,3)]);
  data_list_set(4,   6,   3, "K[1,2]~K[1,2]",  "012211", [2,1,rep(0,2)]);
  data_list_set(5,   7,   4,  "SK[1,3]",   "0121212",  [2, rep(1,2), 0]);
  data_list_set(6,   10,  2,  "K[1,9]",    "0111111111", [3, rep(0,8)]);

  data_list_set(7,   14,  3,  "K[1,6]~K[1,6]",
                "01222222111111", [3,2,rep(0,10)]);

  data_list_set(8,   17,  2,  "K[1,16]",   "01111111111111111", [4,rep(0,15)]);
  data_list_set(9,   17,  4,  "SK[1,8]",   "01212121212121212", [3,rep(1,7),0]);
  data_list_set(10,  17,  4,  "K[1,7]~SK[1,4]",
                "01222222212121212", [3,2,rep(1,3),rep(0,7)]);

  data_list_set(11,  19,  4,  "K[1,5]~SK[1,6]",
                "0122222121212121212", [3,2,rep(1,5),rep(0,5)]);
  data_list_set(12,  25,  5,  "T(2)*T(3,4)~T(3,1)",
                "0123333233332333322121212", [3,rep(2,3),rep(1,3),rep(0,11)]);

  data_list_set(13,  26,  2,  "K[1,25]", 
                "01111111111111111111111111", [5, rep(0,24)]);
  data_list_set(14,  26,  4,  "T(5,4)",   
                "01222212222122221222212222", [3,rep(2,4),rep(0,16)]);
  data_list_set(15,  26,  3,  "K[1,12]~K[1,12]",
                "01222222222222111111111111", [4,3,rep(0,22)]);

  data_list_set(16,  31,  4,  "SK[1,15]",
                "0121212121212121212121212121212", [4,rep(1,14),0]);
  data_list_set(17,  31,  6,  "T(1)*T(2,4)*T(1,1,3)*T(2,3,1)",
                "0123331232323123232312222122221",
                [3,rep(2,4),rep(1,5),rep(0,11)]);
  data_list_set(18,  31,  6,  "T(3,4)*T(3,1,3)",
                "0123331233312333122221222212222", [3, rep(2,5),1,rep(0,17)]);

  data_list_set(19,  35,  4,  "K[1,13]~SK[1,10]",
                "01222222222222212121212121212121212",
                [4,3,rep(1,9),rep(0,13)]);

  data_list_set(20,  37,  2,  "K[1,36]",
                "0111111111111111111111111111111111111",
                [6,rep(0,35)]);
  data_list_set(21,  37,  4,  "K[1,11]~SK[1,12]",
                "0122222222222121212121212121212121212", 
                [4,3,rep(1,11),rep(0,11)]);
  data_list_set(22,  37,  6,  "T(1,5,3,1)",
                "0123232312323231232323123232312323231",
                [3,rep(2,4),rep(1,11),rep(0,5)]);
  
  data_list_set(23,  37,  6,  "T(1,4)*T(2,1,3)*T(3,3,1)",
                "0123331233312323231232323123232312222",
                [3,rep(2,5),rep(1,7),rep(0,11)]);

  data_list_set(24,  42,  3,  "K[1,20]~K[1,20]",
                ["0",rep("1",20),"1",rep("2",20)],
                [5,4,rep(0,38)]);

  data_list_set(25,  46,  4,  "T(6,4)*T(1,14)",
                ["01",rep("2",14),rep("12222",6)],
                [4,3,rep(2,5),rep(0,32)]);

  data_list_set(26,  49,  4,  "SK[1,24]",
                ["0",rep("12",24)],
                [5,rep(1,23),0]);

  data_list_set(27,  50,  2,  "K[1,49]",
                ["0",rep("1",49)],
                [7,rep(0,48)]);
  data_list_set(28,  50,  4,  "T(4)*T(9,4)",
                ["0",rep("12222",9),rep("1",4)],
                [4,rep(2,8),1,rep(0,30)]);

  data_list_set(29, 0, -1, "empty", "", []);
}

{
  my(verbose=0);
  for(i=1,data_size,
     if(verbose,print("i="i));
     my(n=data_n[i],
        vpar=data_vpar[i],
        p=vpar_adjacency_poly(vpar),
        roots=Vec(polrootsreal(p)),
        nonneg_roots=select(n->n>=0,roots),
        spectrum=vecsort(data_spectrum[i]));
     if(verbose,print(spectrum));
     if(verbose,print(nonneg_roots));
     check_equal(nonneg_roots==spectrum, 1,
                 Str("i="i" spectrum"));
     check_equal(is_integral_poly(p), 1,
                 Str("i="i" is integral"));
     check_equal(#vpar,n);
     if(verbose,print());
  );
}

histogram=vector(vecmax(data_n)+1);
for(i=1,data_size, histogram[data_n[i]+1]++);

{
  check_equal(histogram,
[
 1,1,1,0,0,1,1,1,0,0,1,0,0,0,1,0,0,3,0,1,0,0,0,0,0,1,3,0,0,0,0,3,0,0,0,1,0,
 4,0,0,0,0,1,0,0,0,1,0,0,1,2
], "histogram");
}
{
  \\ star of i^2 leaves and 1 centre
  for(i=1,#histogram,
     my(n=i^2+1);
     if(1+n>#histogram,break);
     check_equal(histogram[1+ n] >= 1, 1));
}

\\ Brouwers section 2 Discussion num free trees of 50 vertices
check_equal(vpar_count_free(50), 10545233702911509534);


\\-----------------------------------------------------------------------------
progress("end");
