\\ Copyright 2017, 2018 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(recover,0)
default(strictargs,1)
default(parisizemax,300*10^6);
read("../vpar.gp");
read("../vpar-wip.gp");
read("memoize.gp");
setrand(getwalltime()%2^32);


check_count = 0;
check_exact(got,want,name="") =
{
  check_count++;
  if(got!= want,
     print("wrong: "name);
     print("got  "got);
     print("want "want);
     quit(1),
     option_verbose,
     print("ok "check_count" "name));
}


\\ Return a vector of all partitions of 1..n.
\\ Vector elements are like [[1,3],[2]].
\\ Each entry in the element is a vector of some of 1..n.
\\ The entries are in order of ascending minimum number.
\\ The first entry contains 1, second contains smallest not in the first, etc.
\\
make_labelled_partitions(n) =
{
  my(ret=[ [] ]);
  for(d=1,n,
     my(new_ret=List([]));
     for(i=1,#ret,
        my(old=ret[i]);
        for(pos=1,#old,  \\ d extending one of the positions in old
           my(new=old);
           new[pos]=concat(new[pos],[d]);
           listput(new_ret,new));
        listput(new_ret,concat(old,[[d]])));  \\ d new set
     ret=new_ret);
  Vec(ret);
}
check_exact(make_labelled_partitions(0), [ [] ]);
check_exact(make_labelled_partitions(1), [ [[1]] ]);
check_exact(make_labelled_partitions(2), [ [[1,2]], [[1],[2]] ]);
check_exact(make_labelled_partitions(3), \
            [ [[1,2,3]], [[1,2],[3]], [[1,3],[2]], [[1],[2,3]], [[1],[2],[3]] ]);
\\ A000110 Bell numbers, partition n labelled elements
check_exact(vector(8,n,n--; #make_labelled_partitions(n)), \
            [1, 1, 2, 5,  15, 52, 203, 877]);

labelled_partition_is_monotone(elem) =
{
  my(mins=apply(vecmin,elem),
     maxs=apply(vecmax,elem));
  mins==vecsort(mins) || error();
  maxs==vecsort(maxs);
}
make_labelled_monotone_partitions(n) = \
  select(labelled_partition_is_monotone, make_labelled_partitions(n));

check_exact(vector(9,n,n--; #make_labelled_monotone_partitions(n)), \
            [1, 1, 2, 4, 9, 22, 58, 164, 496]);

A092920_samples = {[
  1, 1, 2, 4, 9, 22, 58, 164, 496, 1601, 5502, 20075, 77531, 315947,
  1354279, 6087421, 28611385, 140239297, 715116827, 3785445032,
  20760746393, 117759236340, 689745339984, 4165874930885, 25911148634728,
  165775085602106
]};


\\-----------------------------------------------------------------------------

c(n,m,func) =
{
  if(n<0 || m<1 || m>n, error());
  if(n<=1 || m==n, return(func(n)));
  sum(t=min(2,m), m,
        func(t)
        * binomial(max(0,m-2),max(0,t-2))
        * sum(h=m-t+1, n-t, c(n-t,h,func)));
}
a(n,func) = if(n==0,1, sum(m=1,n, c(n,m,func)));


\\ 1,1
\\ 2,1 2,2
\\ 3,1 3,2 3,3
\\ 4,1 4,2 4,3 4,4
\\
\\ 4,3 t=2 n-t=2 1+m-t=2 elem 2,2
\\     t=3 n-t=1 1+m-t=1 elem 1,1
\\ recursing n-t and 1+m-t diagonal with t   binomials as m,t
\\ 10,7 t=2 n-t=8 1+m-t=6 elem 8,6
\\      t=7 n-t=3 1+m-t=1 elem 3,1
\\
\\ c(n,m) is the number of strongly monotone partitions with first block
\\ maximum >= m.
c(n,m,func) =
{
  if(n<0 || m<1 || m>n, error());
  if(n<=1 || m==n, return(func(n)));
  c(n,m+1,func)
  + sum(t=min(2,m), m,
          func(t) * binomial(max(0,m-2),max(0,t-2)) * c(n-t,1+m-t,func));
}
a(n,func) = if(n==0,1, c(n,1,func));

c=memoize(c);

check_exact(vector(#A092920_samples,n,n--; a(n,n->1)), A092920_samples);

quit

\\-----------------------------------------------------------------------------

\\ for(n=1,200, print(a(n,vpar_count)*1.0/vpar_count(n,1)));
\\ quit

\\ monotone forests
vector(15,n,n--; a(n,n->vpar_count(n)))

\\ root=1 trees, so fixed min (or max or similar) as root of each
\\ not in OEIS: 2, 6, 28, 194, 1850, 22662, 339392, 6006566, 122628440, 2836595794
vector(15,n,n--; a(n,n->vpar_count(n,-1)))

\\ A092920 strongly monotone partitions
\\ 1, 1, 2, 4, 9, 22, 58, 164, 496, 1601
vector(15,n,n--; a(n,n->1))

\\ not in OEIS: 2, 6, 18, 58, 194, 674, 2426, 9050
vector(15,n,n--; a(n,n->2))

\\ not in OEIS: 3, 7, 21, 65, 219, 783, 2973, 11929, 50451
vector(15,n,n--; a(n,n->min(n,2)))

\\ not in OEIS: 3, 8, 25, 87, 329, 1339, 5842, 27202
vector(15,n,n--; a(n,n->n))

\\ not in OEIS: 3, 11, 51, 287, 1911, 14743, 129495
vector(15,n,n--; a(n,n->n!))

\\ not integers
vector(15,n,n--; a(n,n->1/n!))

\\ not in OEIS: 2, 4, 10, 29, 102, 436, 2262, 14018, 101628
vector(15,n,n--; a(n,n->if(n<=2,1,(n-2)!)))

\\ not in OEIS: 2, 4, 10, 26, 76, 242, 850, 3226
vector(15,n,n--; a(n,n->max(1,n-2)))

\\ not in OEIS: 5, 18, 79, 397, 2157, 12575, 78684
vector(15,n,n--; a(n,n->n^2))

\\ monotone odd length blocks only
\\ not in OEIS: 2, 3, 5, 13, 23, 64, 180, 437, 1683
vector(15,n,n--; a(n,n->n%2))

\\ monotone even length blocks only
\\ not in OEIS: 3, 0, 14, 0, 94, 0, 829, 0, 9315, 0, 129095
\\ not in OEIS: 3,    14,    94,    829   , 9315,    129095
vector(15,n,n--; a(n,n->(n+1)%2))


\\ monotone partitions 
\\ not in OEIS: 2, 5, 12, 30, 80, 234, 756
vector(10,n,n--; #select(p->(#p%2)==0,make_labelled_monotone_partitions(n)))
\\ not in OEIS: 2, 4, 10, 28, 84, 262, 845
vector(10,n,n--; #select(p->(#p%2)==1,make_labelled_monotone_partitions(n)))

\\ all partitions
\\ 3, 8, 25, 97, 434, 2095, 10707
\\ A024430 egf cosh(exp(x)-1) partitions into even number of pieces
vector(10,n,n--; #select(p->(#p%2)==0,make_labelled_partitions(n)))
\\ 2, 7, 27, 106, 443, 2045, 10440
\\ A024429 egf sinh(exp(x)-1) partitions into odd number of pieces
vector(10,n,n--; #select(p->(#p%2)==1,make_labelled_partitions(n)))

is_labelled_partition_even(p) = for(i=1,#p,if(#p[i]%2,return(0))); 1;
is_labelled_partition_odd(p)  = for(i=1,#p,if(!(#p[i]%2),return(0))); 1;

print();
\\ monotone partitions 
\\ not in OEIS: 1, 0, 3, 0, 14, 0, 94, 0, 829
\\ not in OEIS: 1,    3,    14,    94,    829
vector(10,n,n--; #select(is_labelled_partition_even,make_labelled_monotone_partitions(n)))
\\ not in OEIS: 2, 3, 5, 13, 23, 64, 180
vector(10,n,n--; #select(is_labelled_partition_odd,make_labelled_monotone_partitions(n)))

\\ all partitions
\\ not in OEIS: 1, 0, 4, 0, 31, 0, 379, 0, 6556
\\              1,    4,    31,    379,    6556
\\ A005046 partitions into even length blocks
vector(10,n,n--; #select(is_labelled_partition_even,make_labelled_partitions(n)))
\\ 2, 5, 12, 37, 128, 457, 1872
\\ A003724 partitions into odd length blocks
vector(10,n,n--; #select(is_labelled_partition_odd,make_labelled_partitions(n)))








C(n,m) =
{
  if(n<0 || m<1 || m>n, error());
  if(n<=1 || m==n, return(vpar_count(n)));
  sum(t=min(2,m), m,
        vpar_count(t) *
        sum(h=m-t+1, n-t,
              C(n-t,h) * binomial(max(0,m-2),max(0,t-2))));
}
A(n) = if(n==0, 1, sum(m=1,n, C(n,m)));
C=memoize(C);
A=memoize(A);

vector(13,n,n--; A(n))


\\-----------------------------------------------------------------------------
\\ vpar_component_minmax() same order min and max
\\
\\ component min and max vertices both ascending, others any
\\ new initial tree t,
\\ min before min of first existing, meaning <= t
\\ max before max of first existing
\\ permute other t-2 with the rest 
\\
\\ forests
\\              1, 1, 3, 14, 97, 917, 11103, 164411, 2880568
\\ not in OEIS:       3, 14, 97, 917, 11103, 164411, 2880568
\\
\\ trees, is all trees
\\ 1, 1, 2, 9, 64, 625, 7776

\\ forests, with root=min components
\\ not in OEIS: 1, 1, 2, 6, 28, 194, 1850, 22662

{
  for(n=0,9,
     my(count=0);
     forvec(vec=vpar_forvec_from(n,1),  \\ trees or forests
           my(vpar=vpar_from_upascend(vec),
              root_of_vertex=vpar_root_of_vertex_vector(vpar),
              mins =vpar_component_minmax(vpar,min),
              maxs =vpar_component_minmax(vpar,max));
           mins==vpar_roots(vpar) || next();  \\ restrict to root=1 components

           mins=apply(v->root_of_vertex[v], mins);
           maxs=apply(v->root_of_vertex[v], maxs);
           count += (mins==maxs);
           \\ if(mins==maxs,print(vpar));
     );
     print(count);
     \\ print(count - vpar_count(n));   \\ sans single trees
  );
}
quit


\\ T(t) products with the strongly monotone partitions
{
  for(n=0,12,
     my(lst=make_labelled_partitions(n),
        count=0);
     for(i=1,#lst,
        my(elem=lst[i],
           mins=apply(vecmin,elem),
           maxs=apply(vecmax,elem));
        mins==vecsort(mins) || error();
        (maxs==vecsort(maxs)) || next();
        count += prod(i=1,#elem, vpar_count(#elem[i]));
     );
     print(count);
     );
}
quit
