\\ Copyright 2017, 2018 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(recover,0)
default(strictargs,1)
default(parisizemax,300*10^6);
read("../vpar.gp");
read("../vpar-wip.gp");
read("memoize.gp");
setrand(getwalltime()%2^32);

option_verbose = 1;
check_count = 0;
check_exact(got,want,name="") =
{
  check_count++;
  if(got!= want,
     print("wrong: "name);
     print("got  "got);
     print("want "want);
     quit(1),
     option_verbose,
     print("ok "check_count" "name));
}


\\ Return a vector of all partitions of 1..n.
\\ Vector elements are like [[1,3],[2]].
\\ Each entry in the element is a vector of some of 1..n.
\\ The entries are in order of ascending minimum number.
\\ The first entry contains 1, second contains smallest not in the first, etc.
\\
make_labelled_partitions(n) =
{
  my(ret=[ [] ]);
  for(d=1,n,
     my(new_ret=List([]));
     for(i=1,#ret,
        my(old=ret[i]);
        for(pos=1,#old,  \\ d extending one of the positions in old
           my(new=old);
           new[pos]=concat(new[pos],[d]);
           listput(new_ret,new));
        listput(new_ret,concat(old,[[d]])));  \\ d new set
     ret=new_ret);
  Vec(ret);
}
check_exact(make_labelled_partitions(0), [ [] ]);
check_exact(make_labelled_partitions(1), [ [[1]] ]);
check_exact(make_labelled_partitions(2), [ [[1,2]], [[1],[2]] ]);
check_exact(make_labelled_partitions(3), \
            [ [[1,2,3]], [[1,2],[3]], [[1,3],[2]], [[1],[2,3]], [[1],[2],[3]] ]);
\\ A000110 Bell numbers, partition n labelled elements
check_exact(vector(8,n,n--; #make_labelled_partitions(n)), \
            [1, 1, 2, 5,  15, 52, 203, 877]);

labelled_partition_is_strongly_monotone(elem) =
{
  my(mins=apply(vecmin,elem),
     maxs=apply(vecmax,elem));
  mins==vecsort(mins) || error();
  maxs==vecsort(maxs);
}
make_labelled_strongly_monotone_partitions(n) = \
  select(labelled_partition_is_strongly_monotone, make_labelled_partitions(n));

check_exact(vector(9,n,n--; #make_labelled_strongly_monotone_partitions(n)), \
            [1, 1, 2, 4, 9, 22, 58, 164, 496]);

labelled_partition_is_nonoverlap(elem) =
{
  my(mins=apply(vecmin,elem),
     maxs=apply(vecmax,elem));
  for(i=1,#elem-1,
     for(j=i+1,#elem,
        mins[i] < maxs[j] || error();
        if(mins[j] < maxs[i],
           if(! (maxs[j] < maxs[i]), return(0)))));
  1;
}
make_labelled_nonoverlap_partitions(n) = \
  select(labelled_partition_is_nonoverlap, make_labelled_partitions(n));


labelled_partition_is_strongly_nonoverlap(elem) =
{
  my(mins=apply(vecmin,elem),
     maxs=apply(vecmax,elem));
  for(i=1,#elem-1,
     for(j=i+1,#elem,
        mins[i] < maxs[j] || error();
        if(mins[j] < maxs[i],
           if(! (maxs[j] < maxs[i]), return(0)));

        \\ no singletons inside another block
        if(mins[j]==maxs[j]
                    && mins[i] < mins[j] && mins[j] < maxs[i],
           return(0))));
  1;
}
make_labelled_strongly_nonoverlap_partitions(n) = \
  select(labelled_partition_is_strongly_nonoverlap,make_labelled_partitions(n));
\\ A092920

A092920_samples = {[
  1, 1, 2, 4, 9, 22, 58, 164, 496, 1601, 5502, 20075, 77531, 315947,
  1354279, 6087421, 28611385, 140239297, 715116827, 3785445032,
  20760746393, 117759236340, 689745339984, 4165874930885, 25911148634728,
  165775085602106
]};



\\-----------------------------------------------------------------------------
\\ 

make_labelled_nonoverlap_partitions(3)
make_labelled_strongly_nonoverlap_partitions(3)
quit

\\ 
\\ 

\\ P[h] height <= h
\\
\\                       height == h-1
\\   height <= h-1  open     x    x    close   height <= h
\\  |------------|  |----------------------|   |----------|
\\       s                    t                    n-s-t

\\ h=0 is 1 layer no nesting
\\ h=1 is 1 layer with a nest somewhere for 2

Ple(n,h) = sum(i=0,h, Peq(n,i));

Peq(n,h) =
{
  my(debug=1);
  n >=0 || error();
  h >=0 || error();
  if(debug,print("Peq n="n" h="h));
  if(n<h, return(0));

  if(h==0,
     return(1 + sum(s=1,n-1, Peq(n-s,h)));  \\ compositions
     );

  sum(s=0,n-1,
        sum(t=1,n-s,
              if(debug,print(" s="s" t="t));
              if(debug,print("  start "Ple(s,h-1)));
              if(debug,print("  end le "n-s-t","h" = "Ple(n-s-t,h)));
              my(term =
                 Ple(s,h-1)
                 *
                 if(h==199999999999,
                    if(debug,print("  h-1=0 so 1"));
                    1
                    ,
                    sum(u=2,t-1,
                          if(debug,print("  u="u" binomial ",binomial(max(t-2,0),max(u-2,0)),
                                         " recurse "t-u","h-1" = ",Peq(t-u,h-1)));
                          binomial(max(t-2,0),max(u-2,0)) * Peq(t-u,h-1)))
                 *
                 Ple(n-s-t,h));
              if(debug,print("  term "term));
              term
        ));
}
Peq=memoize(Peq);

A(n) = Ple(n,n);

check_exact(Peq(0,0), 1, "Peq 0,0");
check_exact(Peq(0,1), 0, "Peq 0,1");
check_exact(Peq(1,0), 1);
check_exact(Peq(1,1), 0, "Peq 1,1");
check_exact(Peq(2,0), 2);
check_exact(Peq(2,1), 0);
check_exact(Peq(2,2), 0);

check_exact(Peq(3,0), 4, "Peq 3,0");
check_exact(Peq(3,1), 1);
check_exact(Peq(3,2), 0);
check_exact(Ple(3,1), 4);
check_exact(Ple(3,3), 4);
check_exact(A(3), 4);

check_exact(Peq(4,0), 1);
check_exact(Peq(4,1), 7);
check_exact(Peq(4,2), 1);
check_exact(A(4), 9);
check_exact(Peq(5,0), 1);
check_exact(Peq(5,1), 15);
check_exact(Peq(5,2), 8);
check_exact(Peq(5,3), 0);
check_exact(Peq(5,4), 0);
check_exact(A(5), 22);

for(n=0,5, print(A(n)));

quit

\\ 3,0   1 2 3  1-2 3  1 2-3  1-2-3     = 4 ways
\\       ^ 2 v
\\ 3,1   1---3                          = 1 way

\\ monotones
\\ [[[1, 2, 3]],
\\ [[1, 2], [3]],
\\ [[1], [2, 3]]
\\ [[1], [2], [3]]]



\\ 5,1 with s=0
\\         ^  v            ^  v ^  v       ^  v   ^  v     ^  v ^     v     
\\   t=2   1  2 3 4 5      1  2 3  4 5     1  2 3 4  5     1  2 3  4  5        
\\         ^     v         ^     v ^  v                  
\\   t=3   1  2  3 4 5     1  2  3 4  5                   2 = 4
\\         ^       v      
\\   t=4   1  2 3  4 5                                    1 = 4
\\         ^         v      
\\   t=5   1  2 3 4  5                                    1*8 = 8   = 19

\\ 5,2 with s=0
\\           ^     v            3 ways for 2,3,4 to go up 1
\\   t=5   ^ 2  3  4  v
\\   u=2   1          5
\\           ^  v               3 ways 2,3,4, choose 2
\\   t=5   ^ 2  3     v
\\   u=3   1       4  5


\\-----------------------------------------------------------------------------
\\ heights only, nesting, Catalan numbers

\\ 1 2 5 13 35 97 275 794 2327 6905

\\ tt sum t=2,t-2 in middle
\\ 2 6 17 45 118 314 846

\\ P[h] height <= h
\\
\\                       height == h-1
\\   height <= h-1  open               close   height <= h
\\  |------------|  |----------------------|   |----------|
\\       s                    t                    n-s-t

Ple(n,h) = sum(i=0,h, Peq(n,i));

Peq(n,h) =
{
  n >=0 || error();
  h >=0 || error();
  if(n==0 && h==0, return(1));
  if(n<2*h, return(0));

  Peq(n-1,h)
  +
  if(h>=1,
     sum(s=0,n-2*h,
           sum(t=2*h,n-s,
                 Ple(s,h-1)
                 *
                 Peq(t,h-1)
                 *
                 Ple(n-s-t,h)
           )));
}
Peq=memoize(Peq);

A(n) = Ple(n,n);

for(n=0,10, print(A(n)));

check_exact(Peq(2,1), 1);
check_exact(Peq(2,2), 0);

quit

\\-----------------------------------------------------------------------------

\\ Flajolet and Schott, "Non-Overlapping Partitions, Continued Fractions,
\\ Bessel Functions and a Divergent Series", European Journal of
\\ Combinatorics, volume 11, pages 421-432, 1990.
\\ http://algo.inria.fr/flajolet/Publications/bessel.ps
\\
\\ Phillipe Flajolet, "Combinatorial Aspects of Continued Fractions",
\\ Discrete Mathematics, volume 32, 1980, pages 125-161.
\\ http://algo.inria.fr/flajolet/Publications/Flajolet80b.pdf
\\ On continued fraction gfs for stepped paths.
\\
\\ "Enumerating Permutations Avoiding a Pair of Babson-Steingrimsson Patterns"
\\ Anders Claesson and Toufik Mansour
\\

c(n,m) =
{
  if(n<0 || m<1 || m>n, error());
  if(n<=1 || m==n, return(1));
  sum(t=min(2,m), m,    \\ 1 if m=1, otherwise 2 to m
      binomial(max(0,m-2),max(0,t-2)) * sum(h=m+1, n, c(n-t,h-t)));
}
a(n) = if(n==0,1, sum(m=1,n, c(n,m)));
c=memoize(c);


print("A092920 samples length ",#A092920_samples);
{
  for(i=1,#A092920_samples,
     my(n=i-1);
     check_exact(a(n), A092920_samples[i], Str("n="n)));
}

quit



\\-----------------------------------------------------------------------------
{
G5 = 1/(1-x-x^2/
        (1-x-x^2/
         (1-2*x-x^2/
          (1-3*x-x^2/
           (1-4*x-x^2/
            (1-5*x-x^2))))))
}
{
G6 = 1/(1-x-x^2/
        (1-x-x^2/
         (1-2*x-x^2/
          (1-3*x-x^2/
           (1-4*x-x^2/
            (1-5*x-x^2/
             (1-6*x-x^2)))))))
}

check_exact(subst(1-6*x-x^2,x,x-1), 6 - 4*x - x^2);
quit



\\-----------------------------------------------------------------------------

p = a + b*x + c*x^2 + d*x^3 + e*x^4 + f*x^5 + g*x^6

1/(1-x-x^2*p)
1/(1-x-x^2*p) +O(x^7)
Vec(1/(1-x-x^2*p) +O(x^7))

{
  Vec(1/(1-x-x^2*p) +O(x^7))
  ==
  [1,
   1,
   a + 1,
   2*a + b + 1,
   a^2 + 3*a + 2*b + c + 1,
   3*a^2 + 2*a*b  + 4*a + 3*b + 2*c + d + 1,
   a^3  +  6*a^2 + 6*a*b + 2*a*c + b^2  +  5*a + 4*b + 3*c + 2*d + e + 1]
}
quit



\\-----------------------------------------------------------------------------

A_gf(k) =
{
  my(p=1);
  forstep(i=k,0,-1,
         p=1/(1 - max(1,i)*x - x^2*p));
  p;
}


{
  my(limit=100);
  my(v=Vec(A_gf(limit) + O(x^limit)));
  for(i=1,limit,
     my(n=i-1);
     check_exact(a(n), v[i], Str("n="n)));
}


check_exact(A_gf(0), 1/(1-x-x^2), "A_gf(0)");
check_exact(A_gf(1), 1/(1-x-x^2/(1-x-x^2)));
check_exact(A_gf(2), 1/(1-x-x^2/(1-x-x^2/(1-2*x-x^2))));

{
check_exact(A_gf(17), 
1/(1-x-x^2/
   (1-x-x^2/
    (1-2*x-x^2/
     (1-3*x-x^2/
      (1-4*x-x^2/
       (1-5*x-x^2/
        (1-6*x-x^2/
         (1-7*x-x^2/
          (1-8*x-x^2/
           (1-9*x-x^2/
            (1-10*x-x^2/
             (1-11*x-x^2/
              (1-12*x-x^2/
               (1-13*x-x^2/
                (1-14*x-x^2/
                 (1-15*x-x^2/
                  (1-16*x-x^2/
                   (1-17*x-x^2))))))))))))))))))
            );
}

n=25;
A_gf(8) + O(x^25)
lift(A_gf(8) + O(x^20))
n=#A092920_samples;
check_exact(Vec(A_gf(n) + O(x^n)), A092920_samples);
check_exact(Vec(A_gf(12) + O(x^n)), A092920_samples);




\\-----------------------------------------------------------------------------

\\ 1.....m          first partition
\\    l......h      second partition, part of rest

\\ initial partition size t>=2, maximum m
\\ max of rest = h must m < h
\\ when t>m extras of range 1..m are elements from rest
\\ extras = m-t to push t many up to m
\\ the positions of extras cannot be 1 nor m
\\ taking from rest cannot be its h high, only its numbering 1..h-1
\\ so h-1 choose m-t-1
\\
\\ extrapos=1 can only shift down v=1 from rest
\\ extrapos=2 can shift v=1 and binomial others

\\ which positions of 2..m-1 will be taken by shifted rest
\\ m-2 extrapos
\\ 1          6
\\    2 3 4 5   7


\\ c(4,1) = 4    [[1], [2, 3, 4]]
\\               [[1], [2, 3], [4]]
\\               [[1], [2], [3, 4]]
\\               [[1], [2], [3], [4]]
\\ c(4,2) = 2    [[1, 2], [3], [4]]        t=2 h=1
\\               [[1, 2], [3, 4]]          t=2 h=2
\\ c(4,3) = 2    [[1, 2, 3], [4]]
\\               [[1, 3], [2, 4]]
\\ c(4,4) = 1    [[1, 2, 3, 4]]
check_exact(4+2+2+1, 9);

\\ c(5,1) = 9
\\ c(5,2) = 4   [[1, 2], [3, 4, 5]]      h=3
\\              [[1, 2], [3, 4], [5]]    h=2
\\              [[1, 2], [3], [4, 5]]    h=1
\\              [[1, 2], [3], [4], [5]]  h=1
\\  
\\ c(5,3) = 4   [[1, 3], [2, 4], [5]]    t=2 h=2
\\              [[1, 3], [2, 4, 5]]      t=2 h=3
\\              [[1, 2, 3], [4], [5]]    t=3 h=1
\\              [[1, 2, 3], [4, 5]]      t=3 h=2
\\  
\\ c(5,4) = 4   [[1, 4], [2, 3, 5]]      t=2 h=3
\\              [[1, 2, 4], [3, 5]]      t=3 h=2
\\              [[1, 3, 4], [2, 5]]      t=3 h=2
\\              [[1, 2, 3, 4], [5]]      t=4 h=1
\\ c(5,5) = 1
check_exact(9+4+4+4+1, 22);

{ my(v=make_labelled_strongly_monotone_partitions(5));
  v=vecsort(v,(x,y)->lex(vecmax(x[1]),vecmax(y[1])));
  for(i=1,#v, print(v[i]));
}

c(n,m) =
{
  my(debug=0);
  if(debug,print("c("n","m")"));

  if(n<0 || m<1 || m > n, error());
  if(n<=1,
     if(debug,print(" initial 1"));
     return(1));
  if(m==n,
     if(debug,print(" special case 1"));
     return(1));

  my(t_min = min(2,m));  \\ 2 if m==1, otherwise 1
  if(debug,print(" t_min = "t_min));
  \\ my(one=c(n-1,m-1));
  \\ if(debug,print("  c("n-1","m-1") = "one));
  my(ret=sum(t=t_min,m,
               my(extrapos = max(0,m-t),  \\ to bring in from rest
                  h_min = m-t+1,
                  h_max = n-t);
               if(debug,print(" t="t" extrapos "extrapos"  h range "h_min" to "h_max));
               sum(h=h_min,h_max,
                     my(bin_num = max(0,m-2),
                        bin_den = max(0,t-2),
                        bin = binomial(bin_num,bin_den),
                        term=c(n-t,h) * bin);
                     if(debug,print("  h="h" rest c("n-t","h")=",c(n-t,h)
                                    " binomial "bin_num","bin_den" = "bin
                                    "  term "term));
                     term)));
  if(debug,print(" ret "ret));
  ret;
}
A(n) =
{
  print("A("n")");
  if(n==0,
     print(" initial 1");
     return(1));
  sum(m=1,n, c(n,m));
}
C=memoize(C);
A=memoize(A);

limit=10;
{
  for(n=1,limit,
     print("got "A(n)));
}
for(n=1,limit,print(A(n)));

check_exact(c(1,1), 1, "c(1,1)");
check_exact(c(2,1), 1, "c(2,1)");
check_exact(c(2,2), 1);
check_exact(c(3,1), 2, "c(3,1)");
check_exact(c(3,2), 1);
check_exact(c(3,3), 1);
check_exact(c(4,1), 4, "c(4,1)");
check_exact(c(4,2), 2);
check_exact(c(4,3), 2, "c(4,3)");
check_exact(c(4,4), 1);
check_exact(c(5,2), 4);
check_exact(c(5,3), 4, "c(5,3)");


C_list_by_make(n) =
{
  my(v=make_labelled_strongly_monotone_partitions(n),
     ret=vector(n));
  for(i=1,#v, ret[vecmax(v[i][1])]++);
  ret;
}
C_list(n) = vector(n,m, c(n,m));

{
  for(n=1,limit,
     check_exact(C_list(n), C_list_by_make(n),
                 Str("n="n));
     );
}

{
  for(n=1,#A092920_samples,
     check_exact(A(n), A092920_samples[n], Str("n="n)));
}

quit;





\\-----------------------------------------------------------------------------
\\ labelled partitions with max ascending
\\
\\ n=0 1 2 3 4  5
\\   1 1 2 4 9 22 58 164 496 1601
\\ A092920 strongly monotone partitions
\\ no block contains another within its span, span = min to max

\\ gf    1/(1 - x - x^2*B(x)) where B(x) Bessel numbers

\\ n elements, first block max m
\\ first and second blocks can interleave
\\
\\    l......h              2^(l-m-1) choices middle
\\ 1.....m
\\
\\ or not interleave
\\         l......h
\\ 1 ... m

\\ c(n,m) =
\\ {
\\   if(n<=1, return(1));
\\      sum(l=2,n, 
\\            sum(h=l+1,n, 
\\                  2^(l-m-1) * c(


\\ {
\\   for(n=0,9,
\\      my(lst=make_labelled_partitions(n),
\\         count=0);
\\      for(i=1,#lst,
\\         my(elem=lst[i],
\\            mins=apply(vecmin,elem),
\\            maxs=apply(vecmax,elem));
\\         mins==vecsort(mins) || error();
\\         count+= (maxs==vecsort(maxs)));
\\      print(count);
\\      );
\\ }
\\ quit
        

