\\ Copyright 2017, 2018 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(recover,0)
default(strictargs,1)
read("../vpar.gp");
read("../vpar-wip.gp");
read("../test-helpers.gp");
read("memoize.gp");
\\ GP-DEFINE  read("../vpar.gp");
\\ GP-DEFINE  read("../vpar-wip.gp");


\\ Think can be done easily enough with a count and pairings rather than a
\\ state.
\\
\\ Varying the bittest for what states are acceptable to continue would let
\\ perfect and near perfect run in the same table.


\\-----------------------------------------------------------------------------
\\ vpar_is_near_perfect_matching() states


\\ rel fields are booleans indicating the whether some matching at and below
\\ v has the respective characteristics.
\\
\\   with    = somewhere below v there is an unmatched vertex
\\   without = everything below v is matched, don't yet have an unmatched
\\
\\ These with/without don't include v itself.  So when v unmatched the
\\ with/without indicates whether there is some further vertex below v which
\\ is unmatched, and going to be left unmatched.
\\
\\ The aim per test_near_perfect_matching_rel_is_near_at_root() is to find a
\\ matched_with, which is v matched and one unmatched somewhere below.  Or
\\ unmatched_without which is v unmatched but everything else below it
\\ matched.
\\
\\ In a forest can have some evens which are perfect matchings in even size
\\ component trees.

test_near_perfect_matching_rel_initial = {[
  0,  \\ unmatched_with
  1,  \\ unmatched_without
  0,  \\ matched_with
  0   \\ matched_without
]};

test_near_perfect_matching_rel_validate(rel) =
{
  type(rel)=="t_VEC" || error("rel=",rel);
  #rel==4 || error("rel=",rel);
  for(i=1,#rel,
     rel[i]==0 || rel[i]==1 || error("rel=",rel));
}

test_near_perfect_matching_rel_add(p_rel,v_rel) =
{
  print("test_near_perfect_matching_rel_add() "p_rel"  "v_rel);
  test_near_perfect_matching_rel_validate(p_rel);
  test_near_perfect_matching_rel_validate(v_rel);

  my(v_unmatched_with,v_unmatched_without,v_matched_with,v_matched_without);
  [v_unmatched_with,v_unmatched_without,v_matched_with,v_matched_without] = v_rel;
  my(p_unmatched_with,p_unmatched_without,p_matched_with,p_matched_without);
  [p_unmatched_with,p_unmatched_without,p_matched_with,p_matched_without] = p_rel;

  my(new_matched_with
     = ((p_matched_with && v_matched_without)         \\ extend
        || (p_matched_without && v_matched_with)      \\ with below v
        || (p_matched_without && v_unmatched_without) \\ with at v
        || (p_unmatched_with && v_unmatched_without)     \\ new pair
        || (p_unmatched_without && v_unmatched_with)     \\ new pair
        ));
     
  my(new_matched_without
     = ((p_matched_without && v_matched_without)
        || (p_unmatched_without && v_unmatched_without)   \\ new pair
       ));
     
  my(new_unmatched_with
     = ((p_unmatched_with && v_matched_without)         \\ extend
        || (p_unmatched_without && v_matched_with)      \\ inherit with from v
        || (p_unmatched_without && v_unmatched_without) \\ v becomes with
       ));
     
  my(new_unmatched_without
     = (p_unmatched_without && v_matched_without));
     
  [new_unmatched_with, new_unmatched_without,
   new_matched_with, new_matched_without];
}

{
  check_equal(test_near_perfect_matching_rel_add
              ([0,
                1,   \\ unmatched_without
                0,
                0
               ],              
               [1,    \\ unmatched_with
                0,
                0,
                0
               ]),
              
               [0,
                0,
                1,    \\ matched_with
                0
               ]);
}


test_near_perfect_matching_rel_list = List([]);

listput(test_near_perfect_matching_rel_list, \
        test_near_perfect_matching_rel_initial);
{
  \\ 2 = path-2
  \\    *
  \\    |
  \\    *
  listput(test_near_perfect_matching_rel_list,
          test_near_perfect_matching_rel_add
          (test_near_perfect_matching_rel_list[1],
           test_near_perfect_matching_rel_list[1]));
  check_equal(test_near_perfect_matching_rel_list[2],
              [1,    \\ unmatched_with
               0,
               0,
               1]);  \\ matched_without
}
{
  \\ 3 = path-3
  \\    *
  \\    |
  \\    *
  \\    |
  \\    *
  listput(test_near_perfect_matching_rel_list,
          test_near_perfect_matching_rel_add
          (test_near_perfect_matching_rel_list[1],
           test_near_perfect_matching_rel_list[2]));
  check_equal(test_near_perfect_matching_rel_list[3],
              [0,
               1,    \\ unmatched_without
               1,    \\ matched_with
               0]);
}
{
  \\ 4 = star-3
  \\    *
  \\   / \
  \\  *   *
  listput(test_near_perfect_matching_rel_list,
          test_near_perfect_matching_rel_add
          (test_near_perfect_matching_rel_list[2],
           test_near_perfect_matching_rel_list[1]));
  check_equal(test_near_perfect_matching_rel_list[4],
              [0,    \\ unmatched_with
               0,
               1,    \\ matched_with
               0],
             "rel 3");
}
{
  \\ 5 = claw
  \\    *
  \\    |
  \\    *
  \\   / \
  \\  *   *
  listput(test_near_perfect_matching_rel_list,
          test_near_perfect_matching_rel_add
          (test_near_perfect_matching_rel_list[1],
           test_near_perfect_matching_rel_list[4]));
  check_equal(test_near_perfect_matching_rel_list[5],
              [1,    \\ unmatched_with
               0,
               0,    \\ matched_with
               0],
             "rel 4");
}
\\ {
\\   \\ 5 = path-4 root at second
\\   \\     *
\\   \\   / |
\\   \\  *  *
\\   \\     |
\\   \\     *
\\   listput(test_near_perfect_matching_rel_list,
\\           test_near_perfect_matching_rel_add
\\           (test_near_perfect_matching_rel_list[2],
\\            test_near_perfect_matching_rel_list[2]));
\\ }
\\ {
\\   \\ 6
\\   \\     *
\\   \\     |
\\   \\     *
\\   \\   / |
\\   \\  *  *
\\   \\     |
\\   \\     *
\\   listput(test_near_perfect_matching_rel_list,
\\           test_near_perfect_matching_rel_add
\\           (test_near_perfect_matching_rel_list[1],
\\            test_near_perfect_matching_rel_list[5]));
\\ }
\\ {
\\   \\ 8 = claw
\\   \\    *
\\   \\    | \
\\   \\    *  *
\\   \\   / \  \
\\   \\  *   *  *
\\   listput(test_near_perfect_matching_rel_list,
\\           test_near_perfect_matching_rel_add
\\           (test_near_perfect_matching_rel_list[4],
\\            test_near_perfect_matching_rel_list[2]));
\\ }
\\ {
\\   \\ 9
\\   \\     *
\\   \\     | \
\\   \\     *  *
\\   \\   / |   \
\\   \\  *  *    *
\\   \\     |
\\   \\     *
\\   listput(test_near_perfect_matching_rel_list,
\\           test_near_perfect_matching_rel_add
\\           (test_near_perfect_matching_rel_list[6],
\\            test_near_perfect_matching_rel_list[2]));
\\ }

test_near_perfect_matching_rel_list = Vec(test_near_perfect_matching_rel_list);

{
  if(1,
     for(i=1,#test_near_perfect_matching_rel_list,
        my(rel=test_near_perfect_matching_rel_list[i]);
        print("state="i"  ",rel));
     );
}

\\ Return the state number 1 to 5 for the given rel, or return 0 if not one
\\ of the 5 states in test_near_perfect_matching_rel_list.
test_near_perfect_matching_rel_find(rel) =
{
  for(i=1,#test_near_perfect_matching_rel_list,
     if(test_near_perfect_matching_rel_list[i] == rel, return(i)));
  0;
}

\\ Return 1 if rel is a non near perfect matching state, meaning that no
\\ matter what parent or further children, ther won't be a near perfect
\\ matching.
\\
test_near_perfect_matching_rel_is_non(rel) =
{
  test_near_perfect_matching_rel_validate(rel);
  vecsum(rel)==0;
}
{
  for(i=1,#test_near_perfect_matching_rel_list,
     my(rel=test_near_perfect_matching_rel_list[i]);
     check_equal(test_near_perfect_matching_rel_is_non(rel), 0,
                 "test_near_perfect_matching_rel_is_non() on states"));
}

{
  for(p=1,#test_near_perfect_matching_rel_list,
     for(v=1,#test_near_perfect_matching_rel_list,
        my(rel=test_near_perfect_matching_rel_add
               (test_near_perfect_matching_rel_list[p],
                test_near_perfect_matching_rel_list[v]));

        check_equal(test_near_perfect_matching_rel_find(rel)
                    || test_near_perfect_matching_rel_is_non(rel), 1,
                    Str("test_near_perfect_matching_rel transition"
                        " p="p" v="v" to "rel))));
}
print("test_near_perfect_matching_rel_list  ",#test_near_perfect_matching_rel_list);

{
  test_near_perfect_matching_rel_table
  = matrix(#test_near_perfect_matching_rel_list,
           #test_near_perfect_matching_rel_list,
           p,v,
           test_near_perfect_matching_rel_find
           (test_near_perfect_matching_rel_add
            (test_near_perfect_matching_rel_list[p],
             test_near_perfect_matching_rel_list[v])))
}

{
  check_equal(test_near_perfect_matching_rel_list,
              [[0, 1, 0, 0],  \\ 1
               [1, 0, 0, 1],  \\ 2    not at root
               [0, 1, 1, 0],  \\ 3  (equiv to 1)
               [0, 0, 1, 0],  \\ 4    not at root
               [1, 0, 0, 0]   \\ 5
              ]);
}

\\ Return 1 if rel is not an edge almost equimatchable state as a root.
\\ Per vpar_is_equimatchable() states, cannot have 4 as that is a
\\ non-equimatchable as a root.
\\
test_near_perfect_matching_rel_is_near_at_root(rel) =
{
  my(unmatched_with,unmatched_without,matched_with,matched_without);
  [unmatched_with,unmatched_without,matched_with,matched_without] = rel;

  matched_with || unmatched_without;
}
{
  test_near_perfect_matching_rel_near_at_root
  = apply(rel-> test_near_perfect_matching_rel_is_near_at_root(rel),
         test_near_perfect_matching_rel_list);
  check_equal(test_near_perfect_matching_rel_near_at_root,
              [1, 0, 1, 1, 0]);
}

test_near_perfect_matching_rel_is_even_at_root(rel) =
{
  my(unmatched_with,unmatched_without,matched_with,matched_without);
  [unmatched_with,unmatched_without,matched_with,matched_without] = rel;

  matched_without;
}
{
  test_near_perfect_matching_rel_even_at_root
  = apply(rel-> test_near_perfect_matching_rel_is_even_at_root(rel),
         test_near_perfect_matching_rel_list);
  check_equal(test_near_perfect_matching_rel_even_at_root,
              [0, 1, 0, 0, 0]);
}

test_near_perfect_matching_rel_is_equivalent(x,y) =
{
  \\ same final ok-ness
  test_near_perfect_matching_rel_near_at_root[x] ==
  test_near_perfect_matching_rel_near_at_root[y] || return(0);

  test_near_perfect_matching_rel_even_at_root[x] ==
  test_near_perfect_matching_rel_even_at_root[y] || return(0);

  my(m=apply(s->if(s==y,x,s), test_near_perfect_matching_rel_table));

  \\ same transitions when add new child under
  m[x,] == m[y,] || return(0);

  \\ same transitions when are a child under some parent
  m[,x] == m[,y] || return(0);

  1;
}

check_equal(test_near_perfect_matching_rel_is_equivalent(1,3), 1);

test_near_perfect_matching_rel_table = \
  apply(s->if(s==3,1,s), test_near_perfect_matching_rel_table);

{
  for(x=1,#test_near_perfect_matching_rel_list,
     for(y=x+1,#test_near_perfect_matching_rel_list,
        check_equal(test_near_perfect_matching_rel_is_equivalent(x,y),
                    (x==1 && y==3)
                    ,
                    Str("test_near_perfect_matching_rel_is_equivalent()"
                        " x="x" y="y))));
}

\\ 3 -> 1
\\                                       1,2,3,4,5
test_near_perfect_matching_rel_shrink = [1,2,1,3,4];
test_near_perfect_matching_rel_shrink_size = \
  #test_near_perfect_matching_rel_list - 1;
{
  check_equal(test_near_perfect_matching_rel_shrink_size, 4,
              "test_near_perfect_matching_rel_shrink_size");
  check_equal(vecmax(test_near_perfect_matching_rel_shrink),
              test_near_perfect_matching_rel_shrink_size,
              "vecmax test_near_perfect_matching_rel_shrink");
}
vpar_INTERNAL_is_near_perfect_matching_table = {Vecsmall([
                     \\ unm   matched
                     \\ w  wo w  wo
  2,  1,  4,  3,     \\ 0, 1, 0, 0
  3,  2,  3,  0,     \\ 1, 0, 0, 1
  0,  3,  0,  0,     \\ 0, 0, 1, 0
  3,  4,  0,  0      \\ 1, 0, 0, 0      
])};
test_is_near_perfect_matching_num_states = 4;
test_is_near_perfect_matching_near_at_root_bits = 10;
test_is_near_perfect_matching_near_at_root(state) = \
  bittest(test_is_near_perfect_matching_near_at_root_bits,state);

test_is_near_perfect_matching_even_at_root_bits = 4;
test_is_near_perfect_matching_even_at_root(state) = \
  bittest(test_is_near_perfect_matching_even_at_root_bits,state);

{
  \\ vpar_INTERNAL_is_near_perfect_matching_table[]
  \\ generated
  my(m=matrix(test_near_perfect_matching_rel_shrink_size,
              test_near_perfect_matching_rel_shrink_size));
  for(p=1,#test_near_perfect_matching_rel_list,
     my(ps=test_near_perfect_matching_rel_shrink[p]);
     if(!ps, next());
     for(v=1,#test_near_perfect_matching_rel_list,
        my(vs=test_near_perfect_matching_rel_shrink[v]);
        if(!vs, next());

        my(t=test_near_perfect_matching_rel_find
              (test_near_perfect_matching_rel_add
               (test_near_perfect_matching_rel_list[p],
                test_near_perfect_matching_rel_list[v])));
        if(t, t=test_near_perfect_matching_rel_shrink[t]);
        m[ps,vs] = t));

  if(1,
     for(p=1,test_near_perfect_matching_rel_shrink_size,
        for(v=1,test_near_perfect_matching_rel_shrink_size,
           printf(" %2d%s",
                  m[p,v],
                  if(v==test_near_perfect_matching_rel_shrink_size
                     && p==test_near_perfect_matching_rel_shrink_size,"",",")));
     print()));

  my(l=List([]));
  for(p=1,test_near_perfect_matching_rel_shrink_size,
     for(v=1,test_near_perfect_matching_rel_shrink_size,
        listput(l,m[p,v])));
  check_equal(vpar_INTERNAL_is_near_perfect_matching_table,
              Vecsmall(l),
              "vpar_INTERNAL_is_near_perfect_matching_table generated");
}

check_equal(test_is_near_perfect_matching_num_states, \
            test_near_perfect_matching_rel_shrink_size);

{
  \\ vpar_is_near_perfect_matching() near-perfect constant, inline in code
  \\ generated
  my(want=0);
  for(i=1,#test_near_perfect_matching_rel_list,
     if(test_near_perfect_matching_rel_near_at_root[i],
        my(state=test_near_perfect_matching_rel_shrink[i]);
        if(!state, next());
        want=bitor(want, 1 << state)));
  check_equal(10,
              want,
              "vpar_is_near_perfect_matching() near root bits constant");
  check_equal(test_is_near_perfect_matching_near_at_root_bits,
              want);
}
{
  \\ vpar_is_near_perfect_matching() even constant, inline in code
  \\ generated
  my(want=0);
  for(i=1,#test_near_perfect_matching_rel_list,
     if(test_near_perfect_matching_rel_even_at_root[i],
        my(state=test_near_perfect_matching_rel_shrink[i]);
        if(!state, next());
        want=bitor(want, 1 << state)));
  check_equal(4,
              want,
              "vpar_is_near_perfect_matching() even root bits constant");
  check_equal(test_is_near_perfect_matching_even_at_root_bits,
              want);
}

test_is_near_perfect_matching_table_lookup(p,v) =
{
  (   1 <= p && p <= test_is_near_perfect_matching_num_states
   && 1 <= v && v <= test_is_near_perfect_matching_num_states ) || error();
  vpar_INTERNAL_is_near_perfect_matching_table
    [test_is_near_perfect_matching_num_states*p
     + v - test_is_near_perfect_matching_num_states];
}
test_near_perfect_matching_get_state(vpar) =
{
  my(seq=vpar_INTERNAL_seq_upwards(vpar),
     state = vectorsmall(#vpar,v,1));
  for(i=1,#seq,
     my(v=seq[i], p=vpar[v]);
     if(p,
        (state[p]=test_is_near_perfect_matching_table_lookup(state[p],state[v]))
          || return(0);
        ,
        return(state[v])));
  0;
}

test_near_perfect_matching_state_is_equivalent(x,y) =
{
  \\ same final ok-ness
  test_is_near_perfect_matching_near_at_root(x) ==
  test_is_near_perfect_matching_near_at_root(y) || return(0);

  test_is_near_perfect_matching_even_at_root(x) ==
  test_is_near_perfect_matching_even_at_root(y) || return(0);

  \\ same transitions when add new child under
  for(v=1,test_is_near_perfect_matching_num_states,
     if(test_is_near_perfect_matching_table_lookup(x,v) !=
        test_is_near_perfect_matching_table_lookup(y,v),
        return(0)));

  \\ same transitions when under some parent
  for(p=1,test_is_near_perfect_matching_num_states,
     if(test_is_near_perfect_matching_table_lookup(p,x) !=
        test_is_near_perfect_matching_table_lookup(p,y),
        return(0)));

  1;
}
{
  for(x=1,test_is_near_perfect_matching_num_states,
     for(y=x+1,test_is_near_perfect_matching_num_states,
        check_equal(test_near_perfect_matching_state_is_equivalent(x,y),
                    0,
                    Str("test_near_perfect_matching_state_is_equivalent()"
                        " x="x" y="y"  (after shrink)"))));
}

\\ {
\\   \\ almost near perfect state of path-n is not the smallest having its state,
\\   \\ or perhaps only state<=5
\\   for(n=2,6,
\\      my(path=vpar_make_path(n),
\\         vpar=vpar_unlabelled_first(n));
\\      until((vpar=vpar_unlabelled_next(vpar)) == 'none,
\\            my(state=test_near_perfect_matching_get_state(vpar));
\\            if(state==6 && vpar==[0, 1, 2, 1], next());
\\            if(state==6 && vpar==[0, 1, 2, 2, 1], next());
\\            if(state==6 && vpar==[0, 1, 2, 1, 1], next());
\\            if(state==7, next());
\\            if(state==8, next());
\\            \\ print("n="n" state="state" in "vpar);
\\            check_equal(state <= n, 1,
\\                        Str("vpar_is_near_perfect_matching()"
\\                            " expect no vpar has n < state\n"
\\                            " state="state" n="n" vpar="vpar));
\\           ));
\\ }


vpar_is_near_perfect_matching_by_states(vpar) =
{
  print("vpar_is_near_perfect_matching_by_states() "vpar);

  my(count=0);
  if(bitand(#vpar,1),
     my(seq=vpar_INTERNAL_seq_upwards(vpar),
        state = vectorsmall(#vpar,v,1));
     for(i=1,#seq,
        my(v=seq[i], p=vpar[v]);
        print(" v="v" state "state[v]" p="p,if(p,Str(" state "state[p]),""));
        if(p,
           state[p] = vpar_INTERNAL_is_near_perfect_matching_table
                        [(state[p]<<2) + state[v] - 4],
           bittest(14,state[v])                       \\ ok at root
           && (count += bittest(10,state[v])) <= 1)   \\ count odds root
        || return(0);
        ));
  print(" final count="count);
  count==1;

  \\ GP-Test  binary(14) == [   1, 1, 1, 0]  /* ok */
  \\                         4  3  2  1   state (shrunk)
  \\ GP-Test  binary(10) == [   1, 0, 1, 0]  /* count near-perfect */
  \\                         4  3  2  1   state (shrunk)
  \\ Not states 2,4 at root.
}

check_equal(test_near_perfect_matching_get_state([0]), 1);
check_equal(#vpar_INTERNAL_is_near_perfect_matching_table, 4*4);

{
  \\      5
  \\      |
  \\      4
  \\      |
  \\      1
  \\     / \
  \\    2   4
  my(vpar=[3, 1, 5, 1, 0]);
  check_equal(test_near_perfect_matching_get_state(vpar), 3);
  check_equal(test_is_near_perfect_matching_table_lookup(1,1), 2);
  check_equal(test_is_near_perfect_matching_table_lookup(2,1), 3);
  check_equal(test_is_near_perfect_matching_table_lookup(1,3), 4);
  check_equal(test_is_near_perfect_matching_table_lookup(1,4), 3);
  check_equal(vpar_is_near_perfect_matching_by_states(vpar), 1);
}

check_equal(vpar_is_near_perfect_matching_by_states([0,0]), 0);
check_equal(vpar_is_near_perfect_matching_by_states([0,1,0,3]), 0);
check_equal(vpar_is_near_perfect_matching_by_states([0,1,0,3,0]), 1);

{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i],
        got=vpar_is_near_perfect_matching_by_states(vpar));
     print(vpar);
     check_equal(got, vpar_matchnum(vpar)==(#vpar-1)/2,
                 Str("vpar_is_near_perfect_matching_by_states()"
                     " vs vpar_matchnum()"));
  );
}
