\\ Copyright 2020, 2021, 2022 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


\\ Tests related to Matula-Goebel primecode.

default(recover,0);
default(strictargs,1);

option_verbose=1;
option_progress=1;

read("../vpar.gp");
read("../vpar-wip.gp");
read("../test-helpers.gp");
read("OEIS-data.gp");
read("OEIS-data-wip.gp");


\\-----------------------------------------------------------------------------
\\ Num Subtrees

\\ [with, without, vparlist]
made=[ [1,0, [ [0] ]] ]
limit = 50;

join(p,v) =
{
  \\ print("join p="p);
  \\ print("     v="v);
  my(with = p[1] * (v[1]+1),
     without = p[2] + v[1]+v[2]);
  if(with+without > limit, return('none));
  my(p_vpars=p[3],
     v_vpars=v[3],
     m=Map());
  foreach(p[3],p_vpar,
     foreach(v[3],v_vpar,
        my(vpar=vpar_join(p_vpar,1,v_vpar));
        if(!vpar_is_tree(vpar),
           print(p_vpar);
           print(v_vpar);
           print(vpar);
           error());
        vpar=vpar_to_unlabelled(vpar);
        mapput(m,vpar,1)));
  [with, without, Mat(m)[,1]~];
}

expand() =
{
  my(m=Map());
  for(i=1,#made,
     for(j=1,#made,
        my(new=join(made[i],made[j]));
        if(new!='none,
           mapput(m,new,1))));
  my(oldsize=#made);
  made = setunion(made,Mat(m)[,1]~);
  #made != oldsize;
}
\\ {
\\   for(i=1,10,
\\      print(expand());
\\      print("made "made));
\\ }

while(expand(), print(#made));
print("final "#made);

{
  foreach(made,q,
     foreach(q[3],vpar,
        vpar_validate(vpar);
        vpar_is_tree(vpar) || error()));
}

{
  for(i=1,#made,
     my(q=made[i]);
     if(q[1]+q[2]==48,
        print(q)));
}

by_total = vector(limit,i,[]);
{
  for(i=1,#made,
     my(q=made[i]);
     my(total=q[1]+q[2]);
     if(total > limit, next);
     by_total[total] = concat(by_total[total], q[3]));
}
free_undup(vpars) =
{
  if(#vpars==0,return([]));
  my(m=Map());
  foreach(vpars,vpar, mapput(m,vpar_to_free(vpar),1));
  Mat(m)[,1]~;
}
by_total_free=apply(free_undup,by_total);
{
  for(i=1,#by_total,
     print("by total ",i," rooted ",#by_total[i]" free ",#by_total_free[i]," "by_total_free[i]));
}

\\ apply(length,by_total)
\\ num rooted trees with n subtrees
\\ not in OEIS: 1, 0, 1, 0, 0, 2, 0, 0, 0, 2, 2, 0, 0, 0, 3, 0, 4, 0, 0, 2, 3, 0, 0, 5, 4, 0, 0, 6, 0, 4, 0, 6, 0, 7, 0, 7, 5, 0, 0, 6, 12, 0, 0, 12, 10, 0, 3, 6, 4, 0

apply(length,by_total_free)
\\ not in OEIS: 1, 0, 1, 0, 0, 1, 0, 0, 0, 1, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 2, 0, 1, 0, 1, 0, 1, 0, 2, 2, 0, 0, 1, 2, 0, 0, 2, 2, 0, 1, 1, 1, 0

quit


\\ {
\\   my(seen=Map());
\\   for(n=0,13,
\\      my(vpar=vpar_free_first(n));
\\      until((vpar=vpar_free_next(vpar)) == 'none,
\\            my(num=vpar_num_subcons(vpar));
\\            mapput(seen,num,1);
\\            if(num==48,print("48 ways: "vpar));
\\           ));
\\   seen=Mat(seen)[,1]~;
\\   my(limit=100);
\\   seen=select(n->n<=limit,seen);
\\   my(unseen=setminus([1..limit],seen));
\\   print("seen   ",seen);
\\   print("unseen ",unseen);
\\   print("cf     ",OEIS_data("A184164")[1..30])
\\ }
\\ all=OEIS_data("A184164")  \\ not num_seen
\\ 2, 4, 5, 7, 8, 9, 12, 13, 14, 16, 18, 19


\\-----------------------------------------------------------------------------
\\ Caterpillars

\\ select(n->!test_is_caterpillar_by_reroot(vpar_from_primecode(n)),[1..100])
\\ not caterpillars
\\ not in OEIS: 27, 45, 46, 54, 63, 69, 75, 81, 83, 90, 92, 94, 99

{
  \\ 1,2,4,9,20,45,100
  \\ for(n=2,8,
  \\    my(vpar=vpar_unlabelled_first(n),
  \\      count=0);
  \\    until((vpar=vpar_unlabelled_next(vpar)) == 'none,
  \\          count+=test_is_caterpillar_by_reroot(vpar));
  \\    print1(count","));

  \\ quit;
}

{
  for(i=1,#test_sample_trees,
     my(vpar=test_sample_trees[i],
        by_states = vpar_is_caterpillar(vpar),
        by_reroot = test_is_caterpillar_by_reroot(vpar));
     \\ print(vpar);
     check_equal(by_states, by_reroot,
                 "vpar_is_caterpillar() vs test_is_caterpillar_by_reroot()");
     );
}
{
  for(n=2,12,
     my(vpar=vpar_unlabelled_first(n));
     until((vpar=vpar_unlabelled_next(vpar)) == 'none,
           my(by_states = vpar_is_caterpillar(vpar),
              by_reroot = test_is_caterpillar_by_reroot(vpar));
           if(by_states != by_reroot,
              print(vpar" states "by_states" reroot "by_reroot);
              error())));
}
{
  for(n=2,100,
     for(rep=1,100,
     my(vpar=vpar_make_random(n),
        by_states = vpar_is_caterpillar(vpar),
        by_reroot = test_is_caterpillar_by_reroot(vpar));
        if(by_states != by_reroot,
           print(vpar" states "by_states" reroot "by_reroot);
           error())));
}



\\-----------------------------------------------------------------------------
progress("end");
