\\ Copyright 2017, 2018 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


\\ Count how many trees or forests have vertices labelled in order of their
\\ subtree height, so that vpar_seq_subtree_height() is the identity
\\ permutation.

\\              1, 1, 2, 5, 16, 61, 268, 1329, 7342, 44703, 297314, 2143479
\\ not in OEIS:             16, 61, 268, 1329, 7342, 44703, 297314, 2143479

default(recover,0)
read("../vpar.gp");
read("../vpar-wip.gp");
read("memoize.gp");
setrand(getwalltime()%2^32);

option_verbose=0;

check_count = 0;
check_exact(got,want,name="") =
{
  check_count++;
  if(got!= want,
     print("wrong: "name);
     print("got  "got);
     print("want "want);
     quit(1),
     option_verbose,
     print("ok "check_count" "name));
}

\\-----------------------------------------------------------------------------

\\ code
\\ initial L 
\\ 1..L all >= L+1, and one each of L+1 .. L+F
\\ L+1 .. L+F all >= L+F+1


\\ L new leaves
\\ into F buckets >=1, and N-F buckets >=0
\\
bucket(n,f,l, prefix="") =
{
  my(debug=0);
  if(debug, print(prefix"bucket n="n" f="f" l="l));

  if(l==0 && f!=0, return(0));
  if(l==0,return(1));
  if(n<0, return(0));
  if(n<f, return(0));
  if(l<f, return(0));  \\ cannot F>=1
  if(f<0,error());

  my(ret=
  if(f>0,
     sum(c=1,l,
           if(debug, print(prefix"f>0 c="c" binomial "binomial(l,c)));
           binomial(l,c) * bucket(n-1,f-1,l-c, concat(" ",prefix))),  \\ 1 F

     n>0 && n>f,
     sum(c=0,l,
           if(debug, print(prefix"n>0 c="c" binomial "binomial(l,c)));
           binomial(l,c) * bucket(n-1,f,l-c, concat(" ",prefix))),  \\ 0 N

     0));
  if(debug, print(prefix"ret "ret));
  ret;
}
check_exact(bucket(0,0,0), 1);
check_exact(bucket(0,0,1), 0);
check_exact(bucket(1,1,1), 1);
check_exact(bucket(1,1,2), 1);
check_exact(bucket(2,1,1), 1);
check_exact(bucket(2,1,0), 0);
check_exact(bucket(3,2,2), 2, "bucket 3,2,2");
\\quit

\\ n vertices, l leaves
C(n,l) =
{
  my(debug=0);

  if(debug, print("C "n" "l));
  if(l==0 && n==0, return(1));
  if(l==1 && n==1, return(1));
  if(n==0 || l==0, return(0));
  if(n<0 || n<l,return(0));

  sum(f=1,l,
        if(debug,print("f="f"  prod "bucket(n-l,f,l)));
        C(n-l,f) * bucket(n-l,f,l));
}

print("C(1,1) ", C(1,1));
print("C(1,2) ", C(1,2));
print("C(3,1) ", C(3,1));
print("C(3,2) ", C(3,2));
print("C(4,2) ", C(4,2));
print("C(5,2) ", C(5,2));
check_exact(C(1,1), 1);
check_exact(C(2,1), 1);
check_exact(C(2,1), 1);
check_exact(C(5,1), 1);
check_exact(C(2,2), 0);
check_exact(C(4,2), 3);

check_exact(C(3,2), 1);
check_exact(bucket(3,2,2), 2, "bucket 3,2,2");
check_exact(C(3,1), 1);

check_exact(C(5,2), 7);

\\ 5,2 with L=2 is C(3,2)=1 x 2 ways around L
\\              or C(3,1)=1 L1,L2 on F
\\              or C(3,1)=1 x 2 choice L1 onto F1 and x 2 L2 anywhere
\\              = 2 + 1 + 4 = 7


for(n=1,6, my(t=0); for(l=1,max(6,n), my(c=C(n,l)); print1(" "c); t+=c); \
           print(" total "t));
S(n) = sum(l=1,n, C(n,l));

S(1)
vector(12,n,S(n))

S=memoize(S);
C=memoize(C);
bucket=memoize(bucket);

\\ for(n=2,500, print(log(S(n))/log(n)/n));
\\ log(S)/log(n)/n = c
\\ log(S) = c*n*log(n)
\\ S = (c*n)^n

\\ S(60)*1.0
\\ for(n=2,500, print(n"  "S(n)^(1/(n+1))/n));
\\ S = (c*n)^(n+1)
\\ my(n=60); (0.21449*n)^(n+1)
\\ 60  0.2144903114951859113079055417

for(n=3,500, print(n"  "S(n)^(1/(n-2))/n));
\\ 50  0.2520262855817974998580308569
\\ 70  0.2393798019666834837162943665
\\ S = (c*n)^(n-1)     cf all trees n^(n-2)

quit

\\-----------------------------------------------------------------------------
\\ Wrong when attach to same L or non-L.
\\ CC(n,l) =
\\ {
\\   if(n==0, return(0));
\\   if(n==1 && l==1, return(1));
\\   if(n<=l, return(0));
\\   if(l<=0, return(0));
\\ 
\\   CC(n-1,l)*l              \\ onto a leaf for L same
\\   + CC(n-1,l-1)*(n-1-l);   \\ onto non-leaf for L increase
\\ }
\\ for(n=1,6, my(t=0); for(l=1,max(6,n), my(c=CC(n,l)); print1(" "c); t+=c); \
\\            print(" total "t));


\\ n vertices, l leaves
C_list(n,l) =
{
  my(debug=0);
  if(debug, print("C_list "n" "l));
  if(l==0 && n==0, return([ [] ]));
  if(l==1 && n==1, return([ [0] ]));
  if(n<=0 || n<=l, return([]));
  
  my(ret=List([]));
  if(n==l,
     listput(ret, vector(n)));  \\ f=0, l many leaves
  for(f=1,l,
     if(debug, print("C_list "n" "l" f="f" descend"));
     my(above=C_list(n-l,f));
     if(debug, print("C_list "n" "l"  f="f"  above "above));
     for(a=1,#above,
        my(vpar=above[a]);
        vpar_num_childless(vpar)==f || error();
        vpar_childless_vertices(vpar)==[1..f] || error("childless wrong");

        vpar=vpar_concat_forest(vector(l),vpar);

        if(debug, print("iterate ",vector(f,i,[l+1..l+f])));
        forvec(low=vector(l,i, [l+1,n]),
              if(debug, print(" low ",low));
              my(occ=vector(n));
              for(i=1,#low, occ[low[i]]=1);
              if(debug, print(" occ ",occ));
              for(i=l+1,l+f, if(occ[i]==0,next(2)));

              for(i=1,#low, vpar[i]=low[i]);
              if(debug, print(vpar));
              vpar_validate(vpar);
              vpar_childless_vertices(vpar)==[1..l]||error("l childless wrong");
              listput(ret,vpar));
     ));
   Vec(ret);
}

C_list_by_search(n,l) =
{
  my(debug=0,
     idseq=vpar_INTERNAL_identity_perm(n),
     count=0,
     ret=List([]));
  forvec(vec=vpar_forvec_from(n,1), \\ forest
        my(vpar=vpar_from_upascend(vec));
        if(debug,
           print("vpar ",vpar,
                 " childless ",vpar_num_childless(vpar),
                 " seq "vpar_seq_subtree_height(vpar)));
        if(vpar_num_childless(vpar)==l
           && vpar_roots(vpar)==[#vpar]
           && vpar_seq_subtree_height(vpar)==idseq,
           listput(ret,vpar)));
  Vec(ret);
}

S_list(n) = concat(vector(n,l, C_list(n,l)));

C_list(3,2)
C_list_by_search(3,2)
\\quit
for(n=1,6, for(l=1,5, print1(" "#C_list(n,l))); print());
print();
for(n=1,6, for(l=1,5, print1(" "#C_list_by_search(n,l))); print());
