# Copyright 2018, 2019 Kevin Ryde

# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  See the file COPYING.  If not, see
# <http://www.gnu.org/licenses/>.


# Read output from dreadnaut "x" command and write GP code setting
# automorphism generators and info.


use 5.005;
use strict;
use FindBin;
use File::Spec;
use File::Slurp;
use File::Temp;
use IPC::Run;

# uncomment this to run the ### lines
# use Smart::Comments;

$|=1;

my @generators;
while (defined (my $line = readline)) {
  ### $line
  if ($line =~ /^\(\d/) {
    $line =~ tr/ ()/,[]/;
    $line =~ s/\]\[/],[/g;
    $line =~ s/\n//g;
    push @generators, $line;

  } elsif ($line =~ /(\d+) orbit.*?grpsize=([0-9.e]+).*?(\d+) gen/) {
    # grpsize is per Nauty writegroupsize() so is either an exact integer,
    # or a float like 1.551121004333e25
    my $num_orbits = $1;
    my $num_automorphisms = $2;
    my $num_generators = $3;
    my $num_automorphisms_is_exact = ($num_automorphisms =~ /e/ ? 0 : 1);
    ### $num_generators

    unless ($num_generators == scalar(@generators)) {
      print("error()");
      die "oops, ",scalar(@generators)," generators read doesn't match $num_generators gens reported";
    }
    print "dreadnaut_num_orbits        = $num_orbits;\n";
    print "dreadnaut_num_automorphisms = $num_automorphisms;\n";
    print "dreadnaut_num_automorphisms_is_exact = $num_automorphisms_is_exact;\n";
  }
}

### @generators

print "dreadnaut_generator_cycles = {[\n";
foreach my $i (0 .. $#generators) {
  print '[', $generators[$i], ']',
    ($i==$#generators ? '' : ','),
     "\n";
}
print "]};\n";
exit 0;
