\\ Copyright 2017, 2018, 2019, 2020, 2021, 2022 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


\\ Whether to print test progress and names.
if(option_verbose=='option_verbose, option_verbose=0);

\\ Whether to setrand(), default 1 = yes.
if(option_randomize=='option_randomize, option_randomize=1);

\\ Whether to print test section names and progress times.
if(option_progress=='option_progress, option_progress=0);

\\ Intended limit size of #vpar for running exhaustive searches over all
\\ 2^#vpar flags combinations for sets etc.
if(test_limit_flags_search=='test_limit_flags_search, \
   test_limit_flags_search = 10);

\\ Intended limit size of #vpar for running exhaustive searches over all
\\ n! permutations etc.
if(test_limit_perms_search=='test_limit_perms_search, \
   test_limit_perms_search = 8);
\\ GP-Test  8! == 40320

\\ Intended limit size of #vpar for running exhaustive searches over all
\\ n^(n-1) trees etc.
if(test_limit_trees_search=='test_limit_trees_search, \
   test_limit_trees_search = 6);
\\ GP-Test  my(n=6); n^(n-1) == 7776

\\ Intended limit number of sets when making vpar_indsets() and similar.
\\ Number of sets from those can become big.
if(test_limit_sets=='test_limit_sets, \
   test_limit_sets = 10000);



\\-----------------------------------------------------------------------------

iferr(read("test-oeis-samples.gp"),e, \
   read("../test-oeis-samples.gp"));

check_count = 0;

progress_total_time = 0;
progress(str) =
{
  if(option_progress,
     my(t=gettime());
     progress_total_time += t;
     printf("  %.1f seconds\n%s\n", t/1000, str));
}

\\ test_is_gnil() from test_is_gnil.c compiled to ./test_is_gnil.so, or ../
\\ if running from devel subdir.
\\ If install() not possible or compile not possible, then fall-back to
\\ nothing is gnil.
\\ See comments in test_is_gnil.c on usage.
test_is_gnil;
kill(test_is_gnil);
{
  if(default(secure)
     || (iferr(install(test_is_gnil,"G",,"./test_is_gnil.so"); 0,
               e, 1)
         && iferr(install(test_is_gnil,"G",,"../test_is_gnil.so"); 0,
                  e, 1)),
     have_test_is_gnil = 0;
     test_is_gnil(x) = 0;
     ,
     have_test_is_gnil = 1);
}

\\ Check that got===want.
\\ This is the strict type "===" operator so as to ensure say polynomial
\\ returns really are always polynomials, etc.
\\
\\ ENHANCE-ME: Would like to report caller's source file name and line
\\ number.  For now some identifying "name" parameter only.
\\
check_equal(got,want,name="") =
{
  my(bad=0);
  check_count++;

  if(test_is_gnil(got),
     print("oops, got==gnil at "check_count": "name);
     bad=1);
  if(test_is_gnil(want),
     print("oops, want==gnil at "check_count": "name);
     bad=1);

  if(type(got)=="t_VEC" || type(type)=="t_COL",
     for(i=1,#got,
        if(test_is_gnil(got[i]),
           print("oops, element got["i"]==gnil at "check_count": "name);
           bad=1;
           break)));
  if(type(want)=="t_VEC" || type(type)=="t_COL",
     for(i=1,#want,
        if(test_is_gnil(want[i]),
           print("oops, element want["i"]==gnil at "check_count": "name);
           bad=1;
           break)));

  if(type(got)=="t_MAT",
     for(i=1,matsize(got)[1],
        for(j=1,matsize(got)[2],
           if(test_is_gnil(got[i,j]),
              print("oops, element got["i","j"]==gnil at "check_count": "name);
              bad=1;
              break(2)))));
  if(type(want)=="t_MAT",
     for(i=1,matsize(want)[1],
        for(j=1,matsize(want)[2],
           if(test_is_gnil(want[i,j]),
              print("oops, element want["i","j"]==gnil at "check_count": "name);
              bad=1;
              break(2)))));

  if(! (got === want),
     print("wrong at "check_count": "name);
     print("got  "got);
     print("want "want);
     if(got==want,
        print("got type  "type(got));
        print("want type "type(want));
        if(type(got)=="t_POL" || type(got)=="t_RFRAC",
           print("got variable is  "variable(got)));
        if(type(want)=="t_POL" || type(want)=="t_RFRAC",
           print("want variable is "variable(want))));
     bad=1);

  if(bad,
     quit(1),
     option_verbose,
     print("ok "check_count" "name));
}

\\ randomize by getwalltime() in gp 2.9 up, or C library time() in gp 2.7
test_have_getwalltime = iferr(getwalltime();1,e,0);
if(option_verbose, print("test_have_getwalltime = "test_have_getwalltime));
test_have_C_library_time=0;
{
  if(!test_have_getwalltime,
     if(default(secure),
        if(option_verbose, print("not trying install() in secure mode"));
        ,
        test_have_C_library_time
          = iferr(install(time, "lf", "test_C_library_time");1,e,0));
     if(option_verbose, print("test_have_C_library_time = "test_have_C_library_time)));
}
{
  if(option_randomize,
     my(seed=1);
     if(test_have_getwalltime,
        seed=getwalltime()%2^32;
        print("setrand() seed="seed" (from getwalltime())");
        ,
        test_have_C_library_time,
        seed=test_C_library_time();
        print("setrand() seed="seed" (from time())");
        ,
        print("setrand() seed="seed" (no getwalltime(), and no install() for C time())"));
     setrand(seed));
}

\\ Uncomment this to force a particular seed, such as printed above.
\\  setrand(1470495675);


\\-----------------------------------------------------------------------------

{
  test_sample_trees
  = concat([
            [[],
             [0],
             [2,0],
             [2,3,0]
            ],
            vector(11,i, vpar_make_random(i+2)),
            vector(7,i, vpar_make_random(i+5))
           ]);
}
{
  test_sample_forests
  = concat([
            [[],
             [0],
             [2,0],
             [0,0],
             [2,3,0],
             [0,3,0]
            ],
            vector(11,i, vpar_make_random(i+2, 1)),  \\ forest
            vector(7,i, vpar_make_random(i+5, 1))   \\ forest
            ]);
}
test_sample_trees_and_forests = \
  Set(concat(test_sample_trees,test_sample_forests));

{
  for(i=1,#test_sample_trees,
     my(vpar=test_sample_trees[i]);
     vpar_validate(vpar);
     check_equal(vpar_is_cyclic(vpar), 0));
}
{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i]);
     vpar_validate(vpar);
     check_equal(vpar_is_cyclic(vpar), 0));
}

{
  test_sample_preorder_forests
  = concat([
            [[],
             [0],
             [0,1],
             [0,0],
             [0,1,1],
             [0,1,2]
            ],
            vector(11,i, vpar_make_random_preorder(i+2, 1)),  \\ forest
            vector(7,i, vpar_make_random_preorder(i+7, 1))   \\ forest
            ]);
}
{
  for(i=1,#test_sample_preorder_forests,
     my(vpar=test_sample_preorder_forests[i]);
     vpar_validate(vpar);
     check_equal(vpar_is_cyclic(vpar), 0);
     check_equal(vpar_is_preorder(vpar), 1));
}

{
  test_sample_downwards_forests
  = concat([
            [[],
             [0],
             [0,1],
             [0,0],
             [0,1,1],
             [0,1,2]
            ],
            vector(11,i, vpar_make_random_downwards(i+2, 1)),  \\ forest
            vector(7,i, vpar_make_random_downwards(i+7, 1))   \\ forest
            ]);
}
{
  for(i=1,#test_sample_preorder_forests,
     my(vpar=test_sample_preorder_forests[i]);
     vpar_validate(vpar);
     check_equal(vpar_is_cyclic(vpar), 0);
     check_equal(vpar_is_preorder(vpar), 1));
}

\\ FIXME: This is not a uniform distribution.
\\ Maybe vpar_make_random_unlabelled(), perhaps when have an ith for that.
test_make_random_unlabelled(n,forest) = \
  vpar_to_unlabelled(vpar_make_random_preorder(n,forest));

{
  test_sample_unlabelled_forests
  = concat([
            [[],
             [0],
             [0,1],
             [0,0],
             [0,1,1],
             [0,1,2]
            ],
            vector(11,i, test_make_random_unlabelled(i+2, 1)),  \\ forest
            vector(7,i, test_make_random_unlabelled(i+7, 1))   \\ forest
            ]);
}
{
  for(i=1,#test_sample_unlabelled_forests,
     my(vpar=test_sample_unlabelled_forests[i]);
     vpar_validate(vpar);
     check_equal(vpar_to_unlabelled(vpar), vpar,
       "test_sample_unlabelled_forests is canonical vpar_to_unlabelled()"));
}

{
  test_sample_cyclics
  = concat([
            [[1],
             [2,1],
             [2,3,1],
             [2,3,4,5,6, 4, 0]
            ],
            vector(11,n,n+=3;
                   my(vpar);
                   until(vpar_is_cyclic(vpar),
                         vpar=vector(n,v, random(n+1)));
                   vpar)
            ]);
}
{
  for(i=1,#test_sample_cyclics,
     my(vpar=test_sample_cyclics[i]);
     check_equal(vpar_is_cyclic(vpar), 1,
                "test_sample_cyclics is vpar_is_cyclic()"));
}

test_sample_forests_and_cyclics = \
  Set(concat(test_sample_forests,test_sample_cyclics));

test_make_random_unicyclic(n) =
{
  \\ ENHANCE-ME: how to select parents for uniform unicyclic?
  \\ Having made a cycle, must not make another.
  my(vpar);
  until(vpar_num_components(vpar)<=1,
        vpar=vector(n,v, random(n)+1));
  vpar;
}
{
  test_sample_trees_and_unicyclics
  = concat(test_sample_trees,
            vector(15,n, test_make_random_unicyclic(n+1)));
}


\\-----------------------------------------------------------------------------
\\ Fake random() Returning Specific Values

\\ test_random() returns values from test_random_list as a way to force
\\ random numbers to a given desired values.
\\
\\ The first returned is test_random_list[1] and it deleted from the list,
\\ and so on.  Each limit parameter to test_random() must be equal to the
\\ values in test_random_limit_list, so first test_random_limit_list[1]
\\ which is then deleted, etc.
\\
\\ test_random() throws an error if test_random_list is empty (run out of
\\ values).
\\
test_random_list=List([]);
test_random_limit_list=List([]);
test_random(limit) =
{
  \\ print("test_random() limit="limit);
  \\ print("  test_random_list        "test_random_list);
  \\ print("  test_random_limit_list  "test_random_limit_list);

  #test_random_list || error("test_random() no more values");
  #test_random_list == #test_random_limit_list
    || error("oops, different lengths");

  my(r=test_random_list[1],
     want_limit=test_random_limit_list[1]);
  listpop(test_random_list,1);
  listpop(test_random_limit_list,1);

  limit==want_limit
    || error("test_random() limit=",limit," but want_limit=",want_limit,
            " (continuing ",test_random_limit_list);
  (0<=r && r<limit)
    || error("test_random() limit=",limit," but list r=",r);
  r;
}

\\ func is a function (t_CLOSURE).
\\ Return a string which is its definition with call to random() changed to
\\ test_random().  The string can be eval() to make the changed func.
\\
test_random_to_test_random(func) = \
  test_string_replace(Str(func), "random(", "test_random(");

\\ Return string "str" with all occurrences of string "old" changed to
\\ "new".
test_string_replace(str,old,new) =
{
  old=Vecsmall(old);
  #old >= 1 || error("test_string_replace() is for non-empty old");
  str=Vecsmall(str);
  my(i=1);
  while(i+#old-1 <= #str,
        \\ print("i="i"  "Strchr(str)"  compare "Strchr(str[i..i+#old-1]));
        if(str[i..i+#old-1] == old,
           \\ avoid empty ranges for gp 2.7.x
           str=concat([if(i>1,         str[1..i-1], Vecsmall([])),
                       Vecsmall(new),
                       if(i+#old<=#str, str[i+#old..#str], Vecsmall([]))
                      ]);
           i += #new;
           ,
           i++));
  \\ print("final i="i" "Strchr(str));
  Strchr(str);
}


\\-----------------------------------------------------------------------------

\\ poly is a polynomial.
\\ Return the coefficient c of its lowest non-zero term c*x^n.
\\ If poly==0 then return 0.
test_poly_low_coefficient(poly) =
  if(poly, polcoeff(poly,valuation(poly,variable(poly))), 0);

\\ flags is a vector (or Vecsmall) of 0 or 1
\\ return a vector of those v where flags[v]==target
\\ the return is sorted so is a Set()
test_flags_to_Set(flags,target=1) = \
  select(v->flags[v]==target, [1..#flags]);

test_set_is_subset(subset,superset) = setminus(subset,superset)==[];

\\ v is a vector of Set()s of vertex numbers.
\\ Return the setintersect() of them all.
test_setintersect_sets(v) = my(s=v[1]); for(i=2,#v,s=setintersect(s,v[i])); s;

\\ Return the number of real roots of poly, with duplicates counted.
\\ Same as #polrootsreal(poly), that function being new in gp 2.9.
\\ Cf polsturm() which is the count of distinct real roots.y
test_poly_num_real_roots(poly) =
{
  \\ FIXME: Is this right?  Factorizing over rationals gives no repeat roots
  \\ within a factor.
  my(m=factor(poly));
  sum(i=1,matsize(m)[1], if(m[i,1], polsturm(m[i,1]) * m[i,2]));
}
test_poly_is_all_real_roots(poly) = \
  test_poly_num_real_roots(poly)==poldegree(poly);


\\-----------------------------------------------------------------------------
\\ Vpar Things

\\ Return 1 if vpar is isomorphic (vpar_is_isomorphic()) to a rooted path,
\\ with the root as one end of the path.
test_is_path(vpar) = vpar_num_childless(vpar)<=1;

\\ Return 1 if vpar is free isomorphic (vpar_is_free_isomorphic()) to a path,
\\ so root anywhere.
test_is_free_path(vpar) =
{
  \\ two ends degree=1, rest degree=2
  my(degrees=vpar_degrees_vector(vpar));
  degrees==[] || degrees==[0]
  || vecsort(degrees) == vector(#vpar,i, if(i<=2,1,2));
}

\\ Return 1 if vpar is free isomorphic (vpar_is_free_isomorphic()) to a star.
test_is_free_star(vpar) =
{
  my(degrees=vpar_degrees_vector(vpar));
  degrees==[] || degrees==[0]
  || vecsort(degrees) == vector(#vpar,i, if(i<#vpar,1,#vpar-1));
}

\\ PENDING: maybe vpar_make_cycle(), but want the t tail?
\\
\\  +- 1 <- 2 <- 3 <- .. <-- n <- n+1 <- ... <- t
\\  |                        ^
\\  +------------------------+
\\
test_make_cycle(n,t=0) = vector(n+t,i,if(i==1,n,i-1));
{
addhelp(test_make_cycle,
"vpar = test_make_cycle(n,t=0)
Return a cycle of n vertices.
Vertex 1 has parent n and vertex v>1 has parent v-1.  This is the same
parent directions as vpar_make_path(), but 1 has parent cycle back to n.
For example n=5 is [5,1,2,3,4].  n=0 is empty [].  n=1 is self loop [1].
Only some functions accept cyclic vpar.

Optional parameter \"t\" is how many tail vertices under the cycle.
Each v = n+1,...,t has parent v-1.  If n=0 then the return is this tail
path alone and is the same as vpar_make_path(t).");
}

\\ PENDING: vpar_delete_edge() in vpar-wip.gp, maybe
test_delete_edge(vpar,u,v) =
{
  if(vpar[u]==v, vpar[u]=0,
     vpar[v]==u, vpar[v]=0);
  vpar;
}

\\ Return 1 if vpar is bicentral and its two halves are free isomorphic, and
\\ those halves in turn are bicentral isomorphic halves, and so on all the
\\ way down.
\\ Optional parameter "same" can be 1 to demand that halves are connected at
\\ the same vertex in each half, meaning similar, ie. in the same
\\ automorphism orbit.
test_is_isomorphic_halves(vpar,same=0) =
{
  vpar_is_tree(vpar) || return(0);
  while(1,
        if(#vpar<=1, return(1));
        my(c=vpar_centres(vpar));
        #c==2 || return(0);

        vpar=test_delete_edge(vpar,c[1],c[2]);
        my(vpars=vpar_component_trees(vpar));
        #vpars==2 || error("oops, centres ",c," split to "vpars);

        vpar_is_free_isomorphic(vpars[1],vpars[2]) || return(0);
        if(same,
           my(orbitnum=vpar_free_orbitnum_vector(vpar));
           if(orbitnum[c[1]] != orbitnum[c[2]],
              return(0)));

        vpar=vpars[1]);
}

test_concat_forest_vector(vec) =
{
  if(#vec,
     my(vpar=concat(vec),
        offset=0);
     for(i=2,#vec,
        offset += #vec[i-1];
        for(j=offset+1, offset+#vec[i],
           if(vpar[j], vpar[j] += offset)));
     vpar;
     ,
     []);
}
\\ {
\\ addhelp(vpar_concat_forest_vector,
\\ "vpar = vpar_concat_forest_vector(vec)
\\ vec is a vector of vpars.  Return a new vpar which is their concatenation,
\\ making a forest.  Each vec[] can be a forest too.
\\
\\ The return has vec[1] first, then vec[2] with offset +#vec[1] on its vertex
\\ numbers, then vec[3], etc.  If vec is empty or consists entirely of empty
\\ vectors then the returned vpar==[] empty.");
\\ }

\\ Return 1 if vpar is a Motzkin type forest.
\\ This means each childless vertex cannot have a bigger sibling.
\\ In the natural correspondence between ordered forests and binary trees
\\ this is everywhere left subtree absent must also have right subtree absent.
test_is_Motzkin_forest(vpar) =
{
  \\ left absent must right absent
  my(next_sibling=vpar_next_sibling_vector(vpar),
     first_child=vpar_minmax_child_vector(vpar,min));
  for(v=1,#vpar,
     if(!first_child[v] && next_sibling[v], return(0)));
  1;
}

\\ like vpar_indnum(), but by searching sets of flags
test_indnum_by_flags(vpar) =
{
  my(ret=0);
  forvec(flags=vector(#vpar,i,[0,1]),
        if(vpar_is_indset_flags(vpar,flags),
           ret=max(ret,hammingweight(flags))));
  ret;
}

test_domnum_and_count_by_flags(vpar) =
{
  if(#vpar==0,return([0,1]));  \\ empty set only
  my(domnum=#vpar,count=0);
  forvec(flags=vector(#vpar,i,[0,1]),
        if(vpar_is_domset_flags(vpar,flags),
           my(size=vecsum(flags));
           if(size<domnum,  domnum=size;count=1,  \\ new low
              size==domnum, count++)));           \\ equal low
  [domnum,count];
}


\\-----------------------------------------------------------------------------
\\ Number Things

\\ Return 1 if integer n is a power of 2.
test_is_pow2(n) = n>0 && !bitand(n-1,n);

\\ Multinomial coefficient of vec[1..n].
test_multinomial(vec) = vecsum(Vec(vec))! / prod(i=1,#vec,vec[i]!);

\\ Padovan numbers (A000931)
test_Padovan(n) = ([0,1,0; 0,0,1; 1,1,0]^n * [1,0,0]~) [1];

\\ Narayana number triangle (A001263).
test_Narayana(n,k) =
{
  (n>=0 && k>=0) || error("test_Narayana() is for n>=0 and k>=0");
  if(k==0,(n==0), binomial(n,k-1)*binomial(n-1,k-1)/k);
}

\\ number of points in a triangle of t many rows
test_triangular(t) = t*(t+1)/2;

\\ Return 1 if n is a triangular number n = t*(t+1)/2.
\\ A023532 except for offset
test_is_triangular(n) =
{
  \\ t^2 + t - 2n = 0
  \\ t = (-1 +/- sqrt(1 + 8n))/2
  \\ 2t = -1 +/- sqrt(1 + 8n)
  \\ sqrt(1 + 8n) = 2t + 1
  my(r); issquare(8*n+1,&r) && r%2;
}

\\ Return the t of triangular number which is <= n.
\\ So test_triangular(t) <= n < test_triangular(t+1)
test_untriangular_floor(n) =
{
  \\ t^2 + t - 2n = 0
  \\ t = (-1 +/- sqrt(1 + 8n))/2
  (sqrtint(8*n+1)-1) >> 1;
}

\\ Return the t of triangular number which is >= n.
\\ So test_triangular(t-1) < n <= test_triangular(t)
test_untriangular_ceil(n) = \
  if(n, (1 + sqrtint(8*n-7)) >> 1,  0);

\\ same as fromdigits(), but that not available in GP 2.7.x
test_fromdigits(vec,base=10) = subst(Pol(vec),'x,base);

\\ n is an integer whose decimal digits are all 0 or 1.
\\ Return them as an integer interpreted as binary.
test_from_binary(n) = test_fromdigits(digits(n),2);

test_to_binary(n)   = test_fromdigits(digits(n,2));

test_to_base4(n) = test_fromdigits(digits(n,4));
test_from_base4(n) = test_fromdigits(digits(n),4);

\\ Return n with its bits reversed and flipped 0<->1  = A036044.
test_bit_complement_and_reverse(n) = \
  test_fromdigits(apply(b->!b, Vecrev(binary(n))), 2);

\\ Return remainder r = n mod d, with division rounded to the nearest
\\ integer quotient.  So -d/2 < r <= d/2
test_mod_nearest(n,d) =
{
  my(v=divrem(n,d), r=v[2]);
  if(r>d\2, r-d, r);
}

\\ test_ordered_Bell(n) returns the number of ways to distribute integers
\\ 1..n into an ordered list of buckets.  Each bucket is non-empty, and any
\\ number of buckets (1 to n).
\\ See also vpar_depths_vectors_count() which for forests is this count.
\\
test_ordered_Bell(n) = sum(k=0,n,k!*stirling(n,k,2));  \\ A000670

\\ product 1*3*5*7*... for n many terms, A001147
test_odd_double_factorial(n) =
{
  \\ zero or more terms makes sense, not sure what negative would look like
  n>=0 || error("test_odd_double_factorial() is for n>=0");
  prod(i=1,n, 2*i-1);
}


\\-----------------------------------------------------------------------------
\\ Vector Things

\\ Return 1 if vector v is in increasing order, or rather non-decreasing
\\ order since duplicate elements are allowed as long as no decreases.
test_vector_is_increasing(vec) = (vec==vecsort(vec));

\\ Return 1 if vector v is in decreasing order, or rather non-increasing
\\ order since duplicate elements are allowed as long as no increases.
test_vector_is_decreasing(vec) = (vec==vecsort(vec,,4));

\\ Return 1 if vector vec has all elements different, as compared by cmp().
test_vector_is_all_different(vec) = (#Set(vec) == #vec);

\\ Return 1 if vector vec has all elements same, as compared by cmp().
test_vector_is_all_same(vec) =
{
  if(#vec, for(i=2,#vec, if(vec[i]!=vec[1], return(0))));
  1;
}

\\ Return 1 if vector vec contains "elem", as compared by ==.
test_vector_search(vec,elem) = for(i=1,#vec,if(vec[i]==elem,return(i))); 0;

\\ vec is a vector of vectors.
\\ Return it sorted by vecmin(element).
test_vecsort_vecmin(vec,flags=0) = test_vecsort_by_key_func(vec,vecmin,flags);

\\ vec is a vector.
\\ Return vec sorted in order of keys obtained by func called
\\ key=func(vec[i]).  The keys are sorted by usual vecsort lex().
test_vecsort_by_key_func(vec,func,flags=0) = \
  vecextract(vec, vecsort(apply(func,vec),,bitor(flags,1)));

\\ Return element number i from vector v.
\\ The first element is i=0 and at the end of the vector
\\ i is taken modulo length(v).
test_vector_modulo(v,i) = v[(i% #v)+1];

\\ Product v[1]*v[2]*...*v[#v].
\\ Calculated here starting v[1] so the elements can be permutations,
\\ as opposed to prod() which starts with 1 so doesn't accept perms.
test_vector_product(v) =
{
  if(#v==0,return(1));
  my(ret=v[1]);
  for(i=2,#v, ret=ret*v[i]);
  ret;
}

\\ Return vec with its elements randomly permuted.
test_vector_ranperm(vec) = \
  vecextract(vec, vpar_INTERNAL_random_perm(#vec));

\\ Return vec with "value" inserted after position "pos", so vec[pos+1]=value.
\\ pos=1 means after the first element in vec, etc.
\\ pos=0 means before all of vec.
\\ cf Pari library vec_insert(), where new elem stored at vec[pos].
test_vector_insert_after(vec,pos,value) =
{
  \\ gp range extracts have become better at zero tripping, but want to work
  \\ in 2.7.x
  if(pos<=0,    concat(value, vec),
     pos>=#vec, concat(vec, value),
     concat([vec[1..pos], value, vec[pos+1..#vec]]));
}

\\ Return a vector [len1, len2, ...] which are run lengths of values in vec.
test_vector_run_lengths(vec) =
{
  if(#vec==0, return([]));
  my(value=vec[1],len=1,
     ret=List([]));
  for(i=2,#vec,
     if(value==vec[i], len++,
        listput(ret,len); value=vec[i]; len=1));
  listput(ret,len);
  Vec(ret);
}


\\-----------------------------------------------------------------------------
\\ Partitions

\\ p is a vector which is a partition vecsum(p)==n (as from partitions()).
\\ Return it in the form of a vector [ [t,rep], [t,rep], ... ] where each t
\\ is an element of the partition and rep is how many times it occurs in the
\\ partition.
test_partition_to_repetitions(p) =
{
  my(l=List([]));
  for(i=1,#p, if(#l&&l[#l][1]==p[i], l[#l][2]++, listput(l,[p[i],1])));
  Vec(l);
}


\\-----------------------------------------------------------------------------
\\ Compositions (Partitions with Order)

\\ Return the number of compositions of k (with unrestricted composition
\\ lengths and terms).
\\
test_num_compositions(k) = 2^max(0,k-1);

\\ Return a vector of compositions.  Args are like partitions(), but whereas
\\ partitions() has elements in ascending order, the compositions return is
\\ elements in all possible distinct permutations too.
\\
test_compositions(k,a=k,n=k) =
{
  my(v=partitions(k,a,n));
  my(l=List(Set(v)));
  for(i=1,#v,
     for(p=1,(#v[i])!-1,
        my(c=vecextract(v[i], numtoperm(#v[i],p)),
           pos=setsearch(l,c,1));
        if(pos, listinsert(l,c,pos))));
  Vec(l);
}


\\-----------------------------------------------------------------------------
\\ Ordered Partitions of 1..N

\\ vec = test_make_ordered_partitions(n)
\\ Return a vector of all ordered partitions of the numbers 1..n.
\\ This is a vector of vectors of vectors.
\\
\\ A partition of the numbers 1..n is a vector like [ [3,1], [2,4,5] ] which
\\ has all of 1..n put into element vectors.  The partition has the element
\\ vectors in order of their smallest entry.
\\
\\ Ordered partition here means the element vectors have the numbers in an order.
\\ The return is all possible orderings of all possible partitions.
\\ Return count #vec is OEIS A000262 num ways 1..n in ordered lists.
\\
test_make_ordered_partitions(n) =
{
  my(ret=[ [] ]);
  for(d=1,n,
     my(new_ret=List([]));
     for(i=1,#ret,
        my(old=ret[i]);
        for(pos=1,#old,  \\ d extending one of the blocks in old
           my(old_block=old[pos]);
           for(t=0,#old_block,
              my(new_block=test_vector_insert_after(old_block,t,d),
                 new=old);
              new[pos]=new_block;
              listput(new_ret,new)));
        listput(new_ret,concat(old,[[d]])));  \\ d new set
     ret=new_ret);
  Vec(ret);
}

\\ vec is an ordered partition in the form of a vector of vectors
\\ like [ [1,4], [5,2,3] ].
\\ Check that it is in valid form meaning all integers 1..n occurring,
\\ and the ordered lists in order of their smallest member.
\\
test_ordered_partition_validate(vec) =
{
  vecsort(vec, (x,y)->lex(vecmin(x),vecmin(y))) == vec
    || error("test_ordered_partition_validate() not sorted: ",vec);
  if(#vec, vec=concat(vec));
  #Set(vec) == #vec || error("test_ordered_partition_validate() duplicates");
  #vec==0 || vecmin(vec)==1 || error("test_ordered_partition_validate() min is ",vecmin(vec));
  #vec==0 || vecmax(vec)==#vec || error("test_ordered_partition_validate() max is ",vecmax(vec));
}

\\ A105278(n,k) = number of partitions of 1..n into k many ordered lists
test_A105278(n,k) = binomial(n,k) * binomial(n-1,k-1) * (n-k)!;

\\ A005651(n) = sum multinomials over partitions
\\   = ways to partition 1..n into sets and those sets in an ordered list
\\     of non-decreasing set size
\\
test_A005651(n) = vpar_subtree_heights_vectors_count(n,1);


\\-----------------------------------------------------------------------------
\\ Permutation Helpers

\\ vec is a vector of integers.
\\ Return 1 if vec is a permutation, so integers 1..#vec occurring once each.
\\ cf Pari library static isperm() in perm.c.
test_is_perm(vec) =
{
  my(seen=vector(#vec));
  for(i=1,#vec,
     if(type(vec[i])!="t_INT" || vec[i]<1 || vec[i]>#vec || seen[vec[i]],
        return(0));
     seen[vec[i]]++);
  1;
}

test_perms_validate(perms) =
{
  for(i=1,#vec,
     if(!test_is_perm(perms[i]),
        error("bad perms")));
}

\\ Same as libpari identity_perm(), but avoid depending on a dynamic library
\\ install() for tests.
test_identity_perm(n) = vectorsmall(n,i,i);

\\ Return the reversing permutation of n elements, sending 1..n to n..1.
test_perm_reversal(n) = vectorsmall(n,i, n-i+1);

\\ Return the permutation rotating n things upwards, so perm[1]=1+r.
test_perm_rotate_up(n,r=1) = vectorsmall(n,i, ((i-1+r)%n)+1);


\\ test_perm_commute(p,q) returns 1 if permutations p,q commute, so p*q==q*p.
\\ This is the Pari library perm_commute(), or own code if no install().
test_perm_commute;
kill(test_perm_commute);
{
  if(! if(! default(secure),
          iferr(install(perm_commute, "iGG", "test_perm_commute"); 1,
                e, 0)),
     test_perm_commute(p,q) = p*q==q*p);
}

\\ perm is a permutation.
\\ Return the order of element i, so the length of the cycle containing i.
test_perm_elem_order(perm,i) =
{
  my(t=i,len=0);
  until(t==i, len++; t=perm[t]);
  len;
}

\\ perm is a permutation.
\\ Return a vector where each vec[i] = test_perm_elem_order(perm,i) but
\\ calculated all together.
test_perm_elem_orders_vector(perm) =
{
  my(ret=vector(#perm));
  for(i=1,#perm,
     if(!ret[i],
        my(len=test_perm_elem_order(perm,i));
        my(t=i); until(t==i, ret[i]=len; t=perm[t])));
  ret;
}

\\ perm is a permutation.
\\ Return 1 if it is "even", by examining its cycle lengths.
test_perm_is_even_by_cycles(perm) =
{
  my(cycles=test_perm_to_cycles(perm));
  sum(i=1,#cycles, (#cycles[i])%2==0)  \\ even length is odd set transpositions
  % 2 == 0;
}
\\ test_perm_is_even(perm) returns 1 if permutation perm is "even", so it
\\ can be expressed as an even number of element swaps.  Or return 0 if "odd".
\\ Same as permsign(perm)==1 (which is new in gp 2.11 up).
test_perm_is_even =
{
  if(iferr(permorder(Vecsmall([2,1]))==2,e,0),
     (perm) -> permsign(perm)==1,
     test_perm_is_even_by_cycles);
}

\\ test_permorder(perm) = permorder() when available, which is gp 2.11 up,
\\ or otherwise some code here.  perm must be a Vecsmall.
test_permorder;
kill(test_permorder);
{
  if(iferr(permorder(Vecsmall([2,1]))==2,e,0),
     alias(test_permorder,permorder),
     test_permorder = (perm) -> lcm(apply(length,test_perm_to_cycles(perm))));
}

\\ Return a vector of the distinct powers vec = [perm^0, perm^1, perm^2, ... ].
\\ Length #vec == permorder(perm).
test_perm_powers(perm) =
{
  my(p=test_identity_perm(#perm),
     ret=vector(test_permorder(perm)));
  for(i=1,#ret, ret[i]=p; p *= perm);
  p == test_identity_perm(#perm) || error("oops");
  ret;
}

\\ Return perm resized to n elements.
\\ If n>#perm then new elements are unpermuted perm[i]=i.
test_perm_resize(perm,n) =
{
  \\ Special case for n==0 because Vecsmall won't chop down to 0.
  \\ Vecsmall([1,2],0)           == Vecsmall([1,2])
  \\ Vecsmall(Vecsmall([1,2]),0) == Vecsmall([1,2])

  if(n==0, Vecsmall([]),
     my(old=#perm);
     perm=Vecsmall(perm,n);
     for(i=old+1,n, perm[i]=i);
     perm);
}

\\ vec is a vector of permutations.
\\ Check each is in fact a permutation, and not the identity permutation.
test_no_identity_perm(vec,name="") =
{
  if(#vec,
     my(id=test_identity_perm(#vec[1]));
     for(i=1,#vec,
        test_is_perm(vec[i]) || error(name," not a permutation");
        if(vec[i]==id,
           error(name," should not include identity"))));
}

test_perm_to_cycles(perm) =
{
  my(ret=List([]),
     seen=vectorsmall(#perm));
  for(i=1,#perm,
     if(!seen[i],
        my(t=i, l=List([]));
        until(t==i,
              if(seen[t], error("oops, not a permutation"));
              seen[t]=1;
              listput(l,t);
              t=perm[t]);
        listput(ret,Vecsmall(l))));
  Vec(ret);
}
test_perm_to_cycles_sans_fixed(perm) = \
  select(c->#c>1, test_perm_to_cycles(perm));

\\ perms is a vector of permutations.
\\ Return a vector of those which have n fixed, perm[n]==n.
\\ If perms is a group then the result is a subgroup.
test_perms_fixing_n(perms,n) = select(perm->perm[n]==n, perms);

\\ Insert n into permutation "perm", increasing its length by 1.
test_perm_insert_n(perm,n) =
{
  perm = vectorsmall(#perm+1,i,
                     if(i!=n,
                        my(from=i-(i>n));
                        perm[from] + (perm[from]>=n)));
  perm[n]=n;
  perm;
}

\\ Delete n from permutation "perm", reducing its length by 1.
\\ The cycle containing n is shrunk.  perm[predecessor]==n is changed to
\\ the successor so perm[predecessor] = perm[n].
test_perm_delete_n(perm,n) =
{
  my(to=perm[n]);
  perm=perm[^n];
  for(i=1,#perm,
     if(perm[i]==n, perm[i]=to);
     if(perm[i]>n, perm[i]--));
  perm;
}

\\ perms is a vector of permutations.
\\ Return them with element n deleted per test_perm_delete_n().
test_perms_delete_n(perms,n) = \
  apply(perm->test_perm_delete_n(perm,n), perms);

\\ perm is a permutation.
\\ Return 1 if perm is a swap of 2 elements and everything else unchanged.
test_perm_is_swap1(perm) =
{
  my(seen=0);
  for(i=1,#perm,
     if(perm[i]!=i,
        if(perm[perm[i]]!=i, return(0));  \\ cycle >=3
        if(seen++ > 2, return(0))));      \\ more than 2 places of swap
  seen==2;
}

\\ vec is a vector of permutations, each the same size, which are generators
\\ of a permutations group.
\\ Return an orbitnum[] vector, so orbitnum[i] = orbit number of i.
\\ Orbits are numbered in order of their smallest member.
\\
test_perms_orbitnum_vector(perms) =
{
  \\ orbitmin[v] = minimum vertex number in the orbit of v, initially v itself.
  \\ When a perm[v]=u it means u,v are in the same orbit, so merge all in
  \\ their orbits to new orbitmin (min of the two orbitmins).
  \\ Doing this repeatedly until no more merges is effectively multiplying
  \\ perms like generators.
  \\ orbitmin[] is like a vpar structure with a self-loop at each minimum
  \\ and the rest children of whichever of minimums.

  if(#perms==0, return([]));
  my(n=#perms[1],
     orbitmin=test_identity_perm(n),
     stop);
  until(stop,
        stop=1;
        for(v=1,n,
           for(i=1,#perms,
              my(to=perms[i][v]);
              if(orbitmin[v] != orbitmin[to],
                 stop=0;
                 my(new=min(orbitmin[v], orbitmin[to]));
                 for(j=1,n,
                    if(orbitmin[j]==orbitmin[v] || orbitmin[j]==orbitmin[to],
                       orbitmin[j]=new))))));
  vpar_compnum_vector(orbitmin);
}


\\-----------------------------------------------------------------------------
\\ Groups of Permutations

\\ perms is a vector of permutations (Vecsmalls), all the same size.
\\ Return 1 if it is a group, so has identity and inverses and is closed so
\\ product of any two perms[i]*perms[j] is also in perms.
\\
test_perms_is_group(perms) = test_perms_ungroup_elem(perms)=='none;

\\ vec is a vector of permutations (Vecsmalls), all the same size.
\\ If it is a group (test_perms_is_group()) then return symbol 'none.
\\ If it is not a group then return a perm which is missing.
\\ This is some perm which is a product, inverse or identity not in vec.
\\
test_perms_ungroup_elem(vec) =
{
  #vec || error("test_perms_ungroup_elem() is for a non-empty set of perms");
  vec=Set(vec);
  my(id=test_identity_perm(#vec[1]));
  if(!setsearch(vec,id), return(id));

  for(i=1,#vec,
     my(inv=vec[i]^-1);
     if(!setsearch(vec,inv), return(inv));

     for(j=1,#vec,
        my(new_perm=vec[i]*vec[j]);
        setsearch(vec,new_perm) || return(new_perm)));
  'none;
}

\\ Return a vector of all permutations of length n, as Vecsmalls.

\\ This is symmetric group S(n).  The number of elements is n!.
test_perms_Sn(n) = vector(n!,i, Vecsmall(numtoperm(n,i-1)));

\\ Return a vector of all even permutations of length n, as Vecsmalls.
\\ See test_perm_is_even() on even permutations.
\\ This is alternating group A(n).  The number of elements is n!/2.
test_perms_An(n) = select(test_perm_is_even, test_perms_Sn(n));

\\ Return a vector of permutations, as Vecsmalls, which cycle n elements by
\\ 0 to n-1 places.  This is cyclic group C(n).  The number of elements
\\ is n.
test_perms_Cn(n) = vector(n,r, test_perm_rotate_up(n,r-1));

\\ Return a vector of permutations, as Vecsmalls, which are the cyclics per
\\ test_perms_Cn() and also their reversals.  This is the dihedral group of
\\ n elements, being symmetries of an n-sided polygon.  The number of
\\ elements is 2*n, for n>=3.
test_perms_Dn(n) =
{
  my(perms=test_perms_Cn(n));
  if(n<3, perms,
     my(r=test_perm_reversal(n));
     concat(perms, apply(p->r*p,perms)));
}

\\ perms is a vector of permutations.
\\ Return a vector of all n fixed (unchanged) by all of them,
\\ ie. all the test_perms_is_fixed_n(perms,n).
test_perms_fixed_points(perms) = \
  select(n->test_perms_is_fixed_n(perms,n), [1..#perms[1]]);

\\ perms is a vector of permutations.
\\ Return 1 if n is fixed (unchanged) by all of them.
test_perms_is_fixed_n(perms,n) = \
  for(i=1,#perms, if(perms[i][n]!=n, return(0))); 1;

\\ perms is a vector of permutations, all the same length.
\\ Return 1 if it is the full set of permutations S(n) for the perm length.
test_perms_is_Sn(perms) =
{
  forstep(n=#perms[1],1,-1,
     if(test_perms_is_fixed_n(perms,n),
        perms=test_perms_delete_n(perms,n)));
  vecsort(perms) == test_perms_Sn(#perms[1]);
}


\\ gens is a vector of permutations (Vecsmalls).
\\ Return a vector of permutations which is the group of permutations
\\ generated by gens, meaning all combinations of powers and products.
\\ If gens==[] empty then return empty [] likewise.
\\ If gens not empty, or if a size n>=0 is given, then the identity
\\ permutation is always included in the return.
test_perms_generators_to_group(gens,n=-1) =
{
  \\ print("test_perms_generators_to_group()");
  if(n>=0,
     gens=concat(gens,[test_identity_perm(n)]));
  if(!#gens, return([]));

  for(i=1,#gens,
     \\ print(" gens elem = "gens[i]);
     if(!test_is_perm(gens[i]),
        error("test_perms_generators_to_group() not a perm: "gens[i])));

  my(ret=List(Set(gens)),  \\ sorted
     more=1);
  while(more,
        more=0;
        for(i=1,#ret,
           my(perm=ret[i]);
           for(j=1,#gens,
              my(new_perm=perm*gens[j]);
              my(pos=setsearch(ret,new_perm,1));
              if(pos,
                 listinsert(ret,new_perm,pos);
                 more=1))));
  Vec(ret);
}

\\ perms is a vector of permutations.
\\ Return a matrix m[i,j]=k where perms[i]*perms[j] = perms[k], so
\\ identifying which of perms[] results from a product.  If a product is not
\\ in perms then m[i,j]=0.
test_perms_Cayley_table(perms) =
{
  my(idx=vecsort(perms,cmp,1),
     set=vecextract(perms,idx));
  matrix(#perms,#perms,i,j,
         my(p=setsearch(set,perms[i]*perms[j]));
         if(p,idx[p],0));
}

\\ vec is a vector of permutations (Vecsmall()s).
\\ All permutations must be the same size.
\\ Return 1 if they are minimal, meaning none can be removed and still
\\ generate the same perms by all powers and products.
test_perm_generators_is_minimal(vec) =
{
  \\ test_no_identity_perm(vec,"test_perm_generators_is_minimal");
  my(a=test_perms_generators_to_group(vec));
  for(i=1,#vec,
     if(a == test_perms_generators_to_group(vec[^i]),
        return(0)));
  1;
}

\\ "generated" is a List() of permutations sorted as for setsearch etc and
\\ closed under multiplication.
\\ Return the list extended by joining the given perm as a new generator
\\ element, so add all new perms which can be reached by products and powers
\\ of perm and anything in "generated".
\\
test_extend_generated_list(generated,perm) =
{
  \\ Vec(generated) == Set(generated)
  \\   || error("test_extend_generated_list() generated is not a set");

  listinsert(generated, perm, setsearch(generated,perm,1));
  my(more=1);
  while(more,
        more=0;
        for(i=1,#generated,
           for(j=1,#generated,
              my(new_perm = generated[i] * generated[j]);
              my(pos=setsearch(generated,new_perm,1));
              if(pos,
                 listinsert(generated,new_perm,pos);
                 more=1))));
  generated;
}
{
  check_equal(vecsort(test_extend_generated_list(List([Vecsmall([1,2,3])]),
                                                 Vecsmall([1,3,2]))),
            List([ Vecsmall([1, 2, 3]), Vecsmall([1, 3, 2]) ]));

  check_equal(vecsort(test_extend_generated_list(List([Vecsmall([1,2,3]),
                                                       Vecsmall([2,1,3])]),
                                                 Vecsmall([1,3,2]))),
            List([Vecsmall([1, 2, 3]), Vecsmall([1, 3, 2]),
                  Vecsmall([2, 1, 3]), Vecsmall([2, 3, 1]),
                  Vecsmall([3, 1, 2]), Vecsmall([3, 2, 1])]));
}

\\ perms = test_perms_minimum_generators(perms)
\\ perms is a vector of permutations (Vecsmalls) which is a group
\\ per test_perms_is_group().
\\ Return a minimum number of elements from perms which are generators (all
\\ elements are products and powers of the generators).
\\ There might be various possible minimum generator sets.  The one returned
\\ is unspecified.
\\
test_perms_minimum_generators(perms) =
{
  \\ print("test_perms_minimum_generators() "perms);
  \\ test_perms_is_group(perms)
  \\   || error("test_perms_minimum_generators() not closed: ",perms);
  \\ #perms==#Set(perms)
  \\   || error("test_perms_minimum_generators() perms has duplicates");
  if(#perms==0,error("test_perms_minimum_generators() empty perms"));
  if(#perms==1,return([]));

  my(id=test_identity_perm(#perms[1]),
     found=test_perms_minimum_generators_try
           (perms, [], List([ id ]), 1, #perms));
  \\ print(" final found "found);
  if(found=='none, [], found);
}
test_perms_minimum_generators_try(perms,gens,generated,start,m) =
{
  \\ gens[] = vector of set of generators so far
  \\ generated[] = List set of all perms which gens makes
  \\ start = position in perms to start trying to add another to gens[]
  \\ m = smallest set of generators seen so far

  \\ print(" gens = "gens);
  \\ print(" start="start" m="m" #generated="#generated);

  \\ Vec(generated) == Set(generated)
  \\   || error("test_perms_minimum_generators_try() generated is not a set");
  if(#gens+1>=m, return('none));  \\ cannot better than m

  my(best_found='none);
  for(i=start,#perms,
     if(setsearch(generated,perms[i]), next);  \\ already got
     \\ print(" i="i" m="m);

     my(new_gens=concat(gens,[perms[i]]),
        new_generated=test_extend_generated_list(generated,perms[i]));
     if(#new_generated==#perms,  \\ generates all
        \\ print(" found ",new_gens," is ",#new_gens);
        return(new_gens));

     my(new_found = test_perms_minimum_generators_try(perms,
                                                      new_gens, new_generated,
                                                      i+1, m));
     if(new_found!='none,
        #new_found < m || error("oops, not smaller than m="m": "new_found);
        if(best_found=='none || #new_found < #best_found,
           \\ print(" new low found ",#new_found);
           best_found=new_found;
           m=#new_found;
          )));

  best_found;
}
test_perms_minimum_num_generators(perms) =
{
  if(test_perms_is_Sn(perms),
     return(if(#perms==1,0, #perms==2,1, 2)));
  #test_perms_minimum_generators(perms);
}

\\ shift elements up +n, leaving 1..n-1 in each as 1..n-1 no-change perm
test_perms_offset(perms,n) =
{
  my(new_len=#perms[1]+n);
  apply(perm->vectorsmall(new_len,i, if(i<=n,i, perm[i-n]+n)), perms);
}
check_equal(test_perms_offset([ Vecsmall([2,1]) ], 3), \
            [ Vecsmall([1,2,3, 5,4]) ]);

\\ extend elements by +n, leaving new entries as no-change
test_perms_extend(perms,n) =
{
  my(keep=#perms[1],
     new_len=#perms[1]+n);
  apply(perm->vectorsmall(new_len,i, if(i<=keep,perm[i],i)), perms);
}
check_equal(test_perms_extend([ Vecsmall([2,1]) ], 3), \
            [ Vecsmall([2,1, 3,4,5]) ]);

\\ perms1 and perms2 are vectors of permutations.
\\ All perms1 permutation should be the same length, and similarly all
\\ perms2 the same length, but possibly different from perms1.
\\ Return a vector of permutations which are their direct product.
\\       perms1[i] concat perms2[j]
\\ where i,j range over the respective perms1 and perms2, and where the
\\ perms2 values are suitably offset for their new positions after perms1.
\\ perms1 and perms2 do not have to be groups, but if both are then the
\\ return is also a group.
test_perms_direct_product(perms1,perms2) =
{
  my(offset=#perms1[1]);
  for(i=1,#perms2,
     for(j=1,#perms2[i],
        perms2[i][j]+=offset));
  vector(#perms1*#perms2,n, my(qr=divrem(n-1,#perms2));
         concat(perms1[qr[1]+1], perms2[qr[2]+1]));
}

\\ perms is a vector of permutations.
\\ Return a vector of permutations which is test_perms_direct_product() power
\\     perms^n = perms x perms x perms x ... for n many terms.
\\ perms does not have to be a group, but if it is then the return is also a
\\ group.
test_perms_direct_power(perms,n) =
{
  n>=1 || error("test_perms_direct_power() is for n>=1");
  my(power=perms);
  for(i=2,n,
     power=test_perms_direct_product(power,perms));
  power;
}
check_equal(#test_perms_direct_power(test_perms_Sn(2), 3), 2^3);

test_perms_wreath_extras(len1,new_len,perms2) =
{
  apply(perm2->vectorsmall(new_len,i,
                           my(qr=divrem(i-1,len1));
                           (perm2[qr[1]+1]-1)*len1 + qr[2] + 1),
       perms2);
}
check_equal(test_perms_wreath_extras(3,6,[Vecsmall([2,1])]), \
            [Vecsmall([4, 5, 6, 1, 2, 3])]);

\\ perms1 and perms2 are vectors of permutations which are groups.
\\ Return a vector of permutations which is the group formed by direct product
\\ of perms1 and perms2.
\\ The new permutations are each length perms1 plus perms2.  The initial
\\ elements in each new perm are perms1, and the final elements are perms2.
test_perms_wreath_product(perms1,perms2) =
{
  my(power = test_perms_direct_power(perms1, #perms2[1]));
  test_perms_generators_to_group
  (concat(power,
          test_perms_wreath_extras(#perms1[1],#power[1],perms2)));
}

\\ perms is a vector of permutations which are a group.
\\ Return wreath power ((perms wr perms) wr perms) wr ... where n many perms.
test_perms_wreath_power(perms,n) =
{
  n>=1 || error("test_perms_wreath_power() is for n>=1");
  my(power=perms);
  for(i=2,n,
     power=test_perms_wreath_product(power,perms));
  power;
}

\\ perms is a vector of permutations.
\\ Return a vector of vectors of perms which are all the subgroups of the
\\ given perms.
test_perms_subgroups(perms) =
{
  my(l=List([]),
     idpos=if(#perms, test_vector_search(perms,
                                         test_identity_perm(#perms[1]))));
  forvec(flags=vector(#perms,i, if(i==idpos,[1,1],[0,1])),
        my(set=test_flags_to_Set(flags),
           subperms=vecextract(perms,set));
        if(test_perms_is_group(subperms),
           listput(l,subperms)));
  vecsort(Vec(l), (x,y)->lex(#x,#y));  \\ increasing group order
}

\\ perms is a vector of permutations which are a group.
\\ subperms is a subgroup of them.
\\ Return 1 if subperms is a normal subgroup, meaning for g in perms and s
\\ in subperms that g*s*g^-1 is in subperms.
test_perms_is_normal_subgroup(perms,subperms) =
{
  my(subperms_set=Set(subperms));
  for(i=1,#perms,
     my(inv=perms[i]^-1);
     for(j=1,#subperms,
        if(! setsearch(subperms_set, perms[i]*subperms[j]*inv),
           \\ print("missing i="i" j="j);
           return(0))));
  1;
}

\\ perms is a vector of permutations which are a group (test_perms_is_group()).
\\ Return the commutator subgroup (G,G), being all g*h*g^-1*h^-1 for
\\ elements g,h.
test_perms_commutator_subgroup(perms) =
{
  my(invs = apply(perm->perm^-1, perms),
     ret=List([]));
  for(i=1,#perms,
     for(j=1,#perms,
        my(elem = perms[i]*perms[j] * invs[i]*invs[j]);
        my(pos=setsearch(ret,elem,1));
        if(pos,listinsert(ret,elem,pos))));
  Vec(ret);
}
{
  my(perms=test_perms_Sn(3));
  check_equal(#perms, 6);
  check_equal(test_perms_is_group(perms), 1, "S3 is group");
  check_equal(test_perms_commutator_subgroup(perms),
              [ Vecsmall([1,2,3]), Vecsmall([2,3,1]), Vecsmall([3,1,2]) ],
              "test_perms_commutator_subgroup() of S3 is C3");
}
{
  my(perms=test_perms_Sn(5));
  check_equal(#perms, 120);
  check_equal(test_perms_is_group(perms), 1, "S5 is group");
  perms=test_perms_commutator_subgroup(perms);
  check_equal(#perms, 60);
  perms=test_perms_commutator_subgroup(perms);
  check_equal(#perms, 60);  \\ 60 again
}

\\ perms1 and perms2 are vectors of permutations which are groups
\\ (test_perms_is_group()).
\\ Return the commutator group (G,H), being all g*h*g^-1*h^-1 for elements
\\ g in G and h in H.
test_perms_commutator_group(perms1,perms2) =
{
  my(invs1 = apply(perm->perm^-1, perms1),
     invs2 = apply(perm->perm^-1, perms2),
     ret=List([]));
  for(i=1,#perms1,
     for(j=1,#perms2,
        my(elem = perms1[i]*perms2[j] * invs1[i]*invs2[j]);
        my(pos=setsearch(ret,elem,1));
        if(pos,listinsert(ret,elem,pos))));
  Vec(ret);
}

\\ perms is a vector of permutations
\\ Return 1 if they are Abelian, meaning commutative p*q == q*p for all pairs.
test_perms_is_Abelian(perms) =
{
  for(i=1,#perms,
     for(j=i+1,#perms,
        test_perm_commute(perms[i],perms[j]) || return(0)));
  1;
}

\\ perms is a vector of permutations which are a group.
\\ Return 1 if they are a 2-group, meaning number of perms is a power of 2.
test_perms_is_2group(perms) = test_is_pow2(#perms);

\\ perms is a vector of permutations which are a group (test_perms_is_group()).
\\ Return 1 if it is a nilpotent group.
\\ Nilpotent if and only if all elements with coprime orders commute.
test_perms_is_nilpotent(perms) =
{
  my(orders = apply(test_permorder, perms));
  for(i=1,#perms,
     for(j=i+1,#perms,
        if(gcd(orders[i],orders[j])==1
           && !test_perm_commute(perms[i],perms[j]),
           return(0))));
  1;
}

\\ perms is a vector of permutations which are a group (test_perms_is_group()).
\\ Return its nilpotency class, being how many terms in its lower central
\\ series to reach the trivial group.  Or in its upper central series, that
\\ being the same length).
\\ The trivial group itself is class 1, or if one step reach trivial then
\\ class 2, etc.
\\ If perms is not nilpotent then return symbol 'none.
test_perms_nilpotency_class(perms) =
{
  my(reduced=perms,
     ret=1);
  while(#reduced>1,
        ret++;
        my(new_reduced=test_perms_commutator_group(perms,reduced));
        if(#new_reduced >= #reduced, return('none));
        reduced=new_reduced);
  ret;
}

\\ perms is a vector of permutations which are a group (test_perms_is_group()).
\\ Return 1 if it is a solvable group.
\\ This also means polycyclic, since for a finite group solvable==polycyclic.
test_perms_is_solvable(perms) =
{
  \\ derived series of iterated commutator subgroup reaching identity means
  \\ solvable
  while(#perms>1,
        my(new_perms=test_perms_commutator_subgroup(perms));
        if(#new_perms >= #perms, return(0));
        perms=new_perms);
  1;
}

\\ perms is a vector of permutations which are a group (test_perms_is_group()).
\\ Return 1 if it is a supersolvable group.
test_perms_is_supersolvable(perms) =
{
  \\ WRONG: FIXME:
  \\ supersolvables have nilpotent commutator subgroup, but there are other
  \\ supersolvables too

  test_perms_is_nilpotent(test_perms_commutator_subgroup(perms));
}

check_equal(test_perms_is_supersolvable([ Vecsmall([]) ]), 1, \
            "test_perms_is_supersolvable() trivial empty");
check_equal(test_perms_is_supersolvable([ Vecsmall([1,2]) ]), 1);
check_equal(test_perms_is_supersolvable([ Vecsmall([1,2]), Vecsmall([2,1]) ]), 1);
{
  for(n=1,5,
     my(perms=test_perms_Sn(n));
     check_equal(test_perms_is_supersolvable(perms), n<=3,
                 Str("S"n" supersolvable")));
}
{
  my(gens=[ Vecsmall([2,1, 3,4,5,6]),   \\ S4 x S2
            Vecsmall([1,2, 4,3,5,6]),
            Vecsmall([1,2, 6,3,4,5]) ],
     perms = test_perms_generators_to_group(gens));
  check_equal(test_perms_is_group(perms), 1, "S4xS2 is group");
  check_equal(test_perms_is_solvable(perms), 1);

  check_equal(perms, test_perms_direct_product(test_perms_Sn(2),
                                               test_perms_Sn(4)));

  my(commutator=test_perms_commutator_subgroup(perms));
  check_equal(#commutator, 12);
  check_equal(test_perms_is_nilpotent(commutator), 0);
  my(commutator=test_perms_commutator_group(perms,commutator));
  check_equal(#commutator, 12);

  check_equal(test_perms_is_nilpotent(perms), 0);
  check_equal(test_perms_is_supersolvable(perms), 0,
             "S4 x S2 not supersolvable");
}

\\ Return 1 if permutation perm is an involution, meaning it has order <=2
\\ so perm^2 == identity.
\\ The identity permutation is reckoned an involution under this definition.
\\ Believe that is usual.
test_perm_is_involution(perm) =
{
  for(i=1,#perm,
     perm[perm[i]]==i || return(0));
  1;
}

\\ perms is a vector of permutations.
\\ Return 1 if they are all involutions per test_perm_is_involution().
test_perms_is_involutions(perms) =
{
  for(i=1,#perms,
     test_perm_is_involution(perms[i]) || return(0));
  1;
}

\\ func = test_make_vecsearch_func(vec)
\\ vec is a vector of elements.
\\ Return a function to be called pos=func(elem) which returns the position
\\ in vec of element "elem", or 0 if elem is not in vec.
test_make_vecsearch_func(vec) =
{
  my(perm=vecsort(vec,cmp,9),  \\ delete duplicates
     vec_sorted=vecextract(vec,perm));
  (elem)-> my(pos=setsearch(vec_sorted,elem)); if(pos, perm[pos], 0);
}

test_perms_map_validate(perms1,perms2,map_to) =
{
  \\ print("test_perms_map_validate()");
  my(perm1_to_i=test_make_vecsearch_func(perms1),
     perm2_to_i=test_make_vecsearch_func(perms2));

  for(i=1,#perms1,
     map_to[i] || next;
     for(j=1,#perms1,
        map_to[j] || next;
        my(prod_perm=perms1[i]*perms1[j],
           prod_pos=perm1_to_i(prod_perm));
        prod_pos || next;
        perms1[prod_pos] == prod_perm || error();

        my(prod_perm2=perms2[map_to[i]]*perms2[map_to[j]],
           prod_pos2=perm2_to_i(prod_perm2));
        perms2[prod_pos2] == prod_perm2 || error();

        if(map_to[prod_pos] != prod_pos2,
           \\ print(" i= "i" j= "j" prod "prod_pos);
           \\ print(" i2="map_to[i]" j2="map_to[j]" prod "prod_pos2);
           error("i="i" * j="j" = "prod_pos" map_to="map_to[prod_pos]
                 ", but map "map_to[i]" * "map_to[j]" = "prod_pos2);
           return(0))));
  1;
}

test_perms_isomorphic_recurse(perms1,perms2,orders1,orders2,map_to,map_from) =
{
  \\ map_to[x]=y and map_from[x]=y means attempting to reckon perms1[x] and
  \\ perms2[y] equivalent.

  \\ print(" test_perms_isomorphic_recurse() "hammingweight(map_to));
  \\ print("  map_to   = "Vec(map_to));
  \\ print("  map_from = "Vec(map_from));
  my(more);
  until(!more,
        more=0;
        \\ print("  loop check");
        for(i=1,#perms1,
           my(i2=map_to[i]);
           if(!i2, next);
           for(j=1,#perms1,
              my(j2=map_to[j]);
              if(!j2, next);
              my(prod_perm=perms1[i]*perms1[j],
                 prod_pos=setsearch(perms1,prod_perm));
              if(!prod_pos, next);  \\ product not in perms1
              \\ XXprint("  i="i" j="j" prod "prod_pos", map "map_to[i]" "map_to[j]" prod "map_to[prod_pos]);
              my(prod_perm2=perms2[i2]*perms2[j2],
                 prod_pos2=setsearch(perms2,prod_perm2));
              if(!prod_pos2,
                 \\ print("  no such in perms2 (but is in perms1)");
                 return('none));
              if(map_to[prod_pos]==prod_pos2, next); \\ ok, already mapped

              \\ print("  consequent map "prod_pos" -> "prod_pos2);
              if(! (map_to[prod_pos]==0 || map_to[prod_pos]==prod_pos2),
                 \\ print("  not consistent (map_to = "map_to[prod_pos]")");
                 return('none));
              if(! (map_from[prod_pos2]==0 || map_from[prod_pos2]==prod_pos),
                 \\ print("  not consistent (map_from = "map_from[prod_pos2]")");
                 return('none));
              \\ (map_to[prod_pos]==0 && map_from[prod_pos2]==0) || error();
              if(orders1[prod_pos] != orders2[prod_pos2],
                 \\ print("  different orders "orders1[prod_pos]" "orders2[prod_pos2]);
                 return('none));
              map_to[prod_pos]=prod_pos2;
              map_from[prod_pos2]=prod_pos;
              more=1)));

  \\ test_perms_map_validate(perms1,perms2,map_to);

  if(hammingweight(map_to)==#perms1,
     \\ print("  is full map_to = "Vec(map_to));
     return(map_to));

  for(i=1,#perms1,
     if(map_to[i], next);
     for(i2=1,#perms1,
        if(map_from[i2] || orders1[i]!=orders2[i2], next);
        map_to[i]=i2;
        map_from[i2]=i;
        \\ print(" try i="i" i2="i2);
        my(got=test_perms_isomorphic_recurse(perms1,perms2,orders1,orders2,
                                             map_to,map_from));
        if(got != 'none, return(got));
        map_to[i]=0;
        map_from[i2]=0));

  'none;
}

test_perms_isomorphic_perm_unchecked(perms1,perms2) =
{
  \\ print("test_perms_isomorphic_perm_unchecked()");
  if(#perms1!=#perms2, return('none));
  \\ print(" perms1 "perms1);
  \\ print(" perms2 "perms2);

  my(perms1_sort_perm=vecsort(perms1,cmp,1),
     perms2_sort_perm=vecsort(perms2,cmp,1));
  perms1=vecextract(perms1,perms1_sort_perm);
  perms2=vecextract(perms2,perms2_sort_perm);
  \\ print(" sorted1 "perms1);
  \\ print(" sorted2 "perms2);

  setisset(perms1) || error();
  setisset(perms2) || error();

  my(orders1=apply(test_permorder,perms1),
     orders2=apply(test_permorder,perms2));
  \\ print(" orders1 "orders1);
  \\ print(" orders2 "orders2);
  if(vecsort(orders1)!=vecsort(orders2), return('none));

  my(got=test_perms_isomorphic_recurse(perms1, perms2,
                                       orders1, orders2,
                                       vectorsmall(#perms1),
                                       vectorsmall(#perms1)));
  if(got=='none, return('none));

  \\ print(" got  "Vec(got));
  \\ print(" s1   "Vec(perms1_sort_perm));
  \\ print(" s2   "Vec(perms2_sort_perm));
  got = perms2_sort_perm * got * perms1_sort_perm^-1;
  \\ print(" ret  "Vec(got));
  got;
}

\\ p = test_perms_isomorphic_perm(perms1,perms2)
\\ perms1 is a vector of permutations which are a group.
\\ perms2 likewise.
\\ If they are isomorphic as groups then return a Vecsmall permutation p
\\ which is a mapping from perms1 to perms2.  p[i]=j means perms1[i]
\\ corresponds to perms2[j].
\\ If they are not isomorphic as groups then return symbol 'none.
\\ If different group sizes #perms1!=#perms2 then not isomorphic, but the
\\ elements can be different length.
test_perms_isomorphic_perm(perms1,perms2) =
{
  my(got=test_perms_isomorphic_perm_unchecked(perms1,perms2));
  if(got=='none, return('none));
  test_is_perm(got) || error error("oops map not perm: ",got);
  test_perms_map_validate(perms1,perms2,got);
  got;
}

\\ perms1 is a vector of permutations which are a group.
\\ perms2 likewise.
\\ Return 1 if they are isomorphic as groups.
test_perms_is_isomorphic(perms1,perms2) = \
  test_perms_isomorphic_perm(perms1,perms2) != 'none;


\\-----------------------------------------------------------------------------
\\ Pari Groups

test_gens_to_smallgroup(gens) = \
  [gens, Vecsmall(apply(test_permorder,gens))];

\\ Conditions per Pari library group_isA4S4().
test_smallgroup_is_A4(group) =
{
  my(gens=group[1], orders=group[2]);
  orders==Vecsmall([2,2,3]) && ! test_perm_commute(gens[1],gens[3]);
}
test_smallgroup_is_S4(group) =
{
  my(gens=group[1], orders=group[2]);
  orders==Vecsmall([2,2,3,2])
  && ! test_perm_commute(gens[1],gens[3])
  && ! test_perm_commute(gens[3],gens[4]);
}

\\ conditions per galoisinit()
test_smallgroup_validate(group) =
{
  type(group)=="t_VEC" || error("group not a Vec: ",type(group));
  #group==2 || error();

  my(gens = group[1],
     orders = group[2]);
  type(gens)=="t_VEC" || error();
  type(orders)=="t_VECSMALL" || error();
  #gens == #orders || error();

  for(i=1,#gens,
     type(gens[i]) == "t_VECSMALL" || error();
     test_is_perm(gens[i]) || error();
     test_permorder(gens[i]) == orders[i] || error());

  \\ normal within next up
  my(prev=test_perms_generators_to_group(gens[1..1]));
  for(i=2,#gens,
     my(perms=test_perms_generators_to_group(concat(prev, gens[i..i])));
     if(!test_perms_is_normal_subgroup(perms,prev),
        my(rest=[ gens[i..#gens], orders[i..#orders] ]);
        if(! (test_smallgroup_is_A4(rest) || test_smallgroup_is_S4(rest)),
           error("test_smallgroup_validate(): below i="i" not normal")));
     prev=perms);

  \\ gen powers g1^e1 * g2^e2 * ... all distinct
  prev=Set(prev);
  my(seen=vectorsmall(#prev));
  forvec(e=vector(#orders,i,[0,orders[i]-1]),
        my(g=test_smallgroup_exponents_to_elem(group,e),
           pos=setsearch(prev,g));
        if(!pos,
           error("test_smallgroup_validate(): power element not found: "e));
        if(seen[pos],
           error("test_smallgroup_validate(): duplicate power element: "e));
        seen[pos]=1);
  if(hammingweight(seen)!=#prev,
     error("test_smallgroup_validate(): oops, power element count different"));
}

\\ "group" is a small group [gens,orders].
\\ n is an integer element number 1 .. prod(orders), inclusive.
\\ Return a vector of exponents for this element.
test_smallgroup_n_to_exponents(group,n) =
{
  my(vec=vector(#group[2]));
  if(n<1, error("test_smallgroup_n_to_exponents() n<1"));
  n--;
  for(i=1,#vec,
     my(qr=divrem(n,group[2][i]));
     vec[i] = qr[2];
     n=qr[1]);
  if(n, error("test_smallgroup_n_to_exponents() n too big"));
  elem;
}

\\ "group" is a small group [gens,orders].
\\ n is an integer element number 1 .. prod(orders), inclusive.
\\ Return that group element.
test_smallgroup_n_to_elem(group,n) =
{
  n>=1 || error("test_smallgroup_n_to_elem() is for n>=1");
  my(elem=test_identity_perm(#group[1][1]));  \\ identity element
  n--;
  for(i=1,#group[2],
     my(qr=divrem(n,group[2][i]));
     elem *= group[1][i]^qr[2];
     n=qr[1]);
  if(n, error("test_smallgroup_n_to_elem() n too big, remainder ",n));
  elem;
}

\\ "group" is a small group [gens,orders].
\\ "vec" is a vector of exponents
\\ Return generators product gens[1]^vec[1] * gens[2]^vec[2] * ...
test_smallgroup_exponents_to_elem(group,vec) =
{
  my(elem=group[1][1]^vec[1]);
  for(i=2,#group[2],
     elem = elem * group[1][i]^vec[i]);
  elem;
}

\\ "group" is a small group [gens,orders].
\\ "elem" is an element of the group.
\\ Return a vector "vec" of exponents where the generators product
\\ gens[1]^vec[1] * gens[2]^vec[2] * ... = elem.
test_smallgroup_elem_to_exponents(group,elem) =
{
  forvec(vec=vector(#group[2],i, [0, group[2][i]-1]),
        if(test_smallgroup_exponents_to_elem(group,vec) == elem,
           \\ return(vector(#vec,i, group[2][i]-vec[i]));  \\ reversal
           return(vec)));
  error("oops, exponents for elem not found");
}


\\-----------------------------------------------------------------------------
\\ Automorphisms

\\ Return the maximum number of generators returned by
\\ vpar_automorphism_generators() (or vpar_num_automorphism_generators()) on
\\ a tree.
\\ Optional parameter "forest" can be 1 for maximum needed for a forest.
\\ Trees = 0, 0, 0, 1, 2, 2, 2, 3, 4, 4, 4, ... (A004524)
\\ Forest same one earlier.
\\
\\ FIXME: is this right?  It applies only to the generators form currently
\\ made by vpar_automorphism_generators() anyway.
\\
test_num_automorphism_generators_max(n,forest=0) =
{
  n += forest;
  (n>>1) - (bitand(n,3)==2);
}

\\ trees
\\ n=1  2  3  4  5  6  7  8       11 12       15 16
\\   0, 1, 1, 2, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 7, 7, 8, 8,
\\ forests
\\ n=1  2  3  4        7  8       11 12 13 14 15 16
\\   0, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 7, 7, 7, 8, 8, 9,
\\
\\ FIXME: is this formula right?  It applies only to the generators form
\\ currently made by vpar_free_automorphism_generators() anyway.
\\
test_num_free_automorphism_generators_max(n,forest=0) =
{
  if(forest,
     if(n==1,return(0));
     if(n<=13, n++);  \\ three 7s
     ,
     if(n>=5, n--));  \\ three 2s
  (n>>1);
}


\\-----------------------------------------------------------------------------
\\ Quaternions

test_quaternion_mul_elements = [  1,  i,  j,  k,  -1, -i, -j, -k ];
test_quaternion_mul_table = {
  [ 1,  i,  j,  k, -1, -i, -j, -k;
    i, -1,  k, -j, -i,  1, -k,  j;
    j, -k, -1,  i, -j,  k,  1, -i;
    k,  j, -i, -1, -k, -j,  i,  1;
   -1, -i, -j, -k,  1,  i,  j,  k;
   -i,  1, -k,  j,  i, -1,  k, -j;
   -j,  k,  1, -i,  j, -k, -1,  i;
   -k, -j,  i,  1,  k,  j, -i, -1 ];
}

test_quaternion_mul(a,b) =
{
  for(r=1,#test_quaternion_mul_elements,
     if(test_quaternion_mul_elements[r]==a,
        for(c=1,#test_quaternion_mul_elements,
           if(test_quaternion_mul_elements[c]==b,
              return(test_quaternion_mul_table[r,c])))));
  error();
}

test_quaternion_Cayley_table =
{
  matrix(8,8,r,c,
         my(p);
         for(u=1,#test_quaternion_mul_elements,
            if(test_quaternion_mul_table[r,c] == test_quaternion_mul_elements[u],
               p=u;break));
        p);
}
test_quaternion_perms = vector(8,r, Vecsmall(test_quaternion_Cayley_table[r,]));
check_equal(test_perms_is_group(test_quaternion_perms), 1);
check_equal(apply(test_permorder,test_quaternion_perms), [1, 4,4,4, 2, 4,4,4]);


\\-----------------------------------------------------------------------------
\\ Vpar Miscellaneous Helpers

\\ pred is a predicate function to be called bool=pred(vpar,v).
\\ Return a vector of the vertex numbers which satisfy pred.
test_select_vertices(vpar,pred) = select(v->pred(vpar,v), [1..#vpar]);

\\ Return 1 if vpar is a forest of entirely singletons, including an empty
\\ forest vpar==[].
test_is_all_singletons(vpar) = hammingweight(vpar)==0;

\\ Return 1 if vpar contains any singletons (vertex with no neighbours).
test_any_singletons(vpar) =
{
  my(num_children=vpar_num_children_vector(vpar));
  for(v=1,#vpar, if(vpar[v]==0 && num_children[v]==0, return(1)));
  0;
}

\\ "set" is a Set() of vertex numbers.
\\ Return 1 if vertex v has a neighbour in set.
test_is_neighbour_in_set(vpar,v,set) =
{
  my(neighbours=vpar_neighbours_of_vertex(vpar,v));
  for(i=1,#neighbours,
     if(setsearch(set,neighbours[i]), return(1)));
  0;
}

\\ vec[v]=c is the component number c containing vertex v.
\\ Each c is in the range 1 to some maximum.
\\ Return a vector [v1, v2, ...] where smallest[c] = smallest v with vec[v]=c.
test_compnum_to_smallest(vec) =
{
  my(ret=vector(if(#vec,vecmax(vec),0)));
  for(v=1,#vec,
     my(c=vec[v]);
     if(!ret[c], ret[c]=v));
  ret;
}

\\ Return 1 if vpar1 == vpar2 after re-rooting them to the same root vertex
\\ numbers.
\\
test_equal_after_rerooting(vpar1,vpar2) =
{
  vpar1 = vpar_reroot_minmax(vpar1,min);
  vpar2 = vpar_reroot_minmax(vpar2,min);
  vpar1 == vpar2;
}

test_num_loops(vpar) = sum(v=1,#vpar, vpar[v]==v);


\\-----------------------------------------------------------------------------
\\ Sequence Things

\\ Return 1 if seq takes children in ascending order within vpar.
\\ Roots can be in any order.
test_seq_is_ordered_except_roots(vpar,seq) =
{
  \\ last_child[p] is the most recent seen child vertex number of parent p,
  \\ or 0 if no children seen yet.
  my(last=vectorsmall(#vpar));
  for(i=1,#seq,
     my(v=seq[i], p=vpar[v]);
     if(p, if(v<last[p], return(0)); last[p]=v));
  1;
}

\\ Return 1 if seq has siblings consecutive in the sequence, including roots
\\ as siblings of each other consecutive in the sequence.
test_seq_is_siblings_consecutive(vpar,seq) =
{
  my(next_sibling=vpar_next_sibling_vector(vpar));
  for(i=1,#seq,
     my(v=seq[i]);
     if(next_sibling[v],
        next_sibling[v] == if(i==#vpar,0,seq[i+1]) || return(0)));
  1;
}


\\-----------------------------------------------------------------------------
\\ Disjoint Domnum

\\ flags is a vector.
\\ Return 1 if the first non-zero entry in flags is 1, or there are no
\\ non-zero entries at all.
\\ Return 0 if not, so there is a first non-zero entry in flags and it's not 1.
test_flags_is_one_first(flags) =
{
  for(v=1,#flags,
     if(flags[v]==1, return(1));
     if(flags[v]==2, return(0)));
  1;
}

\\ Return a vector of booleans [b,b,b,b,b] which are whether the root of
\\ vpar is the various with_dom through without_undom_undom.
\\
test_is_disjoint_domnum_quantities_by_flags(vpar,flags) =
{
  \\ print("test_is_disjoint_domnum_quantities_by_flags() flags="flags);
  my(dominated=vector(2,i,vectorsmall(#vpar)));
  for(v=1,#vpar,
     if(flags[v], dominated[flags[v]][v]++);    \\ v dom self
     my(p=vpar[v]);
     if(p,
        if(flags[v], dominated[flags[v]][p]++);    \\ v dom p
        if(flags[p], dominated[flags[p]][v]++)));  \\ p dom v
  my(r=vpar_root(vpar));
  \\ print("  r="r" "dominated);
  for(v=1,#vpar,
     if(v!=r,
        if(!(dominated[1][v] && dominated[2][v]),
           \\ print("  v="v" undominated");
           return([0,0,0,0,0]))));
  [ flags[r]==1 &&  dominated[2][r],
    flags[r]==1 && !dominated[2][r],
    !flags[r]   &&  dominated[1][r] &&  dominated[2][r],
    !flags[r]   &&  dominated[1][r] && !dominated[2][r],
    !flags[r]   && !dominated[1][r] && !dominated[2][r] ];
}
check_equal(test_is_disjoint_domnum_quantities_by_flags([0],[0,0]), \
            [0,0,0,0,1], \
            "test_is_disjoint_domnum_quantities_by_flags()");
check_equal(test_is_disjoint_domnum_quantities_by_flags([2,3,0],[2,1,0]), \
            [0,0,0,1,0]);   \\ without_dom_undom

\\ Return a vector of pairs [[s,c],...[s,c]] which are size and count of the
\\ smallest of each with_dom through without_undom_undom in vpar.
\\
test_disjoint_domnum_and_count_quantities_by_flags(vpar) =
{
  my(quantities=matrix(6,2,r,c, if(c==1,-1,0)),
     r=vpar_root(vpar));
  forvec(flags=vector(#vpar,i,[0,2]),
        my(one_first=test_flags_is_one_first(flags));

        my(size=hammingweight(flags),
           this=test_is_disjoint_domnum_quantities_by_flags(vpar,flags));
        this[3] = this[3] && one_first;
        this[5] = this[5] && one_first;
        if(vecsum(this),
           \\ print("flags="flags" size "size"  "this" one_first="one_first);
          );

        for(i=1,#this,
           if(this[i],
              if(size < quantities[i,1] || !quantities[i,2],
                 quantities[i,1] = size; quantities[i,2] = 1;
                 \\ print("  i="i" new low "size);
                 ,
                 size == quantities[i,1],
                 quantities[i,2]++;
                 \\ print("  i="i" equal "size);
                ))));

  \\ print(" final "quantities);
  quantities[6,1] = quantities[1,1];
  quantities[6,2] = quantities[1,2];
  if(quantities[3,2],
     if(quantities[3,1] < quantities[6,1],
        quantities[6,1] = quantities[3,1];
        quantities[6,2] = quantities[3,2];
        ,
        quantities[3,1] == quantities[6,1],
        quantities[6,2] += quantities[3,2]));

  quantities;
}

\\ flags is a vector of entries 0,1,2.
\\ Return 1 if the set of 1s are a dominating set and the set of 2s are another
\\ dominating set, so that together they are a pair of disjoint dominating sets.
\\
test_is_disjoint_domsets_flags(vpar,flags) =
{
  vpar_is_domset_flags(vpar,apply(f->f==1,flags))
  && vpar_is_domset_flags(vpar,apply(f->f==2,flags));
}
\\      1  2
\\      |
\\      2  1   with_undom = 3
\\      |
\\      3  without_undom = 2
\\      |
\\      4  2
\\      |
\\      5  1

test_disjoint_domnum_and_count_by_flags(vpar) =
{
  if(#vpar==0,return([0,1]));  \\ empty set only
  my(disjoint_domnum=#vpar+1,
     count=0);
  forvec(flags=vector(#vpar,i,[0,2]),
        \\ only with 1 first so one way around
        for(v=1,#flags, if(flags[v]==1,break(), flags[v]==2,next(2)));
        if(test_is_disjoint_domsets_flags(vpar,flags),
           my(size=hammingweight(flags));
           \\ print(flags);
           if(size<disjoint_domnum,  disjoint_domnum=size;count=1, \\ new low
              size==disjoint_domnum, count++)));                  \\ equal low
  if(disjoint_domnum==#vpar+1, [-1,0], [disjoint_domnum,count]);
}

test_disjoint_domnum_sets_by_flags(vpar) =
{
  my(l=List([]),
     m=#vpar+1);
  forvec(flags=vector(#vpar,i,[0,2]),
        my(size=hammingweight(flags));
        if(size>m, next());
        for(v=1,#flags, if(flags[v]==1,break(), flags[v]==2,next(2)));
        if(test_is_disjoint_domsets_flags(vpar,flags),
           my(sets=vecsort([test_flags_to_Set(flags,1),
                            test_flags_to_Set(flags,2)]));
           if(size==m, listput(l,sets),           \\ more
                       l=List([sets]); m=size))); \\ new low
  Vec(l);
}


\\-----------------------------------------------------------------------------
\\ Minimal Domination

\\ like vpar_minimal_domsets() but by trying all vpar_is_minimal_domset_flags()
test_minimal_domsets_by_flags(vpar) =
{
  my(l=List([]));
  forvec(flags=vector(#vpar,i,[0,1]),
        if(vpar_is_minimal_domset_flags(vpar,flags),
           listput(l,test_flags_to_Set(flags))));
  Vec(l);
}


\\-----------------------------------------------------------------------------
\\ Semi-Total Domination

test_semitotdomnum_and_count_by_flags(vpar) =
{
  if(#vpar==0,return([0,1]));  \\ empty set only
  my(semitotdomnum=#vpar+1,count=0);
  forvec(flags=vector(#vpar,i,[0,1]),
        if(vpar_is_semitotdomset_flags(vpar,flags),
           my(size=vecsum(flags));
           if(size<semitotdomnum,  semitotdomnum=size;count=1,  \\ new low
              size==semitotdomnum, count++)));           \\ equal low
  if(count==0,semitotdomnum=-1);
  [semitotdomnum,count];
}


\\-----------------------------------------------------------------------------
\\ Equimatchable

\\ Number of states in vpar_INTERNAL_is_equimatchable_table[] etc.
test_is_equimatchable_num_states = 5;

\\ p,v are integer states 1 to 5, return new state of parent when v attached
\\ under p, or return 0 if this combination is not equimatchable (no matter
\\ what further children or what further above).
\\
test_is_equimatchable_table_lookup(p,v) =
{
  (   1 <= p && p <= test_is_equimatchable_num_states
   && 1 <= v && v <= test_is_equimatchable_num_states ) || error();
  vpar_INTERNAL_is_equimatchable_table[test_is_equimatchable_num_states*p
                                       + v
                                       - test_is_equimatchable_num_states];
}

\\ Return the equimatchable state 1 to 5 at the root of vpar, or return 0 if
\\ vpar is not equimatchable.
\\ vpar must be a tree, not a forest.
\\ If vpar==[] empty then return 0.
\\
test_equimatchable_get_state(vpar) =
{
  my(seq=vpar_seq_upascend(vpar),
     state = vectorsmall(#vpar,v,1));
  for(i=1,#seq,
     my(v=seq[i], p=vpar[v]);
     if(p,
        (state[p]=test_is_equimatchable_table_lookup(state[p],state[v]))
          || return(0);
        ,
        return(state[v])));
  0;
}


\\-----------------------------------------------------------------------------
\\ Stable Equimatchable

\\ Number of states in vpar_INTERNAL_is_stable_equimatchable_table[].
test_is_stable_equimatchable_num_states = 9;

\\ p,v are integer states 1 to 9, return new state of parent when v attached
\\ under p, or return 0 if this combination is not edge stable equimatchable
\\ (no matter what further children or what further above).
\\
test_is_stable_equimatchable_table_lookup(p,v) =
{
  (   1 <= p && p <= test_is_stable_equimatchable_num_states
   && 1 <= v && v <= test_is_stable_equimatchable_num_states ) || error();
  vpar_INTERNAL_is_stable_equimatchable_table
    [test_is_stable_equimatchable_num_states*p
     + v - test_is_stable_equimatchable_num_states];
}

\\ same as inline in vpar_is_stable_equimatchable() code
test_is_stable_equimatchable_at_root_bits = 318;
test_is_stable_equimatchable_at_root(state) = \
  bittest(test_is_stable_equimatchable_at_root_bits,state);

\\ Return the edge stable equimatchable state 1 to 8 at the root of vpar, or
\\ return 0 if vpar is not edge stable equimatchable.
\\ vpar must be a tree, not a forest.
\\ If vpar==[] empty then return 0.
\\
test_stable_equimatchable_get_state(vpar) =
{
  my(seq=vpar_seq_upascend(vpar),
     state = vectorsmall(#vpar,v,1));
  for(i=1,#seq,
     my(v=seq[i], p=vpar[v]);
     \\ print(" v="v" state "state[v]" under p="p" state "if(p,state[p],"(none)"));
     if(p,
        (state[p]
         = test_is_stable_equimatchable_table_lookup(state[p],state[v]))
        || return(0);
        ,
        return(state[v])));
  0;
}

test_is_edge_stable_equimatchable_by_cut(vpar) =
{
  if(! vpar_is_equimatchable(vpar), return(0));
  for(v=1,#vpar,
     if(vpar[v],
        my(vpar_cut=vpar);
        vpar_cut[v]=0;
        if(! vpar_is_equimatchable(vpar_cut),
           return(0))));
  1;
}

\\ Return a vector of child vertices where a cut above causes not
\\ equimatchable.  The list includes 0 if the whole is not equimatchable.
test_is_edge_stable_equimatchable_no_cut_list(vpar) =
{
  my(l=List([]));
  if(! vpar_is_equimatchable(vpar),
     listput(l,0));
  for(v=1,#vpar,
     if(vpar[v],
        my(vpar_cut=vpar);
        vpar_cut[v]=0;
        if(! vpar_is_equimatchable(vpar_cut),
           listput(l,v))));
  Vec(l);
}

test_is_vertex_stable_equimatchable_by_removals(vpar) =
{
  vpar_is_equimatchable(vpar) || return(0);
  for(v=1,#vpar,
     if(!vpar_is_equimatchable(vpar_delete_vertex(vpar,v)),
        \\ print("not at "v);
        return(0)));
  1;
}


\\-----------------------------------------------------------------------------
\\ Almost Equimatchable

\\ Number of states in vpar_INTERNAL_is_almost_equimatchable_table[].
test_is_almost_equimatchable_num_states = 11;
\\ check_equal(#vpar_INTERNAL_is_almost_equimatchable_table, \
\\             test_is_almost_equimatchable_num_states^2);

\\ p,v are integer states 1 to 11, return new state of parent when v attached
\\ under p, or return 0 if this combination is not edge almost equimatchable
\\ (no matter what further children or what further above).
\\
test_is_almost_equimatchable_table_lookup(p,v) =
{
  (   1 <= p && p <= test_is_almost_equimatchable_num_states
   && 1 <= v && v <= test_is_almost_equimatchable_num_states ) || error();
  vpar_INTERNAL_is_almost_equimatchable_table
    [test_is_almost_equimatchable_num_states*p
     + v - test_is_almost_equimatchable_num_states];
}

\\ same as inline in vpar_is_almost_equimatchable() code
test_is_almost_equimatchable_at_root_bits = 3024;
test_is_almost_equimatchable_at_root(state) = \
  bittest(test_is_almost_equimatchable_at_root_bits,state);

\\ Return the edge almost equimatchable state 1 to 8 at the root of vpar, or
\\ return 0 if vpar is not edge almost equimatchable.
\\ vpar must be a tree, not a forest.
\\ If vpar==[] empty then return 0.
\\
test_almost_equimatchable_get_state(vpar) =
{
  my(seq=vpar_seq_upascend(vpar),
     state = vectorsmall(#vpar,v,1));
  for(i=1,#seq,
     my(v=seq[i], p=vpar[v]);
     \\ print(" v="v" state "state[v]" under p="p" state "if(p,state[p],"(none)"));
     if(p,
        (state[p]
         = test_is_almost_equimatchable_table_lookup(state[p],state[v]))
        || return(0);
        ,
        return(state[v])));
  0;
}

\\ vpar must be a tree.
\\ Return 1 if it is almost-equimatchable.
test_tree_is_almost_equimatchable_by_matchpoly(vpar) =
{
  \\ biggest and smallest poly terms differ by just 1
  my(poly=vpar_maximal_matchpoly(vpar));
  poldegree(poly) == valuation(poly,'x) + 1;
}

\\ vpar can be a tree or a forest.
\\ Return 1 if it is almost-equimatchable.
test_is_almost_equimatchable_by_matchpoly(vpar) =
{
  if(#vpar==0, return(0));
  my(trees=vpar_component_trees(vpar));
  for(i=1,#trees,
     if(! test_tree_is_almost_equimatchable_by_matchpoly(trees[i]),
        return(0)));
  1;
}


\\-----------------------------------------------------------------------------
\\ Indices

\\ Relation to Wiener index per Klein, Mihalic, Plavsic, Trinajstic:
\\ "Molecular Toplogical Index: Relation With the Wiener
\\ Index". J. Chem. Inf. Comput. Sci., 32 (1992), 304-305.
\\
test_Schultz_index(vpar) = \
  if(#vpar, 4*vpar_Wiener_index(vpar) - #vpar*(#vpar-1),  0);


\\ Modifed Schultz index defined in
\\
\\    Klavzar, Gutman: "Wiener Number of Vertex-Weighted Graphs and a
\\    Chemical Application", Disc. Appl. Maths., volume 80, 1997, pages 73-81.
\\
\\ Relation to Wiener index per Gutman: "Selected Properties of the Schultz
\\ Molecular Topological Index". J. Ch Inf. Comput. Sci., volume 34, 1994,
\\ pages 1087-1089.
\\
\\ In Mehdi Eliasi and Bijan Taeri, "Schultz Polynomials of Composite
\\ Graphs", Applicable Analysis and Discrete Mathematics, volume 2, number
\\ 2, October 2008, pages 285-296, http://www.jstor.org/stable/43666990,
\\ is given 4*W-n*(2n-1).  Is that a misprint?
\\
test_modified_Schultz_index(vpar) = \
  if(#vpar>=2, 4*vpar_Wiener_index(vpar) - (#vpar-1)*(2*#vpar-1),  0);


\\-----------------------------------------------------------------------------
\\ Least Child Monk

\\ Like vpar_is_leastchildmonk() but with min_child established all at once
\\ and searched for root with child with child.
\\
test_is_leastchildmonk_by_min_child(vpar) =
{
  my(min_child=vpar_minmax_child_vector(vpar,min));
  for(v=1,#vpar,
     if(vpar[v]==0 && min_child[v] && min_child[min_child[v]],
        return(0)));  \\ root least child has a child
  1;
}


\\-----------------------------------------------------------------------------
\\ Unlabelled Trees

test_seq_lexmax_by_search(vpar) =
{
  my(max_perm=test_identity_perm(#vpar),
     max_vpar=vpar);
  for(i=1,(#vpar)!,
     my(perm=numtoperm(#vpar,i),
        relabelled=vpar_relabel_perm(vpar,perm));
     if(lex(relabelled,max_vpar) > 0,
        max_perm = perm;
        max_vpar = relabelled));
  Vecsmall(max_perm)^-1;
}
test_to_lexmax_by_search(vpar) = \
  vpar_relabel_seq(vpar,test_seq_lexmax_by_search(vpar));

\\ PENDING vpar_unlabelled_all
test_unlabelled_all(n,forest=0) =
{
  my(ret=vector(vpar_count_unlabelled(n,forest)));
  ret[1]=vpar_unlabelled_first(n);
  for(i=2,#ret,
     ret[i]=vpar_unlabelled_next(ret[i-1],forest));
  ret;
}

\\ Return the number of unlabelled rooted trees of n vertices whose
\\ automorphism group is a 2-group, that being number of automorphisms a
\\ power 2^k.
\\ A248869
test_count_unlabelled_2group(n) = \
  polcoeff(test_count_unlabelled_2group_gf(n+1), n);
\\
\\ The same, as a vector where a[1] = 1 vertices, a[2] = 2 vertices, etc
test_count_unlabelled_2group_vector(len) = \
  Vec(test_count_unlabelled_2group_gf(len+1))[^1];
\\
\\ The same as a gf polynomial where term x^n is how many n vertices.
\\ len many terms x^0 to x^(len-1), and series O(x^len).
test_count_unlabelled_2group_gf(len) =
{
  my(L=O('x^len),
     g = 'x + L);
  for(i=1,len-1,     \\ root above 0 to 2 of each solvable size i=1..limit
     g *= (1 + 'x^i + 'x^(2*i) + L)^polcoeff(g,i));
  g+1;
}

\\-----------------------------------------------------------------------------
\\ Free Trees

\\ Return the number of distinct values from func(vpar) called over vpar
\\ free tree representatives.  func can return any type comparable by cmp()
\\ as per setsearch() etc.
\\
test_count_free_distinct_func(n,func) =
{
  my(l=List([]),
     vpar=vpar_free_first(n));
  until((vpar=vpar_free_next(vpar))=='none,  \\ trees
        my(val=func(vpar));
        my(pos=setsearch(l,val,1));
        if(pos, listinsert(l,val,pos)));
  #l;
}

\\ PENDING vpar_free_all
test_free_all(n) =
{
  my(ret=vector(vpar_count_free(n)));
  ret[1]=vpar_free_first(n);
  for(i=2,#ret,
     ret[i]=vpar_free_next(ret[i-1]));
  ret;
}


\\-----------------------------------------------------------------------------
\\ Free Tree Primary Iteration

test_is_preorder_depths_vector(vec,forest=0) =
{
  if(#vec,
     if(vec[1]!=0, return(0));
     forest=!forest;  \\ 1 for tree, 0 for forest
     for(v=2,#vec,
        if(!(forest <= vec[v] && vec[v] <= vec[v-1]+1), return(0))));
  1;
}

\\ "depths" is a depths vector, per vpar_is_depths_vector().
\\ Return 1 if it is depths of a "primary" free tree representative
\\ as per vpar_free_next().
\\ The code here is by splittling into trees L1 and L2.
\\
test_depths_is_free_primary_by_vectors(depths) =
{
  test_is_preorder_depths_vector(depths) || error();
  depths[1]==0 || error();
  depths[2]==1 || error();

  \\ m is vertex number of depth==1 at position>=3
  my(m);
  for(i=3,#depths, if(depths[i]==1, m=i;break()));
  if(!m, return(0));

  my(depths1=vector(m-2,i, depths[i+1]-1),   \\ L1
     depths2=concat(0,depths[m..#depths]));  \\ L2
  \\ print("depths1 "depths1" height "vecmax(depths1));
  \\ print("depths2 "depths2" height "vecmax(depths2));

  test_is_preorder_depths_vector(depths1) || error();
  test_is_preorder_depths_vector(depths2) || error();

  my(c=lex(vecmax(depths1),vecmax(depths2)));
  if(!c,
     c=lex(#depths1,#depths2);
     if(!c,
        c=lex(depths1,depths2)));
  c<=0;
}
check_equal(test_depths_is_free_primary_by_vectors([0,1,2,1,2]), 1);
check_equal(test_depths_is_free_primary_by_vectors([0,1,2,1,1]), 1);
check_equal(test_depths_is_free_primary_by_vectors([0,1,1,1,1]), 1);

\\ vpar is a free tree or forest.
\\ Return vertices [r,m,e] of the component which vpar_free_next() will act on.
\\     r = root
\\     m = first child in L2
\\     e = last vertex of the component = last in L2
\\
test_free_r_m_e(vpar) =
{
  forstep(v=#vpar,1,-1,
     if(vpar[v],
        my(e=v,  \\ last vertex in the last non-singleton component
           r=vpar_root_of_vertex(vpar,e));
        for(m=r+2,#vpar,
           if(vpar[m]==r,
              return([r,m,e])));
        return([r,0,e])));
  [if(#vpar,1,0), 0, #vpar];
}

\\ vpar is a free tree or forest.
\\ Return the vertex at the top of the L1 subtree.
\\ For a tree this is 2, for a forest it may be later.
test_free_L1_top(vpar) =
{
  my(rme=test_free_r_m_e(vpar),
     r=rme[1]);
  if(r+1<=#vpar, r+1, 0);
}

\\ vpar is a free tree or forest.
\\ Return the height of the L1 subtree.
test_free_L1_height(vpar) =
{
  my(v=test_free_L1_top(vpar));
  if(v, vpar_subtree_height(vpar,v), -1);
}

\\ vpar is a free tree or forest.
\\ Return the height of the L2 subtree.
test_free_L2_height(vpar) =
{
  my(rme=test_free_r_m_e(vpar),
     r=rme[1], m=rme[2]);
  if(m, vpar_subtree_height(vpar,m)+1,
     #vpar, 0,
     -1);
}

test_free_L1_size(vpar) =
{
  my(rme=test_free_r_m_e(vpar),
     r=rme[1], m=rme[2]);
  if(r, m-r-1, 0);
}
test_free_L2_size(vpar) =
{
  my(rme=test_free_r_m_e(vpar),
     r=rme[1], m=rme[2], e=rme[3]);
  if(r, e-m+2, 0);
}

\\ vpar is a free tree or forest.
\\ Return a new vpar tree which is sub-tree L1 extracted.
test_free_L1_tree(vpar) =
{
  my(v=test_free_L1_top(vpar));
  if(v, vpar_subtree(vpar,v), []);
}
test_free_L2_tree(vpar) =
{
  my(v=test_free_L1_top(vpar));
  if(v, vpar_delete_subtree(vpar,v), []);
}


\\ vpar is an unlabelled representative.
\\ Return 1 if it is free primary, either tree or forest.
test_unlabelled_is_free_primary(vpar) =
{
  my(vpars=vpar_component_trees(vpar));
  for(i=1,#vpars,
     vpar_preorder_depths_is_free_primary(vpar_depths_vector(vpars[i]))
       || return(0));
  1;
}

\\ vpar is an unlabelled representative.
\\ Return the next vpar which is a free forest.
test_free_next_forest_by_search(vpar) =
{
  while((vpar=vpar_unlabelled_next(vpar,1))!='none,  \\ forests
        if(test_unlabelled_is_free_primary(vpar),
           break));
  vpar;
}

\\ vpar is pre-order labelled.
\\ Perform the unlabelled "next" operation at p.
\\ This means replacing p..#vpar with copies of q=vpar[p] to p-1 inclusive.
test_unlabelled_next_at(vpar,p) =
{
  while(p>1 && !vpar[p], p--);
  if(p==1,
     'none,

     my(q=vpar[p],
        len=p-q);
     \\ print("p="p" q="q);
     until(p++>#vpar, \\ pre-increment
           \\ print(" set p="p" to "vpar[p-len] + if((p-q)%len, len));
           vpar[p] = vpar[p-len] + if((p-q)%len, len));
     vpar);
}

\\ vpar is pre-order labelled.
\\ If vpar is not already a free tree then advance to the next lex smaller
\\ which is and return that.  Or return symbol 'none if no next.
\\
\\ FIXME: Force to L2 height big enough with a path down.
\\
test_free_tree_ceil(vpar) =
{
  \\ print("test_free_tree_ceil()");
  if(vpar==[] || vpar==[0] || vpar==[0,1], return(vpar));

  while(vpar!='none,
        \\ print(" at "vpar);
        \\ print("  r,m,e "test_free_r_m_e(vpar));
        \\ print("  heights "test_free_L1_height(vpar)" "test_free_L2_height(vpar));

        my(rme=test_free_r_m_e(vpar),
           r=rme[1], m=rme[2], e=rme[3],
           c);
        (c=if(m,0,1))  \\ without m is L1 bigger
        || (c=lex(test_free_L1_height(vpar), test_free_L2_height(vpar)))
        || (c=lex(test_free_L1_size(vpar), test_free_L2_size(vpar)))
        || (c=lex(test_free_L1_tree(vpar), test_free_L2_tree(vpar)));
        \\ print("  c="c);
        if(vpar_is_tree(vpar) && c<=0, break);

        my(p=m-1);
        if(p<1,
           forstep(v=e,#vpar,-1,
              if(vpar[v] && vpar[vpar[v]],
                 \\ print("  found grandparent of v="v);
                 p=v)));
        \\ print("  next at p="p);
        if(p<1, return('none));
        vpar = test_unlabelled_next_at(vpar,p));
  vpar;
}


\\-----------------------------------------------------------------------------
\\ Ordered Trees

\\    siblingpos=0    siblingpos=1
\\            P            P
\\           /            /
\\          *            * 1 of
\\         / \          /              /
\\        *   *        *             +3
\\       /            / \            / \
\\      V            *   *         +1   +2
\\                  /              /
\\                 V              V
\\
\\ Return vpar modified so that vpar_is_isomorphic() and friends act as on
\\ an ordered original vpar.  This is done by changing each edge to the
\\ paths shown above so that each child of P has a length below which is its
\\ sibling position.  This is done above each root of vpar too, as if those
\\ roots had edge up to an imaginary 0 vertex.  Doing that orders the roots.
\\ The degree-2 branch in the inserted path distinguishes the path end when
\\ v has a single child, which will be another path down.
\\
\\ Total new vertices added is 3*#vpar + vecsum(vpar_siblingpos_vector(vpar)).
\\
test_ordered_to_unlabelled(vpar) =
{
  my(siblingpos=vpar_siblingpos_vector(vpar),
     new=List(vpar));

  for(v=1,#vpar,
     my(p=vpar[v]);
     listput(new,#new+3);   \\ +1 parent +3
     new[v]=#new;
     for(i=0,siblingpos[v],    \\ +2 parent +3 etc
        listput(new,#new+2));
     listput(new,p));
  Vec(new);
}

check_equal(test_ordered_to_unlabelled([]), [], \
            "test_ordered_to_unlabelled() empty");
{
  \\     4
  \\    / \
  \\   2   3
  \\  /
  \\ 1
  my(want=[2,4,4,0]);
  vpar_validate(want);
  check_equal(test_ordered_to_unlabelled([0]), want);
}
{
  \\
  \\     5      9
  \\    / \     |
  \\   3   4    8
  \\  /         | \
  \\ 1          6  7
  \\            |
  \\            2
  my(want=[3,6, 5,5,0, 8,8,9,0]);
  vpar_validate(want);
  check_equal(test_ordered_to_unlabelled([0,0]), want,
              "test_ordered_to_unlabelled() two");
}

\\-----------------------------------------------------------------------------
\\ Generating Functions

\\ "func" is a function to be called value=func(i) for integer i>=0.
\\ Return a polynomial sum func(i)*x^i of n terms i=0 to n-1 and O(x^n).
\\
test_func_to_gf(func,limit) = \
  Polrev(vector(limit,i,i--; func(i))) + O(x^limit);

\\ "func" is a function to be called value=func(i) for integer i>=0.
\\ Return a polynomial sum func(i)*x^i/i! of n terms i=0 to n-1 and O(x^n).
\\
test_func_to_egf(func,limit) = \
  Polrev(vector(limit,i,i--; func(i)/i!)) + O(x^limit);

\\ vec is a vector of terms.  Return an t_SER egf of those,
\\ vec[1]*x/1! + vec[2]*x^2/2! + vec[3]/1!*x^3/3! + ... + O(x^(#vec+1)).
\\ "offset" is the power x^offset for first term vec[1], default 1.
\\ "limit" is the O(x^limit) of the series.
\\ Default limit is all terms of vec[] (and respecting the given "offset").
\\
test_vector_to_egf(vec,offset=1,limit=#vec+offset) = \
  sum(i=1,min(#vec,limit-offset), my(n=i-1+offset); vec[i]*x^n/n!) + O(x^limit);

\\ g is a polynomial generating function, either a plain polynomial or
\\ t_RFRAC, representing an ordinary generating function.
\\ Return coefficients of its first n terms x^0 .. x^(n-1).
test_gf_terms(g,n) =
{
  my(x = variable(g),
     zeros = min(n,valuation(g,x)),
     v = Vec(g + O(x^n)));
  if(zeros>=0, concat(vector(zeros,i,0), v),
               v[-zeros+1 .. #v]);
}
test_gf_term(g,n) = test_gf_terms(g,n+1)[n+1];

\\ g is a polynomial generating function, either a plain polynomial or
\\ t_RFRAC, representing an exponential generating function.
\\ Return coefficients of its first n terms x^0 .. x^(n-1), each multiplied
\\ up i! so as to get the original sequence terms.
test_egf_terms(g,n) = my(v=test_gf_terms(g,n)); for(i=1,#v, v[i]*=(i-1)!); v;
test_egf_term(g,n) = test_egf_terms(g,n+1)[n+1];

\\ egf of sequence f(n) = 1, is exp(x)
test_egf1s(limit) = test_func_to_egf(n->1, limit);

\\ egf of sequence f(n) = n, is x*exp(x)
test_egfN(limit) = test_func_to_egf(n->n, limit);

\\ Lambert W function, being egf of (-n)^(n-1)
test_LambertW(limit) = test_func_to_egf(n-> if(n==0,0, (-n)^(n-1) ), limit);


\\-----------------------------------------------------------------------------
\\ Weigh Transform

test_weigh_transform(a) =
{
  my(L=O(x^(#a+1)),
     b = prod(i=1,#a, (1+x^i + L)^a[i]));
  Vec((b-1)/x);
}


\\-----------------------------------------------------------------------------
\\ Automorphism Stable

\\ Return 1 if vpar is rooted semi-stable at v.
\\ This means the rooted automorphisms of vpar with v deleted are equal to
\\ the rooted automorphisms of vpar which have v fixed.  So deleting v does
\\ not introduce any new symmetry.
\\ All rooted trees are semi-stable at some vertex.  The last vertex of
\\ vpar_seq_premax() is semi-stable, and possibly others.
\\
\\ On deleting a vertex v with children, the subtrees below it become new
\\ component trees.  They are reckoned as new components rooted at the
\\ children of v.  If isomorphic to the rest of the tree above, or another
\\ forest component, then v is not semi-stable.
\\
test_is_rooted_semistable_at_vertex(vpar,v) =
{
  my(perms=vpar_automorphism_generators(vpar_delete_vertex(vpar,v)));
  for(i=1,#perms,
     if(! vpar_is_automorphism(vpar, test_perm_insert_n(perms[i],v)),
        return(0)));
  1;
}
test_is_rooted_semistable_at_vertex_by_automorphisms(vpar,v) =
{
  \\ print("test_is_rooted_semistable_at_vertex_by_automorphisms() "vpar" at v="v);
  my(perms=vpar_automorphisms(vpar));
  \\ print(" all perms "perms);

  perms=test_perms_fixing_n(perms,v);
  \\ print(" fixing v "perms);

  perms=test_perms_delete_n(perms,v);
  perms=vecsort(perms);
  \\ print(" deleted v "perms);

  \\ print(" cf "vecsort(vpar_free_automorphisms(vpar_delete_vertex(vpar,v))));
  perms == vecsort(vpar_automorphisms(vpar_delete_vertex(vpar,v)));
}

\\ Return 1 if vpar is free semi-stable at v.
\\ This means the free automorphisms of vpar with v deleted are equal to the
\\ free automorphisms of vpar which have v fixed.  So deleting v does not
\\ introduce any new symmetry.
\\
test_is_free_semistable_at_vertex(vpar,v) =
{
  my(perms=vpar_free_automorphism_generators(vpar_delete_vertex(vpar,v)));
  for(i=1,#perms,
     if(! vpar_is_free_automorphism(vpar, test_perm_insert_n(perms[i],v)),
        return(0)));
  1;
}
test_is_free_semistable_at_vertex_by_automorphisms(vpar,v) =
{
  \\ print("test_is_free_semistable_at_vertex_by_automorphisms() "vpar" at v="v);
  my(perms=vpar_free_automorphisms(vpar));
  \\ print(" all perms "perms);

  perms=test_perms_fixing_n(perms,v);
  \\ print(" fixing v "perms);

  perms=test_perms_delete_n(perms,v);
  perms=vecsort(perms);
  \\ print(" deleted v "perms);

  \\ print(" cf "vecsort(vpar_free_automorphisms(vpar_delete_vertex(vpar,v))));
  perms == vecsort(vpar_free_automorphisms(vpar_delete_vertex(vpar,v)));
}

\\ Return a vector of the free semistable vertices of vpar.
test_free_semistable_vertices(vpar) = \
  test_select_vertices(vpar,test_is_free_semistable_at_vertex);

\\ seq = test_max_free_stablizing_seq(vpar)
\\ Return a longest stablizing sequence of vpar as a Vecsmall.
\\ Successively removing vertices in seq does not introduce any new
\\ automorphisms.
\\ If vpar is semistable then it has a sequence of length at least 1,
\\ if not then seq will be empty.
\\
\\ This code gives the lexicographically smallest (take the smallest vertex
\\ first) among those of maximum length, but don't rely on that.
\\
test_max_free_stablizing_seq(vpar) =
{
  my(m=Vecsmall([]));
  for(v=1,#vpar,
     if(test_is_free_semistable_at_vertex(vpar,v),
        my(seq=test_max_free_stablizing_seq
               (vpar_delete_vertex(vpar,v)));
        if(#seq >= #m,
           for(i=1,#seq, if(seq[i]>=v, seq[i]++));
           m=concat(Vecsmall([v]), seq);
           if(#m==#vpar, break))));
  m;
}

\\ seq = test_seq_free_stabilizing_by_search(vpar)
\\ Return a stablizing sequence of vpar as a Vecsmall, or symbol 'none if
\\ vpar is not stable.
\\ Successively removing vertices in sequence seq does not introduce any new
\\ automorphisms.
\\ This code gives the lexicographically smallest stabilizing sequence (take
\\ the smallest possible vertex first), but don't rely on that.
\\
test_seq_free_stabilizing_by_search(vpar) =
{
  my(seq=test_max_free_stablizing_seq(vpar));
  if(#seq==#vpar, seq, 'none);
}

\\ len = test_free_stability_index_by_search(vpar)
\\ Return the stability index of vpar.
\\ This is the length of the longest stabilizing sequence in vpar
\\ (length of test_max_free_stablizing_seq()).
\\ If vpar is not semistable then it doesn't even have a single vertex
\\ removable and the return is 0.
\\
test_free_stability_index_by_search(vpar) = \
  #test_max_free_stablizing_seq(vpar);


\\ Return 1 if vpar has prev(vpar,seq[i]) true for successive seq[i]
\\ deletes.  So pred(vpar,seq[1]) true, then delete seq[1] and have
\\ pred(vpar_del,seq[2]) true, etc.  The given seq should be vertex number
\\ in the given vpar.  The calculation renumbers down as necessary when
\\ vertices are deleted.
\\
test_seq_is_pred_deletes(vpar,seq,pred) =
{
  for(i=1,#seq,
     my(v=seq[i]);
     pred(vpar,v) || return(0);
     vpar=vpar_delete_vertex(vpar,v);
     for(j=i+1,#seq, if(seq[j]>v, seq[j]--)));
  1;
}

\\ Return 1 if seq is a rooted stablizing sequence in vpar.
\\ This means vpar is rooted semi-stable at seq[1], and on removing that
\\ vertex seq[2] is rooted semi-stable in the remainder, etc.
test_seq_is_rooted_stablizing(vpar,seq) =
{
  test_seq_is_pred_deletes(vpar,seq, test_is_rooted_semistable_at_vertex);
}

\\ Return 1 if seq is a free stablizing sequence in vpar.
\\ This means vpar is free semi-stable at seq[1], and on removing that
\\ vertex seq[2] is free semi-stable in the remainder, etc.
test_seq_is_free_stablizing(vpar,seq) =
{
  test_seq_is_pred_deletes(vpar,seq, test_is_free_semistable_at_vertex);
}

\\ Return vpar with all roots linked together as a cycle.
\\ A root gets parent as the next smaller root (or first root get last as
\\ parent).  This is the same order as test_make_cycle().
test_cyclic_roots(vpar) =
{
  \\ print("  roots "roots);
  my(roots=vpar_roots(vpar),
     prev=roots[1]);
  forstep(i=#roots,1,-1,
     vpar[prev]=roots[i]; prev=roots[i]);
  \\ print("  return "vpar);
  vpar;
}


\\-----------------------------------------------------------------------------
\\ Dreadnaut
\\
\\ Used for comparing vpar_automorphisms() against what nauty finds, but
\\ otherwise not sure how much use.

test_to_dreadnaut(vpar,down=0) =
{
  \\ print("test_to_dreadnaut() "vpar);

  \\ "zero" secret idea for an option vertex numbers starting 0
  my(l=List([]),
     prev=1,
     zero=0);
  listput(l, Str("n="#vpar" $="1-zero" g"));    \\ start

  \\ neighbours[v] = vector of neighbour vertex numbers, being parent or
  \\ children or both according to "down"
  my(neighbours=if(!down, apply(p->if(p,[p],[]), vpar),   \\ parent or empty
                   down==1, vpar_children_vector(vpar),
                   vpar_neighbours_vector(vpar)));
  for(v=1,#vpar,
     if(#neighbours[v],
        if(v==prev+1, listput(l, "; "),                \\ increment
           v!=prev,   listput(l, Str(" "v-zero":")));  \\ new "from"
        my(join="");
        for(c=1,#neighbours[v],
           listput(l, Str(join,neighbours[v][c]-zero));
           join=",");
        prev=v));

  listput(l, ".");               \\ end
  \\ print(l);
  concat(Vec(l));
}
{
addhelp(test_to_dreadnaut,
"str = test_to_dreadnaut(vpar,down=0)
Return a dreadnaut command string creating vpar.  Dreadnaut is the
command-line interface to Nauty and Traces, http://cs.anu.edu.au/~bdm/nauty

str is for example \"n=4 $=1 g 3; 3 4:1.\" which sets n vertices, $=1 first
vertex number 1, then edges child -> parent using various shorthands.
It is one line and no final newline so can be output with print(str) etc.
The exact form of syntax and shorthands is unspecified.

Optional parameter \"down\" can be 1 to give edges down parent -> child, or
can be 2 for edges both up and down.  If dreadnaut is in its default
undirected mode then up, down, or both, are all the same.

vpar can contain cycles.  They become cycles in dreadnaut, including
self-loops vertex to itself.  Dreadnaut circa version 2.7 only accepts
self-loops in its directed mode.  Using down=2 for edges both ways has the
effect of undirected graph in directed mode.

See test_dreadnaut_roots() to distinguish roots.");
}

test_dreadnaut_roots(vpar) =
{
  \\ ret[] is a vector of strings built.
  \\ Each string is a single root like "1" or a range like "1:4".
  \\ The length created is exact.  Each v which is a root preceded by a
  \\ non-root, or the start of vpar, is a string in ret.
  \\
  my(num_roots=vpar_num_roots(vpar),
     ret=vector(sum(v=1,#vpar, !vpar[v] && (v==1 || vpar[v-1])) + 2),
     upto=1,
     sep="");
  ret[1]="f=[";
  for(v=1,#vpar,
     if(!vpar[v],  \\ a root
        my(b=v);
        while(v+1<=#vpar && !vpar[v+1], v++);  \\ consecutive roots
        ret[upto++] = Str(sep, b, if(v==b,"", Str(":",v)));
        sep=" "));
  ret[#ret] = "]";
  concat(ret);
}
{
addhelp(test_dreadnaut_roots,
"str = test_dreadnaut_roots(vpar)
Return a dreadnaut \"f\" command string which is an initial partition
distinguishing the roots of vpar.  Dreadnaut is the command-line interface
to Nauty and Traces, http://cs.anu.edu.au/~bdm/nauty

str is for example \"f=[1 4:6]\".  It is one line and no final newline so
can be output with print(str) etc.  Ranges are used for compact
representation but the exact form is unspecified.

In dreadnaut's default undirected mode, roots distinguished from other
vertices has the effect of treating vpar as rooted for finding
automorphisms.  Nauty circa its version 2.7 is usually faster running
undirected with distinguished roots than it is running fully directed.");
}


\\-----------------------------------------------------------------------------
\\ Bulgarian Solitaire

\\ p is a partition as from partitions() or forpart(), ie. a Vecsmall of
\\ integers sorted in ascending order.
\\ Return a new partition which is after the Bulgarian solitaire step.
\\ The step is to subtract 1 from each element, sum them as a new term,
\\ and discard any 0s.
\\ The return is a Vecsmall sorted in ascending order as usual for partitions.
test_Bulgarian_solitaire_step(p) =
{
  for(i=1,#p, p[i]--);
  p=Vec(p);
  Vecsmall(vecsort(select(x->x!=0, concat(p,[#p]))));
}

\\ Return a vpar which is the Bulgarian solitaire digraph, or tree.
\\ Each vertex v represents a partition.
\\ Partitions are numbered in order of their return from partitions(), which
\\ is the order forpart() generates them, and is by increasing length then
\\ lexicographically among equal lengths.
\\ A step which goes to the same partition is reckoned a root so its vpar[v]=0.
\\ When n is triangular the vpar is a tree (one root).
\\ When n is not triangular it has a cycle or cycles.
\\ Optional parameter "loops" can be 1 to have everywhere a parent, so that
\\ the fixed point v->v vertices are self-loops instead of roots.
test_make_Bulgarian_solitaire(n,loops=0) =
{
  my(parts=partitions(n));
  parts=Set(parts);  \\ actually partitions() already returns a Set()
  vector(#parts,v,
         my(parent=test_Bulgarian_solitaire_step(parts[v]));
         if(loops || parent!=parts[v],
            setsearch(parts,parent))); \\ with loops or not a self-step, lookup
}


\\-----------------------------------------------------------------------------
\\ Miscellaneous

\\ A098725
test_A098725(n) =
{
  n>=0 || error("test_A098725() is for n>=0");  \\ OFFSET=0
  valuation(3*n-2,2)%2==0;
}

\\ A096268 period doubling, parity count low 0-bits
test_A096268(n) =
{
  n>=0 || error("test_A096268() is for n>=0");  \\ OFFSET=0
  valuation(n+1,2) % 2;
}

\\ A035263 period doubling opposite
test_A035263(n) =
{
  n>=1 || error("test_A035263() is for n>=1");  \\ OFFSET=1
  valuation(n,2)%2==0;
}

\\ A065883 remove factors of 4
test_A065883(n) = n>>(2*valuation(max(n,1),4));

\\ A134667 periodic 0, 1, 0, 0, 0, -1,  OFFSET=0
test_A134667(n) = [0,1,0, 0,0,-1][n%6+1];

\\ A084091 gf = sum(k>=0, x^2^k/(1 + x^2^k + x^2^(k+1))) giving +1,0,-1 terms
\\ +1 at (6m+1)*4^j
\\ -1 at (6m+5)*4^j
\\ everywhere else 0, which is odd CountLowZeros, and n==0 mod 3.
test_A084091(n) = if(n&&valuation(n,2)%2,0, (n%=3)==2,-1, n);

\\ Return the highest 1-bit in n only (A053644).
test_highbit(n) =
{
  n>=0 || error("test_highbit() is for n>=0");
  if(n==0,0, 1<<logint(n,2));
}

\\ Return the length of n in bits.
test_bitlength(n) = if(n==0,0, logint(n,2)+1);

\\ Jorg Arndt's C code in A108918 (and fxtbook)
test_negidx2lexrev(k) =
{
  k>=1 || error("test_negidx2lexrev() is for k>=1");
  my(z=0, h=test_highbit(k));
  while(k,
        while(bitand(h,k)==0, h >>= 1); \\ find 1-bit
        z = bitxor(z, h);               \\ set in return
        k++;
        k = bitand(k, h-1));            \\ mask down to notice end k=0
  z;
}
test_A108918(n) = test_negidx2lexrev(n);


\\-----------------------------------------------------------------------------

\\ Return a vector of len entries which is func(row,col) called for
\\ anti-diagonals in OEIS style.
\\
\\          col=1 col=2 col=3
\\   row=1:   1     3     6
\\   row=2:   2     5
\\   row=3:   4
\\
\\ Optional row_start and col_start can be different starts for those.
\\ Optional "down" can be 1 to go by downwards diagonals instead
\\ (which is rows and columns same interpretation, just down instead).
\\
test_antidiagonals_of_func(len,func, row_start=1,col_start=1,down=0) =
{
  my(v=vector(len),upto=0);
  for(d=0,len,
     for(o=0,d,
        if(upto>=#v,break(2));
        my(row=d-o, col=o);
        if(down,[row,col]=[col,row]);
        v[upto++]=func(row+row_start,col+col_start)));
  v;  
}

\\ A00709 prime'th recurrence prime(prime(...prime(1))7)
test_A007097_primeth(n) =
{
  n>=0 || error();
  my(ret=1); for(i=1,n,ret=prime(ret)); ret;
}

\\-----------------------------------------------------------------------------
